using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Security.Cryptography;
using Microsoft.Win32;

namespace org {

    public static class VocaloSysUtil {
        private static bool s_initialized = false;

        private static string s_dll_path2 = "";
        private static string s_editor_path2 = "";
        private static string s_voicedbdir2 = "";
        private static List<SingerConfig> s_installed_singers2 = new List<SingerConfig>();
        private static List<SingerConfig> s_singer_configs2 = new List<SingerConfig>();

        private static string s_dll_path1 = "";
        private static string s_editor_path1 = "";
        private static string s_voicedbdir1 = "";
        private static List<SingerConfig> s_installed_singers1 = new List<SingerConfig>();
        private static List<SingerConfig> s_singer_configs1 = new List<SingerConfig>();

        private const int MAX_SINGERS = 0x4000;

        public static void init() {
            init_vocalo2();
            init_vocalo1();
        }

        /// <summary>
        /// VOCALOID1システムのプロパティを取得
        /// </summary>
        private static void init_vocalo1() {
            // vocaloid1 dll path
            RegistryKey v1application = null;
            v1application = Registry.LocalMachine.OpenSubKey( "SOFTWARE\\VOCALOID\\APPLICATION", false );
            if ( v1application != null ) {
                string[] keys = v1application.GetSubKeyNames();
                for ( int i = 0; i < keys.Length; i++ ) {
                    RegistryKey key = v1application.OpenSubKey( keys[i], false );
                    if ( key != null ) {
                        string name = (string)key.GetValue( "PATH" );
                        if ( name.ToLower().EndsWith( "\\vocaloid.dll" ) ) {
                            s_dll_path1 = name;
                        } else if ( name.ToLower().EndsWith( "\\vocaloid.exe" ) ) {
                            s_editor_path1 = name;
                        }
                        key.Close();
                    }
                }
                v1application.Close();
            }

            // voicedbdir for vocaloid1
            RegistryKey v1database = Registry.LocalMachine.OpenSubKey( "SOFTWARE\\VOCALOID\\DATABASE\\VOICE", false );
            if ( v1database != null ) {
                s_voicedbdir1 = (string)v1database.GetValue( "VOICEDIR", "" );
#if DEBUG
                Console.WriteLine( "s_voicedbdir1=" + s_voicedbdir1 );
#endif
                // インストールされている歌手のVOICEIDSTRを列挙
                string[] singer_voiceidstrs = v1database.GetSubKeyNames();
                List<string> vvoice_keys = new List<string>();
                List<SingerConfig> vvoice_values = new List<SingerConfig>();
                foreach ( string voiceidstr in singer_voiceidstrs ) {
                    RegistryKey singer = v1database.OpenSubKey( voiceidstr );
                    if ( singer == null ) {
                        continue;
                    }
                    RegistryKey vvoice = singer.OpenSubKey( "vvoice" );
                    if ( vvoice != null ) {
                        string[] vvoices = vvoice.GetValueNames();

                        // インストールされた歌手の.vvdを読みにいく
                        // installdir以下の、拡張子.vvdのファイルを探す
                        foreach ( string file in Directory.GetFiles( Path.Combine( s_voicedbdir1, voiceidstr ), "*.vvd" ) ) {
                            SingerConfig config = SingerConfig.readSingerConfig( file, 0 ); //とりあえずプログラムチェンジは0
                            s_installed_singers1.Add( config );
                        }

                        // vvoice*.vvdを読みにいく。
                        foreach ( string s in vvoices ) {
#if DEBUG
                            Console.WriteLine( "s=" + s );
#endif
                            string file = Path.Combine( s_voicedbdir1, s + ".vvd" );
                            if ( File.Exists( file ) ) {
                                SingerConfig config = SingerConfig.readSingerConfig( file, 0 );
                                vvoice_keys.Add( s );
                                vvoice_values.Add( config );
                            }
                        }
                    }
                    singer.Close();
                }

                // voice.mapを読み込んで、s_singer_configs1のプログラムチェンジを更新する
                string map = Path.Combine( s_voicedbdir1, "voice.map" );
                if ( File.Exists( map ) ) {
                    using ( FileStream fs = new FileStream( map, FileMode.Open, FileAccess.Read ) ) {
                        byte[] dat = new byte[8];
                        fs.Seek( 0x20, SeekOrigin.Begin );
                        for ( int i = 0; i < MAX_SINGERS; i++ ) {
                            fs.Read( dat, 0, 8 );
                            ulong value = makelong_le( dat );
                            if ( value >= 1 ) {
#if DEBUG
                                Console.WriteLine( "value=" + value );
#endif
                                for ( int j = 0; j < vvoice_keys.Count; j++ ) {
                                    if ( vvoice_keys[j] == "vvoice" + value ) {
                                        vvoice_values[j].Program = i;
                                    }
                                }
                            }
                        }
                    }
                }

                // s_installed_singers1のSingerConfigのProgramとOriginalを適当に頒番する
                for ( int i = 0; i < s_installed_singers1.Count; i++ ) {
                    s_installed_singers1[i].Program = i;
                    s_installed_singers1[i].Original = i;
                }

                // s_singer_configs1を更新
                for ( int i = 0; i < vvoice_values.Count; i++ ) {
                    for ( int j = 0; j < s_installed_singers1.Count; j++ ) {
                        if ( vvoice_values[i].VOICEIDSTR == s_installed_singers1[j].VOICEIDSTR ) {
                            vvoice_values[i].Original = s_installed_singers1[j].Program;
                            break;
                        }
                    }
                    s_singer_configs1.Add( vvoice_values[i] );
                }
                v1database.Close();
            }
#if DEBUG
            Console.WriteLine( "installed" );
            foreach ( SingerConfig sc in s_installed_singers1 ) {
                Console.WriteLine( "VOICENAME=" + sc.VOICENAME + "; VOICEIDSTR=" + sc.VOICEIDSTR + "; Program=" + sc.Program + "; Original=" + sc.Original );
            }
            Console.WriteLine( "singer configs" );
            foreach ( SingerConfig sc in s_singer_configs1 ) {
                Console.WriteLine( "VOICENAME=" + sc.VOICENAME + "; VOICEIDSTR=" + sc.VOICEIDSTR + "; Program=" + sc.Program + "; Original=" + sc.Original );
            }
#endif
        }

        /// <summary>
        /// VOCALOID2システムのプロパティを取得
        /// </summary>
        private static void init_vocalo2() {
            // 最初はvstiとeditorのパスを取得
            RegistryKey v2application = Registry.LocalMachine.OpenSubKey( "SOFTWARE\\VOCALOID2\\APPLICATION", false );
            if ( v2application == null ) {
                v2application = Registry.LocalMachine.OpenSubKey( "SOFTWARE\\VOCALOID2_DEMO\\APPLICATION", false );
            }
            if ( v2application != null ) {
                string[] keys = v2application.GetSubKeyNames();
                for ( int i = 0; i < keys.Length; i++ ) {
                    RegistryKey key = v2application.OpenSubKey( keys[i], false );
                    if ( key != null ) {
                        string name = (string)key.GetValue( "PATH" );
                        if ( name.ToLower().EndsWith( "\\vocaloid2.dll" ) ) {
                            s_dll_path2 = name;
                        } else if ( name.ToLower().EndsWith( "\\vocaloid2_demo.dll" ) ) {
                            s_dll_path2 = name;
                        } else if ( name.ToLower().EndsWith( "\\vocaloid2.exe" ) ) {
                            s_editor_path2 = name;
                        }
                        key.Close();
                    }
                }
                v2application.Close();
            }

            // 歌声データベースを取得
            RegistryKey v2database = Registry.LocalMachine.OpenSubKey( "SOFTWARE\\VOCALOID2\\DATABASE\\VOICE", false );
            if ( v2database != null ) {
                // データベース（というよりもvoice.map）が保存されているパスを取得
                s_voicedbdir2 = (string)v2database.GetValue( "VOICEDIR", "" );
                // インストールされている歌手のVOICEIDSTRを列挙
                string[] singer_voiceidstrs = v2database.GetSubKeyNames();
                List<string> vvoice_keys = new List<string>();
                List<SingerConfig> vvoice_values = new List<SingerConfig>();
                foreach ( string voiceidstr in singer_voiceidstrs ) {
                    RegistryKey singer = v2database.OpenSubKey( voiceidstr );
                    if ( singer == null ) {
                        continue;
                    }
                    string installdir = (string)singer.GetValue( "INSTALLDIR", "" );
#if DEBUG
                    Console.WriteLine( "installdir=" + installdir );
#endif
                    RegistryKey vvoice = singer.OpenSubKey( "vvoice" );
                    if ( vvoice != null ) {
                        string[] vvoices = vvoice.GetValueNames();

                        // インストールされた歌手の.vvdを読みにいく
                        // installdir以下の、拡張子.vvdのファイルを探す
                        foreach ( string file in Directory.GetFiles( Path.Combine( installdir, voiceidstr ), "*.vvd" ) ) {
                            SingerConfig config = SingerConfig.readSingerConfig( file, 0 ); //とりあえずプログラムチェンジは0
                            s_installed_singers2.Add( config );
                        }

                        // vvoice*.vvdを読みにいく。場所は、installdirではなく、s_voicedbdir2
                        foreach ( string s in vvoices ) {
                            string file = Path.Combine( s_voicedbdir2, s + ".vvd" );
                            if ( File.Exists( file ) ) {
                                SingerConfig config = SingerConfig.readSingerConfig( file, 0 );
                                vvoice_keys.Add( s );
                                vvoice_values.Add( config );
                            }
                        }
                    }
                    singer.Close();
                }

                // voice.mapを読み込んで、s_singer_configs2のプログラムチェンジを更新する
                string map = Path.Combine( s_voicedbdir2, "voice.map" );
                if ( File.Exists( map ) ) {
                    using ( FileStream fs = new FileStream( map, FileMode.Open, FileAccess.Read ) ) {
                        byte[] dat = new byte[8];
                        fs.Seek( 0x20, SeekOrigin.Begin );
                        for ( int i = 0; i < MAX_SINGERS; i++ ) {
                            fs.Read( dat, 0, 8 );
                            ulong value = makelong_le( dat );
                            if ( value >= 1 ) {
#if DEBUG
                                Console.WriteLine( "value=" + value );
#endif
                                for ( int j = 0; j < vvoice_keys.Count; j++ ) {
                                    if ( vvoice_keys[j] == "vvoice" + value ) {
                                        vvoice_values[j].Program = i;
                                    }
                                }
                            }
                        }
                    }
                }

                // s_installed_singers2のSingerConfigのProgramとOriginalを適当に頒番する
                for ( int i = 0; i < s_installed_singers2.Count; i++ ) {
                    s_installed_singers2[i].Program = i;
                    s_installed_singers2[i].Original = i;
                }

                // s_singer_configs2を更新
                for ( int i = 0; i < vvoice_values.Count; i++ ) {
                    for ( int j = 0; j < s_installed_singers2.Count; j++ ) {
                        if ( vvoice_values[i].VOICEIDSTR == s_installed_singers2[j].VOICEIDSTR ) {
                            vvoice_values[i].Original = s_installed_singers2[j].Program;
                            break;
                        }
                    }
                    s_singer_configs2.Add( vvoice_values[i] );
                }
                v2database.Close();
            }
#if DEBUG
            Console.WriteLine( "installed" );
            foreach ( SingerConfig sc in s_installed_singers2 ) {
                Console.WriteLine( "VOICENAME=" + sc.VOICENAME + "; VOICEIDSTR=" + sc.VOICEIDSTR + "; Program=" + sc.Program + "; Original=" + sc.Original );
            }
            Console.WriteLine( "singer configs" );
            foreach ( SingerConfig sc in s_singer_configs2 ) {
                Console.WriteLine( "VOICENAME=" + sc.VOICENAME + "; VOICEIDSTR=" + sc.VOICEIDSTR + "; Program=" + sc.Program + "; Original=" + sc.Original );
            }
#endif
        }

        /// <summary>
        /// Transform the byte array(length=8) to unsigned long, assuming that the byte array is little endian.
        /// </summary>
        /// <param name="oct"></param>
        /// <returns></returns>
        private static ulong makelong_le( byte[] oct ) {
            return (ulong)oct[7] << 56 | (ulong)oct[6] << 48 | (ulong)oct[5] << 40 | (ulong)oct[4] << 32 | (ulong)oct[3] << 24 | (ulong)oct[2] << 16 | (ulong)oct[1] << 8 | (ulong)oct[0];
        }
    }


    class DbgUtl3 {
        private static List<string> s_cache = new List<string>();
        //private static string s_voicedbdir = "";
        private static List<SingerConfig> m_singer_configs = new List<SingerConfig>();
        private static List<string> s_registry = new List<string>();

        static void Main( string[] args ) {
            VocaloSysUtil.init();
            using ( StreamWriter log = new StreamWriter( "log.txt" ) ) {
                log.WriteLine( typeof( DbgUtl3 ).ToString() );
                try {
                    // vocaloid2 dll path
                    Microsoft.Win32.RegistryKey regkey_dll = null;
                    regkey_dll = Microsoft.Win32.Registry.LocalMachine.OpenSubKey( "SOFTWARE\\VOCALOID2\\APPLICATION", false );
                    if ( regkey_dll == null ) {
                        regkey_dll = Microsoft.Win32.Registry.LocalMachine.OpenSubKey( "SOFTWARE\\VOCALOID2_DEMO\\APPLICATION", false );
                    }
                    if ( regkey_dll != null ) {
                        string[] keys = regkey_dll.GetSubKeyNames();
                        for ( int i = 0; i < keys.Length; i++ ) {
                            Microsoft.Win32.RegistryKey key = regkey_dll.OpenSubKey( keys[i], false );
                            if ( key != null ) {
                                string name = (string)key.GetValue( "PATH" );
                                if ( name.ToLower().EndsWith( "\\vocaloid2.dll" ) ) {
                                    //s_dll_path2 = name;
                                } else if ( name.ToLower().EndsWith( "\\vocaloid2_demo.dll" ) ) {
                                    //s_dll_path2 = name;
                                } else if ( name.ToLower().EndsWith( "\\vocaloid2.exe" ) ) {
                                    //s_editor_path2 = name;
                                }
                                key.Close();
                            }
                        }
                        regkey_dll.Close();
                    }

                    // voicedbdir for vocaloid2
                    Microsoft.Win32.RegistryKey regkey_voicedb2 = Microsoft.Win32.Registry.LocalMachine.OpenSubKey( "SOFTWARE\\VOCALOID2\\DATABASE\\VOICE", false );
                    if ( regkey_voicedb2 == null ) {
                        regkey_voicedb2 = Microsoft.Win32.Registry.LocalMachine.OpenSubKey( "SOFTWARE\\VOCALOID2_DEMO\\DATABASE\\VOICE", false );
                    }
                    if ( regkey_voicedb2 != null ) {
                        string[] keys = regkey_voicedb2.GetSubKeyNames();
                        for ( int i = 0; i < keys.Length; i++ ) {
                            Microsoft.Win32.RegistryKey key = regkey_voicedb2.OpenSubKey( keys[i], false );
                            if ( key != null ) {
                                log.WriteLine( "******************************************************************************" );
                                log.WriteLine( "# Registry #" );
                                string s_voicedbdir = (string)key.GetValue( "INSTALLDIR" );
                                log.WriteLine( "NAME=" + (string)key.GetValue( "NAME" ) );
                                log.WriteLine( "INSTALLDIR=" + s_voicedbdir );
                                string voicedbdir = Path.GetFileName( key.Name );
                                log.WriteLine( "ID=" + getmd5( voicedbdir ) );

                                log.WriteLine( "------------------------------------------------------------------------------" );
                                log.WriteLine( "# vvoice #" );
                                List<SingerConfig> singer_configs = new List<SingerConfig>();
                                Microsoft.Win32.RegistryKey key2 = key.OpenSubKey( "vvoice", false );
                                List<string> vvoices = new List<string>( key2.GetValueNames() );
                                foreach ( string s in vvoices ) {
                                    log.WriteLine( s );
                                    string[] vvd = Directory.GetFiles( s_voicedbdir, "*" + s + "*" );
                                    foreach ( string s2 in vvd ) {
                                        log.WriteLine( "    " + Path.GetFileName( s2 ) );
                                    }
                                    string file = Path.Combine( s_voicedbdir, s + ".vvd" );
                                    //log.WriteLine( "    File.Exists(\"" + file + "\")=" + File.Exists( file ) );
                                    if ( File.Exists( file ) ) {
                                        string ret = "";
                                        SingerConfig sc = SingerConfig.readSingerConfig( file, 0, ref ret );
                                        singer_configs.Add( sc );
                                        log.WriteLine( "    " + ret );
                                    }
                                }

                                log.WriteLine( "------------------------------------------------------------------------------" );
                                log.WriteLine( "# Sub Directories in INSTALLDIR #" );
                                foreach ( string di in Directory.GetDirectories( s_voicedbdir ) ) {
                                    log.WriteLine( getmd5( Path.GetFileName( di ) ) );
                                }
                                //log.WriteLine( "File.Exists(\"voice.map\")=" + File.Exists( Path.Combine( s_voicedbdir, "voice.map" ) ) );
                                
                                log.WriteLine( "------------------------------------------------------------------------------" );
                                log.WriteLine( "# *.map file in INSTALLDIR #" );
                                string[] map = Directory.GetFiles( s_voicedbdir, "*.map" );
                                foreach ( string s in map ) {
                                    log.WriteLine( Path.GetFileName( s ) );
                                }

                                log.WriteLine( "------------------------------------------------------------------------------" );
                                log.WriteLine( "# *.vvd file in INSTALLDIR #" );
                                string[] vvds = Directory.GetFiles( s_voicedbdir, "*.vvd" );
                                foreach ( string s in vvds ) {
                                    log.WriteLine( Path.GetFileName( s ) );
                                }

                                string subdir = Path.Combine( s_voicedbdir, voicedbdir );
                                log.WriteLine( "------------------------------------------------------------------------------" );
                                log.WriteLine( "# *.map file in INSTALLDIR\\" + getmd5( voicedbdir ) + " #" );
                                map = Directory.GetFiles( subdir, "*.map" );
                                foreach ( string s in map ) {
                                    log.WriteLine( Path.GetFileName( s ) );
                                }

                                log.WriteLine( "------------------------------------------------------------------------------" );
                                log.WriteLine( "# *.vvd file in INSTALLDIR\\" + getmd5( voicedbdir ) + " #" );
                                vvds = Directory.GetFiles( subdir, "*.vvd" );
                                foreach ( string s in vvds ) {
                                    log.WriteLine( Path.GetFileName( s ) );
                                }


                                log.WriteLine( "------------------------------------------------------------------------------" );
                                log.WriteLine( "# Singers #" );
                                foreach ( SingerConfig sc in singer_configs ) {
                                    log.WriteLine( "VOICENAME=" + sc.VOICENAME );
                                    log.WriteLine( "VOICEIDSTR=" + getmd5( sc.VOICEIDSTR ) );
                                }

                                log.WriteLine( "------------------------------------------------------------------------------" );
                                log.WriteLine( "# Installed Singers #" );
                                string dir = Path.Combine( s_voicedbdir, voicedbdir );
                                if ( Directory.Exists( dir ) ) {
                                    string[] files = Directory.GetFiles( dir, "*.vvd" );
                                    foreach ( string s2 in files ) {
                                        string file = Path.Combine( dir, s2 );
                                        if ( File.Exists( file ) ) {
                                            string ret = "";
                                            SingerConfig inst = SingerConfig.readSingerConfig( file, -1, ref ret );
                                            log.WriteLine( "file=" + Path.GetFileName( s2 ) );
                                            log.WriteLine( "VOICENAME=" + inst.VOICENAME );
                                            log.WriteLine( "VOICEIDSTR=" + getmd5( inst.VOICEIDSTR ) );
                                            log.WriteLine( "ret=" + ret );
                                        }
                                    }

                                }
                                key.Close();
                                break;
                            }
                        }
                        regkey_voicedb2.Close();
                    }
                } catch (Exception ex ) {
                    Console.WriteLine( "ex=" + ex );
                }
            }
        }

        /// <summary>
        /// Transform the byte array(length=8) to unsigned long, assuming that the byte array is little endian.
        /// </summary>
        /// <param name="oct"></param>
        /// <returns></returns>
        private static ulong makelong_le( byte[] oct ) {
            return (ulong)oct[7] << 56 | (ulong)oct[6] << 48 | (ulong)oct[5] << 40 | (ulong)oct[4] << 32 | (ulong)oct[3] << 24 | (ulong)oct[2] << 16 | (ulong)oct[1] << 8 | (ulong)oct[0];
        }

        public static string getmd5( string s ) {
            MD5 md5 = MD5.Create();
            byte[] data = Encoding.Unicode.GetBytes( s );
            byte[] hash = md5.ComputeHash( data );
            return BitConverter.ToString( hash ).ToLower().Replace( "-", "" );
        }
    }

    public class SingerConfig : ICloneable {
        public string ID = "";
        public string FORMAT = "";
        public string VOICEIDSTR = "";
        public string VOICENAME = "Unknown";
        public int Breathiness;
        public int Brightness;
        public int Clearness;
        public int Opening;
        public int GenderFactor;
        public int Original;
        public int Program;

        public SingerConfig() {
        }

        public object Clone() {
            SingerConfig ret = new SingerConfig();
            ret.ID = ID;
            ret.FORMAT = FORMAT;
            ret.VOICEIDSTR = VOICEIDSTR;
            ret.VOICENAME = VOICENAME;
            ret.Breathiness = Breathiness;
            ret.Brightness = Brightness;
            ret.Clearness = Clearness;
            ret.Opening = Opening;
            ret.GenderFactor = GenderFactor;
            ret.Original = Original;
            ret.Program = Program;
            return ret;
        }

        public static void decode_vvd_bytes( ref byte[] dat ) {
            for ( int i = 0; i < dat.Length; i++ ) {
                byte M = (byte)(dat[i] >> 4);
                byte L = (byte)(dat[i] - (M << 4));
                byte newM = endecode_vvd_m( M );
                byte newL = endecode_vvd_l( L );
                dat[i] = (byte)((newM << 4) | newL);
            }
        }

        static byte endecode_vvd_l( byte value ) {
            switch ( value ) {
                case 0x0:
                    return 0xa;
                case 0x1:
                    return 0xb;
                case 0x2:
                    return 0x8;
                case 0x3:
                    return 0x9;
                case 0x4:
                    return 0xe;
                case 0x5:
                    return 0xf;
                case 0x6:
                    return 0xc;
                case 0x7:
                    return 0xd;
                case 0x8:
                    return 0x2;
                case 0x9:
                    return 0x3;
                case 0xa:
                    return 0x0;
                case 0xb:
                    return 0x1;
                case 0xc:
                    return 0x6;
                case 0xd:
                    return 0x7;
                case 0xe:
                    return 0x4;
                case 0xf:
                    return 0x5;
            }
            return 0x0;
        }

        static byte endecode_vvd_m( byte value ) {
            switch ( value ) {
                case 0x0:
                    return 0x1;
                case 0x1:
                    return 0x0;
                case 0x2:
                    return 0x3;
                case 0x3:
                    return 0x2;
                case 0x4:
                    return 0x5;
                case 0x5:
                    return 0x4;
                case 0x6:
                    return 0x7;
                case 0x7:
                    return 0x6;
                case 0x8:
                    return 0x9;
                case 0x9:
                    return 0x8;
                case 0xa:
                    return 0xb;
                case 0xb:
                    return 0xa;
                case 0xc:
                    return 0xd;
                case 0xd:
                    return 0xc;
                case 0xe:
                    return 0xf;
                case 0xf:
                    return 0xe;
            }
            return 0x0;
        }

        public static SingerConfig readSingerConfig( string file, int original ) {
            string f = "";
            return readSingerConfig( file, original, ref f );
        }

        public static SingerConfig readSingerConfig( string file, int original, ref string log ) {
            SingerConfig sc = new SingerConfig();
            //original = original;
            sc.ID = "VOCALOID:VIRTUAL:VOICE";
            sc.FORMAT = "2.0.0.0";
            sc.VOICEIDSTR = "";
            sc.VOICENAME = "Miku";
            sc.Breathiness = 0;
            sc.Brightness = 0;
            sc.Clearness = 0;
            sc.Opening = 0;
            sc.GenderFactor = 0;
            sc.Original = original; //original = 0;
            FileStream fs = null;
            try {
                fs = new FileStream( file, FileMode.Open, FileAccess.Read );
                int length = (int)fs.Length;
                byte[] dat = new byte[length];
                fs.Read( dat, 0, length );
                decode_vvd_bytes( ref dat );
                for ( int i = 0; i < dat.Length - 1; i++ ) {
                    if ( dat[i] == 0x17 && dat[i + 1] == 0x10 ) {
                        dat[i] = 0x0d;
                        dat[i + 1] = 0x0a;
                    }
                }
                string str = bocoree.cp932.convert( dat );
                string crlf = ((char)0x0d).ToString() + ((char)0x0a).ToString();
                string[] spl = str.Split( new string[] { crlf }, StringSplitOptions.RemoveEmptyEntries );

                foreach ( string s in spl ) {
                    int first = s.IndexOf( '"' );
                    int first_end = get_quated_string( s, first );
                    int second = s.IndexOf( '"', first_end + 1 );
                    int second_end = get_quated_string( s, second );
                    char[] chs = s.ToCharArray();
                    string id = new string( chs, first, first_end - first + 1 );
                    string value = new string( chs, second, second_end - second + 1 );
                    id = id.Substring( 1, id.Length - 2 );
                    value = value.Substring( 1, value.Length - 2 );
                    value = value.Replace( "\\\"", "\"" );
                    log += "\"" + id + "\" ";
                    if ( id == "ID" ) {
                        sc.ID = value;
                    } else if ( id == "FORMAT" ) {
                        sc.FORMAT = value;
                    } else if ( id == "VOICEIDSTR" ) {
                        sc.VOICEIDSTR = value;
                    } else if ( id == "VOICENAME" ) {
                        sc.VOICENAME = value;
                    } else if ( id == "Breathiness" ) {
                        try {
                            sc.Breathiness = int.Parse( value );
                        } catch {
                        }
                    } else if ( id == "Brightness" ) {
                        try {
                            sc.Brightness = int.Parse( value );
                        } catch {
                        }
                    } else if ( id == "Clearness" ) {
                        try {
                            sc.Clearness = int.Parse( value );
                        } catch {
                        }
                    } else if ( id == "Opening" ) {
                        try {
                            sc.Opening = int.Parse( value );
                        } catch {
                        }
                    } else if ( id == "Gender:Factor" ) {
                        try {
                            sc.GenderFactor = int.Parse( value );
                        } catch {
                        }
                    }
                }
            } catch {

            } finally {
                if ( fs != null ) {
                    fs.Close();
                }
            }
            return sc;
        }

        /// <summary>
        /// 位置positionにある'"'から，次に現れる'"'の位置を調べる．エスケープされた\"はスキップされる．'"'が見つからなかった場合-1を返す
        /// </summary>
        /// <param name="s"></param>
        /// <param name="position"></param>
        /// <returns></returns>
        static int get_quated_string( string s, int position ) {
            if ( position < 0 ) {
                return -1;
            }
            char[] chs = s.ToCharArray();
            if ( position >= chs.Length ) {
                return -1;
            }
            if ( chs[position] != '"' ) {
                return -1;
            }
            int end = -1;
            for ( int i = position + 1; i < chs.Length; i++ ) {
                if ( chs[i] == '"' && chs[i - 1] != '\\' ) {
                    end = i;
                    break;
                }
            }
            return end;
        }

        public string[] ToStringArray() {
            List<string> ret = new List<string>();
            ret.Add( "\"ID\":=:\"" + ID + "\"" );
            ret.Add( "\"FORMAT\":=:\"" + FORMAT + "\"" );
            ret.Add( "\"VOICEIDSTR\":=:\"" + VOICEIDSTR + "\"" );
            ret.Add( "\"VOICENAME\":=:\"" + VOICENAME.Replace( "\"", "\\\"" ) + "\"" );
            ret.Add( "\"Breathiness\":=:\"" + Breathiness + "\"" );
            ret.Add( "\"Brightness\":=:\"" + Brightness + "\"" );
            ret.Add( "\"Clearness\":=:\"" + Clearness + "\"" );
            ret.Add( "\"Opening\":=:\"" + Opening + "\"" );
            ret.Add( "\"Gender:Factor\":=:\"" + GenderFactor + "\"" );
            return ret.ToArray();
        }

        public override string ToString() {
            string[] r = ToStringArray();
            string ret = "";
            foreach ( string s in r ) {
                ret += s + "\n";
            }
            return ret;
        }
    }
}

namespace bocoree {

#if __cplusplus
    class cp932 {
#else
    public class cp932 {
#endif


#if __cplusplus
        const static int _LEN_DICT = 9484;
        static int _DICT_SRC[_LEN_DICT][2];
        static void init_cor(){
            int n[_LEN_DICT][2] = {
#else
        private static readonly Dictionary<int, int> _DICT = new Dictionary<int, int>() {
#endif
            { 0, 0 },
            { 1, 1 },
            { 2, 2 },
            { 3, 3 },
            { 4, 4 },
            { 5, 5 },
            { 6, 6 },
            { 7, 7 },
            { 8, 8 },
            { 9, 9 },
            { 10, 10 },
            { 11, 11 },
            { 12, 12 },
            { 13, 13 },
            { 14, 14 },
            { 15, 15 },
            { 16, 16 },
            { 17, 17 },
            { 18, 18 },
            { 19, 19 },
            { 20, 20 },
            { 21, 21 },
            { 22, 22 },
            { 23, 23 },
            { 24, 24 },
            { 25, 25 },
            { 26, 26 },
            { 27, 27 },
            { 28, 28 },
            { 29, 29 },
            { 30, 30 },
            { 31, 31 },
            { 32, 32 },
            { 33, 33 },
            { 34, 34 },
            { 35, 35 },
            { 36, 36 },
            { 37, 37 },
            { 38, 38 },
            { 39, 39 },
            { 40, 40 },
            { 41, 41 },
            { 42, 42 },
            { 43, 43 },
            { 44, 44 },
            { 45, 45 },
            { 46, 46 },
            { 47, 47 },
            { 48, 48 },
            { 49, 49 },
            { 50, 50 },
            { 51, 51 },
            { 52, 52 },
            { 53, 53 },
            { 54, 54 },
            { 55, 55 },
            { 56, 56 },
            { 57, 57 },
            { 58, 58 },
            { 59, 59 },
            { 60, 60 },
            { 61, 61 },
            { 62, 62 },
            { 64, 64 },
            { 65, 65 },
            { 66, 66 },
            { 67, 67 },
            { 68, 68 },
            { 69, 69 },
            { 70, 70 },
            { 71, 71 },
            { 72, 72 },
            { 73, 73 },
            { 74, 74 },
            { 75, 75 },
            { 76, 76 },
            { 77, 77 },
            { 78, 78 },
            { 79, 79 },
            { 80, 80 },
            { 81, 81 },
            { 82, 82 },
            { 83, 83 },
            { 84, 84 },
            { 85, 85 },
            { 86, 86 },
            { 87, 87 },
            { 88, 88 },
            { 89, 89 },
            { 90, 90 },
            { 91, 91 },
            { 92, 92 },
            { 93, 93 },
            { 94, 94 },
            { 95, 95 },
            { 96, 96 },
            { 97, 97 },
            { 98, 98 },
            { 99, 99 },
            { 100, 100 },
            { 101, 101 },
            { 102, 102 },
            { 103, 103 },
            { 104, 104 },
            { 105, 105 },
            { 106, 106 },
            { 107, 107 },
            { 108, 108 },
            { 109, 109 },
            { 110, 110 },
            { 111, 111 },
            { 112, 112 },
            { 113, 113 },
            { 114, 114 },
            { 115, 115 },
            { 116, 116 },
            { 117, 117 },
            { 118, 118 },
            { 119, 119 },
            { 120, 120 },
            { 121, 121 },
            { 122, 122 },
            { 123, 123 },
            { 124, 124 },
            { 125, 125 },
            { 126, 126 },
            { 127, 127 },
            { 128, 128 },
            { 161, 33 },
            { 162, 33169 },
            { 163, 33170 },
            { 165, 92 },
            { 166, 124 },
            { 167, 33176 },
            { 168, 33102 },
            { 169, 99 },
            { 170, 97 },
            { 171, 33249 },
            { 172, 33226 },
            { 173, 45 },
            { 174, 82 },
            { 175, 33104 },
            { 176, 33163 },
            { 177, 33149 },
            { 178, 50 },
            { 179, 51 },
            { 180, 33100 },
            { 181, 33738 },
            { 182, 33271 },
            { 183, 33093 },
            { 184, 33091 },
            { 185, 49 },
            { 186, 111 },
            { 187, 33250 },
            { 192, 65 },
            { 193, 65 },
            { 194, 65 },
            { 195, 65 },
            { 196, 65 },
            { 197, 65 },
            { 198, 65 },
            { 199, 67 },
            { 200, 69 },
            { 201, 69 },
            { 202, 69 },
            { 203, 69 },
            { 204, 73 },
            { 205, 73 },
            { 206, 73 },
            { 207, 73 },
            { 208, 68 },
            { 209, 78 },
            { 210, 79 },
            { 211, 79 },
            { 212, 79 },
            { 213, 79 },
            { 214, 79 },
            { 215, 33150 },
            { 216, 79 },
            { 217, 85 },
            { 218, 85 },
            { 219, 85 },
            { 220, 85 },
            { 221, 89 },
            { 222, 84 },
            { 223, 115 },
            { 224, 97 },
            { 225, 97 },
            { 226, 97 },
            { 227, 97 },
            { 228, 97 },
            { 229, 97 },
            { 230, 97 },
            { 231, 99 },
            { 232, 101 },
            { 233, 101 },
            { 234, 101 },
            { 235, 101 },
            { 236, 105 },
            { 237, 105 },
            { 238, 105 },
            { 239, 105 },
            { 240, 100 },
            { 241, 110 },
            { 242, 111 },
            { 243, 111 },
            { 244, 111 },
            { 245, 111 },
            { 246, 111 },
            { 247, 33152 },
            { 248, 111 },
            { 249, 117 },
            { 250, 117 },
            { 251, 117 },
            { 252, 117 },
            { 253, 121 },
            { 254, 116 },
            { 255, 121 },
            { 913, 33695 },
            { 914, 33696 },
            { 915, 33697 },
            { 916, 33698 },
            { 917, 33699 },
            { 918, 33700 },
            { 919, 33701 },
            { 920, 33702 },
            { 921, 33703 },
            { 922, 33704 },
            { 923, 33705 },
            { 924, 33706 },
            { 925, 33707 },
            { 926, 33708 },
            { 927, 33709 },
            { 928, 33710 },
            { 929, 33711 },
            { 931, 33712 },
            { 932, 33713 },
            { 933, 33714 },
            { 934, 33715 },
            { 935, 33716 },
            { 936, 33717 },
            { 937, 33718 },
            { 945, 33727 },
            { 946, 33728 },
            { 947, 33729 },
            { 948, 33730 },
            { 949, 33731 },
            { 950, 33732 },
            { 951, 33733 },
            { 952, 33734 },
            { 953, 33735 },
            { 954, 33736 },
            { 955, 33737 },
            { 956, 33738 },
            { 957, 33739 },
            { 958, 33740 },
            { 959, 33741 },
            { 960, 33742 },
            { 961, 33743 },
            { 963, 33744 },
            { 964, 33745 },
            { 965, 33746 },
            { 966, 33747 },
            { 967, 33748 },
            { 968, 33749 },
            { 969, 33750 },
            { 1025, 33862 },
            { 1040, 33856 },
            { 1041, 33857 },
            { 1042, 33858 },
            { 1043, 33859 },
            { 1044, 33860 },
            { 1045, 33861 },
            { 1046, 33863 },
            { 1047, 33864 },
            { 1048, 33865 },
            { 1049, 33866 },
            { 1050, 33867 },
            { 1051, 33868 },
            { 1052, 33869 },
            { 1053, 33870 },
            { 1054, 33871 },
            { 1055, 33872 },
            { 1056, 33873 },
            { 1057, 33874 },
            { 1058, 33875 },
            { 1059, 33876 },
            { 1060, 33877 },
            { 1061, 33878 },
            { 1062, 33879 },
            { 1063, 33880 },
            { 1064, 33881 },
            { 1065, 33882 },
            { 1066, 33883 },
            { 1067, 33884 },
            { 1068, 33885 },
            { 1069, 33886 },
            { 1070, 33887 },
            { 1071, 33888 },
            { 1072, 33904 },
            { 1073, 33905 },
            { 1074, 33906 },
            { 1075, 33907 },
            { 1076, 33908 },
            { 1077, 33909 },
            { 1078, 33911 },
            { 1079, 33912 },
            { 1080, 33913 },
            { 1081, 33914 },
            { 1082, 33915 },
            { 1083, 33916 },
            { 1084, 33917 },
            { 1085, 33918 },
            { 1086, 33920 },
            { 1087, 33921 },
            { 1088, 33922 },
            { 1089, 33923 },
            { 1090, 33924 },
            { 1091, 33925 },
            { 1092, 33926 },
            { 1093, 33927 },
            { 1094, 33928 },
            { 1095, 33929 },
            { 1096, 33930 },
            { 1097, 33931 },
            { 1098, 33932 },
            { 1099, 33933 },
            { 1100, 33934 },
            { 1101, 33935 },
            { 1102, 33936 },
            { 1103, 33937 },
            { 1105, 33910 },
            { 8208, 33117 },
            { 8213, 33116 },
            { 8216, 33125 },
            { 8217, 33126 },
            { 8220, 33127 },
            { 8221, 33128 },
            { 8224, 33269 },
            { 8225, 33270 },
            { 8229, 33124 },
            { 8230, 33123 },
            { 8240, 33265 },
            { 8242, 33164 },
            { 8243, 33165 },
            { 8251, 33190 },
            { 8451, 33166 },
            { 8470, 34690 },
            { 8481, 34692 },
            { 8491, 33264 },
            { 8544, 34644 },
            { 8545, 34645 },
            { 8546, 34646 },
            { 8547, 34647 },
            { 8548, 34648 },
            { 8549, 34649 },
            { 8550, 34650 },
            { 8551, 34651 },
            { 8552, 34652 },
            { 8553, 34653 },
            { 8560, 64064 },
            { 8561, 64065 },
            { 8562, 64066 },
            { 8563, 64067 },
            { 8564, 64068 },
            { 8565, 64069 },
            { 8566, 64070 },
            { 8567, 64071 },
            { 8568, 64072 },
            { 8569, 64073 },
            { 8592, 33193 },
            { 8593, 33194 },
            { 8594, 33192 },
            { 8595, 33195 },
            { 8658, 33227 },
            { 8660, 33228 },
            { 8704, 33229 },
            { 8706, 33245 },
            { 8707, 33230 },
            { 8711, 33246 },
            { 8712, 33208 },
            { 8715, 33209 },
            { 8721, 34708 },
            { 8730, 33251 },
            { 8733, 33253 },
            { 8734, 33159 },
            { 8735, 34712 },
            { 8736, 33242 },
            { 8741, 33121 },
            { 8743, 33224 },
            { 8744, 33225 },
            { 8745, 33215 },
            { 8746, 33214 },
            { 8747, 33255 },
            { 8748, 33256 },
            { 8750, 34707 },
            { 8756, 33160 },
            { 8757, 33254 },
            { 8765, 33252 },
            { 8786, 33248 },
            { 8800, 33154 },
            { 8801, 33247 },
            { 8806, 33157 },
            { 8807, 33158 },
            { 8810, 33249 },
            { 8811, 33250 },
            { 8834, 33212 },
            { 8835, 33213 },
            { 8838, 33210 },
            { 8839, 33211 },
            { 8869, 33243 },
            { 8895, 34713 },
            { 8978, 33244 },
            { 9312, 34624 },
            { 9313, 34625 },
            { 9314, 34626 },
            { 9315, 34627 },
            { 9316, 34628 },
            { 9317, 34629 },
            { 9318, 34630 },
            { 9319, 34631 },
            { 9320, 34632 },
            { 9321, 34633 },
            { 9322, 34634 },
            { 9323, 34635 },
            { 9324, 34636 },
            { 9325, 34637 },
            { 9326, 34638 },
            { 9327, 34639 },
            { 9328, 34640 },
            { 9329, 34641 },
            { 9330, 34642 },
            { 9331, 34643 },
            { 9472, 33951 },
            { 9473, 33962 },
            { 9474, 33952 },
            { 9475, 33963 },
            { 9484, 33953 },
            { 9487, 33964 },
            { 9488, 33954 },
            { 9491, 33965 },
            { 9492, 33956 },
            { 9495, 33967 },
            { 9496, 33955 },
            { 9499, 33966 },
            { 9500, 33957 },
            { 9501, 33978 },
            { 9504, 33973 },
            { 9507, 33968 },
            { 9508, 33959 },
            { 9509, 33980 },
            { 9512, 33975 },
            { 9515, 33970 },
            { 9516, 33958 },
            { 9519, 33974 },
            { 9520, 33979 },
            { 9523, 33969 },
            { 9524, 33960 },
            { 9527, 33976 },
            { 9528, 33981 },
            { 9531, 33971 },
            { 9532, 33961 },
            { 9535, 33977 },
            { 9538, 33982 },
            { 9547, 33972 },
            { 9632, 33185 },
            { 9633, 33184 },
            { 9650, 33187 },
            { 9651, 33186 },
            { 9660, 33189 },
            { 9661, 33188 },
            { 9670, 33183 },
            { 9671, 33182 },
            { 9675, 33179 },
            { 9678, 33181 },
            { 9679, 33180 },
            { 9711, 33276 },
            { 9733, 33178 },
            { 9734, 33177 },
            { 9792, 33162 },
            { 9794, 33161 },
            { 9834, 33268 },
            { 9837, 33267 },
            { 9839, 33266 },
            { 12288, 33088 },
            { 12289, 33089 },
            { 12290, 33090 },
            { 12291, 33110 },
            { 12293, 33112 },
            { 12294, 33113 },
            { 12295, 33114 },
            { 12296, 33137 },
            { 12297, 33138 },
            { 12298, 33139 },
            { 12299, 33140 },
            { 12300, 33141 },
            { 12301, 33142 },
            { 12302, 33143 },
            { 12303, 33144 },
            { 12304, 33145 },
            { 12305, 33146 },
            { 12306, 33191 },
            { 12307, 33196 },
            { 12308, 33131 },
            { 12309, 33132 },
            { 12317, 34688 },
            { 12319, 34689 },
            { 12353, 33439 },
            { 12354, 33440 },
            { 12355, 33441 },
            { 12356, 33442 },
            { 12357, 33443 },
            { 12358, 33444 },
            { 12359, 33445 },
            { 12360, 33446 },
            { 12361, 33447 },
            { 12362, 33448 },
            { 12363, 33449 },
            { 12364, 33450 },
            { 12365, 33451 },
            { 12366, 33452 },
            { 12367, 33453 },
            { 12368, 33454 },
            { 12369, 33455 },
            { 12370, 33456 },
            { 12371, 33457 },
            { 12372, 33458 },
            { 12373, 33459 },
            { 12374, 33460 },
            { 12375, 33461 },
            { 12376, 33462 },
            { 12377, 33463 },
            { 12378, 33464 },
            { 12379, 33465 },
            { 12380, 33466 },
            { 12381, 33467 },
            { 12382, 33468 },
            { 12383, 33469 },
            { 12384, 33470 },
            { 12385, 33471 },
            { 12386, 33472 },
            { 12387, 33473 },
            { 12388, 33474 },
            { 12389, 33475 },
            { 12390, 33476 },
            { 12391, 33477 },
            { 12392, 33478 },
            { 12393, 33479 },
            { 12394, 33480 },
            { 12395, 33481 },
            { 12396, 33482 },
            { 12397, 33483 },
            { 12398, 33484 },
            { 12399, 33485 },
            { 12400, 33486 },
            { 12401, 33487 },
            { 12402, 33488 },
            { 12403, 33489 },
            { 12404, 33490 },
            { 12405, 33491 },
            { 12406, 33492 },
            { 12407, 33493 },
            { 12408, 33494 },
            { 12409, 33495 },
            { 12410, 33496 },
            { 12411, 33497 },
            { 12412, 33498 },
            { 12413, 33499 },
            { 12414, 33500 },
            { 12415, 33501 },
            { 12416, 33502 },
            { 12417, 33503 },
            { 12418, 33504 },
            { 12419, 33505 },
            { 12420, 33506 },
            { 12421, 33507 },
            { 12422, 33508 },
            { 12423, 33509 },
            { 12424, 33510 },
            { 12425, 33511 },
            { 12426, 33512 },
            { 12427, 33513 },
            { 12428, 33514 },
            { 12429, 33515 },
            { 12430, 33516 },
            { 12431, 33517 },
            { 12432, 33518 },
            { 12433, 33519 },
            { 12434, 33520 },
            { 12435, 33521 },
            { 12436, 33684 },
            { 12443, 33098 },
            { 12444, 33099 },
            { 12445, 33108 },
            { 12446, 33109 },
            { 12449, 33600 },
            { 12450, 33601 },
            { 12451, 33602 },
            { 12452, 33603 },
            { 12453, 33604 },
            { 12454, 33605 },
            { 12455, 33606 },
            { 12456, 33607 },
            { 12457, 33608 },
            { 12458, 33609 },
            { 12459, 33610 },
            { 12460, 33611 },
            { 12461, 33612 },
            { 12462, 33613 },
            { 12463, 33614 },
            { 12464, 33615 },
            { 12465, 33616 },
            { 12466, 33617 },
            { 12467, 33618 },
            { 12468, 33619 },
            { 12469, 33620 },
            { 12470, 33621 },
            { 12471, 33622 },
            { 12472, 33623 },
            { 12473, 33624 },
            { 12474, 33625 },
            { 12475, 33626 },
            { 12476, 33627 },
            { 12477, 33628 },
            { 12478, 33629 },
            { 12479, 33630 },
            { 12480, 33631 },
            { 12481, 33632 },
            { 12482, 33633 },
            { 12483, 33634 },
            { 12484, 33635 },
            { 12485, 33636 },
            { 12486, 33637 },
            { 12487, 33638 },
            { 12488, 33639 },
            { 12489, 33640 },
            { 12490, 33641 },
            { 12491, 33642 },
            { 12492, 33643 },
            { 12493, 33644 },
            { 12494, 33645 },
            { 12495, 33646 },
            { 12496, 33647 },
            { 12497, 33648 },
            { 12498, 33649 },
            { 12499, 33650 },
            { 12500, 33651 },
            { 12501, 33652 },
            { 12502, 33653 },
            { 12503, 33654 },
            { 12504, 33655 },
            { 12505, 33656 },
            { 12506, 33657 },
            { 12507, 33658 },
            { 12508, 33659 },
            { 12509, 33660 },
            { 12510, 33661 },
            { 12511, 33662 },
            { 12512, 33664 },
            { 12513, 33665 },
            { 12514, 33666 },
            { 12515, 33667 },
            { 12516, 33668 },
            { 12517, 33669 },
            { 12518, 33670 },
            { 12519, 33671 },
            { 12520, 33672 },
            { 12521, 33673 },
            { 12522, 33674 },
            { 12523, 33675 },
            { 12524, 33676 },
            { 12525, 33677 },
            { 12526, 33678 },
            { 12527, 33679 },
            { 12528, 33680 },
            { 12529, 33681 },
            { 12530, 33682 },
            { 12531, 33683 },
            { 12532, 33684 },
            { 12533, 33685 },
            { 12534, 33686 },
            { 12539, 33093 },
            { 12540, 33115 },
            { 12541, 33106 },
            { 12542, 33107 },
            { 12849, 34698 },
            { 12850, 34699 },
            { 12857, 34700 },
            { 12964, 34693 },
            { 12965, 34694 },
            { 12966, 34695 },
            { 12967, 34696 },
            { 12968, 34697 },
            { 13059, 34661 },
            { 13069, 34665 },
            { 13076, 34656 },
            { 13080, 34659 },
            { 13090, 34657 },
            { 13091, 34667 },
            { 13094, 34666 },
            { 13095, 34660 },
            { 13099, 34668 },
            { 13110, 34662 },
            { 13115, 34670 },
            { 13129, 34655 },
            { 13130, 34669 },
            { 13133, 34658 },
            { 13137, 34663 },
            { 13143, 34664 },
            { 13179, 34686 },
            { 13180, 34703 },
            { 13181, 34702 },
            { 13182, 34701 },
            { 13198, 34674 },
            { 13199, 34675 },
            { 13212, 34671 },
            { 13213, 34672 },
            { 13214, 34673 },
            { 13217, 34677 },
            { 13252, 34676 },
            { 13261, 34691 },
            { 19968, 35050 },
            { 19969, 37530 },
            { 19971, 36533 },
            { 19975, 38556 },
            { 19976, 36836 },
            { 19977, 36431 },
            { 19978, 36835 },
            { 19979, 35258 },
            { 19981, 38259 },
            { 19982, 38750 },
            { 19984, 39072 },
            { 19985, 35150 },
            { 19988, 35470 },
            { 19989, 39073 },
            { 19990, 37026 },
            { 19991, 39360 },
            { 19992, 35701 },
            { 19993, 38328 },
            { 19998, 36837 },
            { 20001, 38844 },
            { 20006, 38336 },
            { 20008, 64104 },
            { 20010, 39074 },
            { 20013, 37510 },
            { 20017, 39075 },
            { 20018, 35832 },
            { 20022, 39076 },
            { 20024, 35547 },
            { 20025, 37455 },
            { 20027, 36581 },
            { 20028, 39077 },
            { 20031, 39078 },
            { 20034, 39079 },
            { 20035, 37972 },
            { 20037, 35702 },
            { 20043, 37974 },
            { 20045, 37857 },
            { 20046, 36033 },
            { 20047, 38482 },
            { 20053, 58728 },
            { 20054, 39080 },
            { 20055, 36838 },
            { 20056, 39081 },
            { 20057, 35251 },
            { 20061, 35811 },
            { 20062, 36078 },
            { 20063, 38631 },
            { 20066, 39844 },
            { 20081, 38800 },
            { 20083, 37883 },
            { 20094, 35491 },
            { 20096, 35668 },
            { 20098, 39082 },
            { 20101, 39083 },
            { 20102, 38841 },
            { 20104, 38748 },
            { 20105, 37256 },
            { 20106, 39085 },
            { 20107, 36502 },
            { 20108, 37873 },
            { 20110, 39088 },
            { 20113, 35165 },
            { 20114, 36061 },
            { 20116, 36060 },
            { 20117, 35044 },
            { 20120, 39018 },
            { 20121, 39017 },
            { 20123, 36273 },
            { 20124, 34975 },
            { 20126, 39089 },
            { 20127, 39090 },
            { 20128, 39091 },
            { 20129, 38483 },
            { 20130, 39092 },
            { 20132, 36080 },
            { 20133, 35045 },
            { 20134, 38546 },
            { 20136, 35740 },
            { 20139, 35741 },
            { 20140, 35742 },
            { 20141, 37600 },
            { 20142, 38842 },
            { 20144, 39093 },
            { 20147, 39094 },
            { 20150, 39095 },
            { 20154, 36972 },
            { 20160, 36697 },
            { 20161, 36973 },
            { 20162, 39100 },
            { 20164, 39098 },
            { 20166, 39099 },
            { 20167, 35703 },
            { 20170, 36257 },
            { 20171, 35310 },
            { 20173, 39097 },
            { 20174, 39096 },
            { 20175, 38311 },
            { 20180, 36453 },
            { 20181, 36452 },
            { 20182, 37308 },
            { 20183, 39101 },
            { 20184, 38260 },
            { 20185, 37093 },
            { 20189, 33111 },
            { 20190, 39102 },
            { 20191, 39104 },
            { 20193, 64105 },
            { 20195, 37347 },
            { 20196, 38879 },
            { 20197, 35016 },
            { 20205, 39103 },
            { 20206, 35260 },
            { 20208, 35778 },
            { 20210, 37511 },
            { 20214, 35983 },
            { 20215, 39105 },
            { 20219, 37955 },
            { 20220, 64106 },
            { 20224, 64107 },
            { 20225, 35561 },
            { 20227, 64108 },
            { 20233, 39106 },
            { 20234, 35017 },
            { 20237, 36062 },
            { 20238, 35562 },
            { 20239, 38298 },
            { 20240, 38064 },
            { 20241, 35704 },
            { 20250, 35311 },
            { 20252, 39141 },
            { 20253, 37728 },
            { 20271, 38028 },
            { 20272, 39108 },
            { 20276, 38074 },
            { 20278, 38880 },
            { 20280, 36940 },
            { 20281, 64109 },
            { 20282, 36454 },
            { 20284, 36503 },
            { 20285, 35262 },
            { 20291, 37583 },
            { 20294, 37441 },
            { 20295, 39112 },
            { 20301, 35018 },
            { 20302, 37601 },
            { 20303, 36698 },
            { 20304, 36274 },
            { 20305, 38723 },
            { 20307, 37324 },
            { 20309, 35261 },
            { 20310, 64110 },
            { 20311, 39111 },
            { 20313, 38749 },
            { 20314, 39107 },
            { 20315, 39109 },
            { 20316, 36332 },
            { 20317, 39110 },
            { 20318, 39747 },
            { 20329, 39118 },
            { 20335, 39121 },
            { 20336, 39119 },
            { 20339, 35264 },
            { 20341, 38329 },
            { 20342, 39113 },
            { 20347, 39117 },
            { 20348, 36081 },
            { 20351, 36455 },
            { 20355, 35492 },
            { 20358, 39122 },
            { 20360, 39114 },
            { 20362, 64112 },
            { 20363, 38881 },
            { 20365, 36504 },
            { 20367, 39115 },
            { 20369, 39120 },
            { 20370, 64111 },
            { 20372, 64114 },
            { 20374, 39123 },
            { 20376, 39116 },
            { 20378, 64113 },
            { 20379, 35743 },
            { 20381, 35019 },
            { 20384, 35744 },
            { 20385, 35263 },
            { 20395, 39748 },
            { 20397, 38553 },
            { 20398, 38286 },
            { 20399, 36082 },
            { 20405, 36942 },
            { 20406, 38837 },
            { 20415, 38358 },
            { 20418, 35927 },
            { 20419, 37283 },
            { 20420, 35298 },
            { 20425, 64097 },
            { 20426, 36722 },
            { 20429, 64115 },
            { 20430, 39127 },
            { 20432, 39132 },
            { 20433, 39130 },
            { 20436, 39125 },
            { 20439, 37293 },
            { 20440, 39128 },
            { 20442, 39131 },
            { 20443, 39129 },
            { 20445, 38363 },
            { 20447, 39126 },
            { 20449, 36941 },
            { 20451, 38547 },
            { 20452, 39133 },
            { 20453, 39134 },
            { 20462, 36675 },
            { 20463, 39147 },
            { 20467, 37999 },
            { 20469, 38229 },
            { 20470, 39142 },
            { 20472, 38382 },
            { 20474, 35252 },
            { 20478, 39146 },
            { 20479, 64118 },
            { 20485, 39140 },
            { 20486, 39149 },
            { 20489, 37233 },
            { 20491, 36034 },
            { 20493, 38011 },
            { 20495, 57541 },
            { 20497, 39148 },
            { 20498, 37756 },
            { 20500, 39137 },
            { 20502, 36084 },
            { 20505, 36083 },
            { 20506, 39135 },
            { 20510, 64119 },
            { 20511, 36568 },
            { 20513, 39143 },
            { 20514, 64117 },
            { 20515, 38381 },
            { 20516, 37484 },
            { 20517, 39139 },
            { 20518, 35985 },
            { 20520, 39136 },
            { 20521, 39144 },
            { 20522, 39138 },
            { 20523, 38863 },
            { 20524, 39145 },
            { 20525, 39008 },
            { 20534, 35812 },
            { 20537, 35984 },
            { 20544, 64116 },
            { 20546, 64122 },
            { 20547, 39150 },
            { 20550, 64120 },
            { 20551, 39151 },
            { 20552, 39155 },
            { 20553, 35020 },
            { 20559, 38350 },
            { 20560, 39154 },
            { 20565, 39153 },
            { 20566, 39157 },
            { 20570, 39156 },
            { 20572, 37602 },
            { 20581, 35986 },
            { 20588, 39158 },
            { 20592, 64121 },
            { 20594, 36547 },
            { 20596, 37284 },
            { 20597, 37603 },
            { 20598, 35828 },
            { 20600, 39159 },
            { 20605, 35669 },
            { 20608, 39160 },
            { 20613, 39162 },
            { 20621, 38484 },
            { 20625, 35974 },
            { 20628, 64123 },
            { 20632, 36432 },
            { 20633, 38133 },
            { 20634, 39161 },
            { 20652, 36291 },
            { 20653, 38754 },
            { 20658, 39164 },
            { 20659, 39234 },
            { 20660, 39163 },
            { 20661, 36290 },
            { 20663, 36765 },
            { 20670, 35928 },
            { 20674, 39235 },
            { 20677, 35789 },
            { 20681, 39232 },
            { 20682, 39233 },
            { 20685, 37805 },
            { 20687, 37276 },
            { 20689, 35745 },
            { 20693, 38508 },
            { 20694, 39236 },
            { 20696, 64125 },
            { 20698, 38843 },
            { 20702, 39237 },
            { 20707, 39240 },
            { 20709, 39238 },
            { 20711, 37229 },
            { 20717, 39239 },
            { 20718, 39241 },
            { 20724, 64124 },
            { 20725, 39243 },
            { 20729, 39242 },
            { 20731, 38342 },
            { 20736, 35670 },
            { 20737, 39245 },
            { 20738, 39246 },
            { 20740, 35245 },
            { 20745, 39244 },
            { 20754, 36594 },
            { 20756, 39249 },
            { 20757, 39248 },
            { 20758, 39247 },
            { 20760, 39124 },
            { 20762, 39250 },
            { 20767, 36766 },
            { 20769, 39251 },
            { 20778, 38724 },
            { 20786, 38615 },
            { 20791, 39253 },
            { 20794, 39252 },
            { 20795, 39255 },
            { 20796, 39254 },
            { 20799, 39256 },
            { 20800, 39257 },
            { 20801, 35058 },
            { 20803, 36019 },
            { 20804, 35930 },
            { 20805, 36699 },
            { 20806, 37531 },
            { 20807, 35746 },
            { 20808, 37094 },
            { 20809, 36085 },
            { 20810, 64126 },
            { 20811, 36238 },
            { 20812, 39259 },
            { 20813, 38598 },
            { 20814, 37733 },
            { 20816, 36505 },
            { 20818, 39258 },
            { 20820, 39260 },
            { 20826, 37757 },
            { 20828, 35477 },
            { 20834, 39261 },
            { 20836, 64128 },
            { 20837, 37884 },
            { 20840, 37203 },
            { 20841, 39263 },
            { 20842, 39264 },
            { 20843, 38058 },
            { 20844, 36086 },
            { 20845, 39002 },
            { 20846, 39265 },
            { 20849, 35748 },
            { 20853, 38330 },
            { 20854, 37300 },
            { 20855, 35823 },
            { 20856, 37716 },
            { 20860, 35987 },
            { 20864, 39266 },
            { 20866, 39267 },
            { 20869, 37856 },
            { 20870, 35198 },
            { 20873, 39270 },
            { 20874, 36347 },
            { 20876, 39269 },
            { 20877, 36292 },
            { 20879, 39271 },
            { 20880, 58348 },
            { 20881, 39272 },
            { 20882, 38496 },
            { 20883, 39273 },
            { 20885, 39274 },
            { 20886, 39275 },
            { 20887, 36839 },
            { 20889, 36554 },
            { 20893, 64129 },
            { 20896, 35493 },
            { 20898, 39278 },
            { 20900, 39276 },
            { 20901, 38587 },
            { 20902, 39277 },
            { 20904, 38265 },
            { 20905, 39279 },
            { 20906, 39280 },
            { 20907, 39281 },
            { 20908, 37758 },
            { 20912, 39285 },
            { 20913, 39283 },
            { 20914, 39284 },
            { 20915, 39282 },
            { 20916, 36321 },
            { 20917, 39286 },
            { 20918, 38632 },
            { 20919, 38882 },
            { 20925, 39287 },
            { 20926, 64130 },
            { 20932, 37030 },
            { 20933, 39288 },
            { 20934, 36729 },
            { 20937, 39289 },
            { 20939, 37532 },
            { 20940, 38845 },
            { 20941, 37760 },
            { 20950, 39363 },
            { 20955, 39290 },
            { 20956, 60067 },
            { 20957, 35779 },
            { 20960, 39291 },
            { 20961, 38525 },
            { 20966, 36744 },
            { 20967, 37370 },
            { 20969, 39293 },
            { 20970, 37858 },
            { 20972, 64131 },
            { 20973, 39294 },
            { 20976, 39296 },
            { 20977, 35405 },
            { 20981, 39297 },
            { 20982, 35749 },
            { 20984, 37834 },
            { 20985, 35226 },
            { 20986, 36719 },
            { 20989, 38047 },
            { 20990, 39298 },
            { 20992, 37761 },
            { 20995, 36974 },
            { 20996, 39299 },
            { 20998, 38314 },
            { 20999, 37080 },
            { 21000, 35488 },
            { 21002, 35495 },
            { 21003, 39300 },
            { 21006, 39302 },
            { 21009, 35929 },
            { 21012, 39301 },
            { 21013, 64132 },
            { 21015, 38897 },
            { 21021, 36745 },
            { 21028, 38075 },
            { 21029, 38346 },
            { 21031, 39303 },
            { 21033, 38808 },
            { 21034, 39304 },
            { 21038, 39305 },
            { 21040, 37790 },
            { 21043, 39306 },
            { 21046, 37031 },
            { 21047, 36348 },
            { 21048, 35988 },
            { 21049, 39307 },
            { 21050, 36456 },
            { 21051, 36239 },
            { 21059, 37604 },
            { 21060, 39309 },
            { 21063, 37285 },
            { 21066, 36333 },
            { 21067, 39310 },
            { 21068, 39311 },
            { 21069, 37199 },
            { 21071, 39308 },
            { 21076, 39313 },
            { 21078, 38485 },
            { 21083, 36228 },
            { 21086, 39312 },
            { 21091, 35989 },
            { 21092, 36316 },
            { 21093, 38029 },
            { 21097, 39316 },
            { 21098, 39314 },
            { 21103, 38299 },
            { 21104, 36840 },
            { 21105, 39323 },
            { 21106, 35460 },
            { 21107, 39317 },
            { 21108, 39315 },
            { 21109, 37230 },
            { 21117, 39319 },
            { 21119, 39318 },
            { 21123, 35427 },
            { 21127, 35968 },
            { 21128, 39324 },
            { 21129, 38827 },
            { 21133, 39320 },
            { 21137, 39325 },
            { 21138, 39322 },
            { 21140, 39321 },
            { 21147, 38861 },
            { 21148, 64133 },
            { 21151, 36087 },
            { 21152, 35265 },
            { 21155, 38898 },
            { 21158, 64134 },
            { 21161, 36757 },
            { 21162, 37751 },
            { 21163, 36229 },
            { 21164, 39328 },
            { 21165, 39329 },
            { 21167, 64375 },
            { 21169, 38883 },
            { 21172, 38986 },
            { 21173, 39331 },
            { 21177, 36088 },
            { 21180, 39330 },
            { 21182, 35406 },
            { 21184, 64135 },
            { 21185, 39332 },
            { 21187, 38517 },
            { 21189, 37562 },
            { 21191, 38725 },
            { 21193, 38359 },
            { 21197, 39333 },
            { 21202, 59603 },
            { 21205, 37806 },
            { 21207, 39334 },
            { 21208, 35496 },
            { 21209, 38577 },
            { 21211, 64136 },
            { 21213, 36767 },
            { 21214, 39335 },
            { 21215, 38373 },
            { 21216, 39339 },
            { 21218, 37032 },
            { 21219, 39336 },
            { 21220, 35790 },
            { 21222, 39337 },
            { 21223, 35497 },
            { 21234, 35917 },
            { 21235, 39340 },
            { 21237, 39341 },
            { 21240, 39342 },
            { 21241, 39343 },
            { 21242, 36569 },
            { 21246, 36089 },
            { 21247, 38620 },
            { 21248, 64137 },
            { 21249, 38630 },
            { 21250, 37877 },
            { 21253, 38383 },
            { 21254, 39344 },
            { 21255, 64138 },
            { 21256, 39345 },
            { 21261, 39347 },
            { 21263, 39349 },
            { 21264, 39348 },
            { 21269, 39350 },
            { 21270, 35259 },
            { 21271, 38507 },
            { 21273, 36346 },
            { 21274, 39351 },
            { 21277, 37240 },
            { 21280, 36768 },
            { 21281, 35751 },
            { 21283, 39352 },
            { 21284, 64139 },
            { 21290, 38105 },
            { 21295, 39353 },
            { 21297, 39354 },
            { 21299, 39355 },
            { 21304, 39356 },
            { 21305, 38211 },
            { 21306, 35814 },
            { 21307, 35043 },
            { 21311, 37821 },
            { 21312, 39357 },
            { 21313, 36700 },
            { 21315, 37095 },
            { 21317, 39359 },
            { 21318, 39358 },
            { 21319, 36769 },
            { 21320, 36063 },
            { 21321, 39361 },
            { 21322, 38076 },
            { 21325, 39362 },
            { 21329, 38106 },
            { 21330, 37298 },
            { 21331, 37356 },
            { 21332, 35750 },
            { 21335, 37868 },
            { 21336, 37456 },
            { 21338, 38030 },
            { 21340, 38509 },
            { 21342, 39364 },
            { 21344, 37096 },
            { 21350, 35924 },
            { 21353, 39365 },
            { 21358, 39366 },
            { 21359, 35147 },
            { 21360, 35059 },
            { 21361, 35563 },
            { 21362, 64140 },
            { 21363, 37286 },
            { 21364, 35696 },
            { 21365, 38801 },
            { 21367, 39369 },
            { 21368, 35253 },
            { 21371, 39368 },
            { 21375, 35752 },
            { 21378, 39370 },
            { 21380, 38639 },
            { 21395, 64141 },
            { 21398, 39371 },
            { 21400, 38864 },
            { 21402, 36090 },
            { 21407, 36020 },
            { 21408, 39372 },
            { 21413, 39374 },
            { 21414, 39373 },
            { 21416, 36990 },
            { 21417, 35160 },
            { 21421, 35197 },
            { 21422, 39375 },
            { 21424, 39376 },
            { 21426, 64142 },
            { 21427, 36021 },
            { 21430, 39377 },
            { 21435, 35726 },
            { 21442, 36433 },
            { 21443, 39378 },
            { 21448, 38548 },
            { 21449, 36275 },
            { 21450, 35705 },
            { 21451, 38726 },
            { 21452, 37231 },
            { 21453, 38077 },
            { 21454, 36603 },
            { 21460, 36710 },
            { 21462, 36582 },
            { 21463, 36595 },
            { 21465, 36758 },
            { 21467, 38078 },
            { 21469, 64143 },
            { 21471, 39381 },
            { 21473, 35170 },
            { 21474, 37232 },
            { 21475, 36091 },
            { 21476, 36035 },
            { 21477, 35813 },
            { 21480, 39385 },
            { 21481, 37440 },
            { 21482, 37372 },
            { 21483, 35753 },
            { 21484, 36770 },
            { 21485, 39386 },
            { 21486, 39384 },
            { 21487, 35266 },
            { 21488, 37348 },
            { 21489, 36534 },
            { 21490, 36458 },
            { 21491, 35141 },
            { 21494, 35472 },
            { 21495, 36230 },
            { 21496, 36457 },
            { 21498, 39387 },
            { 21505, 39388 },
            { 21507, 35688 },
            { 21508, 35429 },
            { 21512, 36231 },
            { 21513, 35687 },
            { 21514, 37597 },
            { 21515, 35140 },
            { 21516, 37807 },
            { 21517, 38588 },
            { 21518, 36160 },
            { 21519, 38809 },
            { 21520, 37734 },
            { 21521, 36092 },
            { 21531, 35918 },
            { 21533, 39397 },
            { 21535, 35809 },
            { 21536, 38505 },
            { 21542, 38107 },
            { 21545, 39396 },
            { 21547, 35548 },
            { 21548, 39391 },
            { 21549, 39392 },
            { 21550, 39394 },
            { 21558, 39395 },
            { 21560, 35706 },
            { 21561, 36993 },
            { 21563, 38315 },
            { 21564, 39393 },
            { 21565, 39389 },
            { 21566, 36065 },
            { 21568, 39390 },
            { 21570, 38979 },
            { 21574, 38384 },
            { 21576, 37606 },
            { 21577, 36064 },
            { 21578, 36240 },
            { 21582, 39398 },
            { 21585, 37851 },
            { 21599, 39402 },
            { 21608, 36604 },
            { 21610, 36596 },
            { 21616, 39405 },
            { 21617, 39403 },
            { 21619, 38561 },
            { 21621, 39400 },
            { 21622, 39409 },
            { 21623, 39404 },
            { 21627, 39407 },
            { 21628, 36036 },
            { 21629, 38589 },
            { 21632, 39408 },
            { 21636, 39410 },
            { 21638, 39412 },
            { 21642, 64146 },
            { 21643, 36334 },
            { 21644, 39009 },
            { 21646, 39401 },
            { 21647, 39399 },
            { 21648, 39411 },
            { 21650, 39406 },
            { 21660, 64145 },
            { 21666, 39414 },
            { 21668, 39490 },
            { 21669, 39416 },
            { 21672, 39420 },
            { 21673, 64147 },
            { 21675, 39488 },
            { 21676, 39417 },
            { 21679, 39517 },
            { 21682, 36327 },
            { 21683, 35408 },
            { 21688, 39415 },
            { 21692, 39492 },
            { 21693, 35060 },
            { 21694, 39491 },
            { 21696, 34979 },
            { 21697, 38249 },
            { 21698, 39489 },
            { 21700, 39418 },
            { 21703, 39413 },
            { 21704, 39419 },
            { 21705, 36294 },
            { 21720, 39493 },
            { 21729, 35061 },
            { 21730, 39502 },
            { 21733, 39494 },
            { 21734, 39495 },
            { 21736, 36771 },
            { 21737, 38537 },
            { 21741, 39500 },
            { 21742, 39499 },
            { 21746, 37710 },
            { 21754, 39501 },
            { 21757, 39498 },
            { 21759, 64148 },
            { 21764, 35155 },
            { 21766, 36276 },
            { 21767, 36943 },
            { 21775, 39496 },
            { 21776, 37762 },
            { 21780, 39497 },
            { 21782, 34976 },
            { 21806, 39507 },
            { 21807, 38722 },
            { 21809, 36773 },
            { 21811, 39513 },
            { 21816, 39512 },
            { 21817, 39503 },
            { 21822, 37313 },
            { 21824, 39504 },
            { 21828, 37357 },
            { 21829, 39509 },
            { 21830, 36772 },
            { 21836, 39506 },
            { 21839, 38626 },
            { 21843, 35931 },
            { 21846, 39510 },
            { 21847, 39511 },
            { 21852, 39508 },
            { 21853, 39514 },
            { 21859, 39505 },
            { 21883, 39520 },
            { 21884, 39525 },
            { 21886, 39521 },
            { 21888, 39516 },
            { 21891, 39526 },
            { 21892, 37200 },
            { 21894, 64149 },
            { 21895, 39528 },
            { 21897, 36161 },
            { 21898, 39518 },
            { 21899, 37533 },
            { 21912, 39522 },
            { 21913, 39515 },
            { 21914, 35499 },
            { 21916, 35564 },
            { 21917, 35461 },
            { 21918, 39523 },
            { 21919, 39519 },
            { 21927, 35990 },
            { 21928, 39529 },
            { 21929, 39527 },
            { 21930, 37234 },
            { 21931, 35689 },
            { 21932, 35754 },
            { 21934, 39524 },
            { 21936, 35826 },
            { 21942, 35171 },
            { 21956, 39533 },
            { 21957, 39531 },
            { 21959, 39589 },
            { 21972, 39536 },
            { 21978, 39530 },
            { 21980, 39534 },
            { 21983, 39532 },
            { 21987, 36459 },
            { 21988, 39535 },
            { 22007, 39538 },
            { 22009, 39543 },
            { 22013, 39541 },
            { 22014, 39540 },
            { 22022, 37457 },
            { 22025, 35267 },
            { 22036, 39537 },
            { 22038, 39539 },
            { 22039, 36774 },
            { 22040, 35154 },
            { 22043, 39542 },
            { 22057, 35292 },
            { 22063, 39554 },
            { 22065, 36858 },
            { 22066, 39549 },
            { 22068, 39547 },
            { 22070, 39548 },
            { 22072, 39550 },
            { 22082, 35164 },
            { 22092, 37208 },
            { 22094, 39544 },
            { 22096, 39545 },
            { 22107, 35482 },
            { 22116, 39553 },
            { 22120, 35565 },
            { 22122, 39556 },
            { 22123, 39552 },
            { 22124, 39555 },
            { 22132, 38316 },
            { 22136, 37843 },
            { 22138, 38070 },
            { 22144, 39558 },
            { 22150, 39557 },
            { 22151, 35428 },
            { 22154, 39559 },
            { 22159, 39562 },
            { 22164, 39561 },
            { 22176, 39560 },
            { 22178, 37976 },
            { 22181, 39563 },
            { 22190, 39564 },
            { 22196, 39566 },
            { 22198, 39565 },
            { 22204, 39568 },
            { 22208, 39571 },
            { 22209, 39569 },
            { 22210, 39567 },
            { 22211, 39570 },
            { 22216, 39572 },
            { 22222, 39573 },
            { 22225, 39574 },
            { 22227, 39575 },
            { 22231, 39576 },
            { 22232, 39268 },
            { 22234, 36602 },
            { 22235, 36460 },
            { 22238, 35313 },
            { 22240, 35062 },
            { 22243, 37475 },
            { 22254, 39577 },
            { 22256, 36258 },
            { 22258, 35021 },
            { 22259, 36989 },
            { 22265, 39578 },
            { 22266, 36037 },
            { 22269, 36241 },
            { 22271, 39580 },
            { 22272, 39579 },
            { 22275, 38366 },
            { 22276, 39581 },
            { 22280, 39583 },
            { 22281, 39582 },
            { 22283, 39584 },
            { 22285, 39585 },
            { 22287, 35991 },
            { 22290, 35200 },
            { 22291, 39586 },
            { 22294, 39588 },
            { 22296, 39587 },
            { 22300, 39590 },
            { 22303, 37753 },
            { 22310, 39591 },
            { 22311, 34995 },
            { 22312, 36317 },
            { 22317, 35932 },
            { 22320, 37486 },
            { 22327, 39592 },
            { 22328, 39593 },
            { 22331, 39595 },
            { 22336, 39596 },
            { 22338, 36322 },
            { 22343, 35791 },
            { 22346, 38486 },
            { 22350, 39594 },
            { 22351, 39597 },
            { 22352, 36287 },
            { 22353, 36162 },
            { 22361, 64150 },
            { 22369, 39601 },
            { 22372, 36259 },
            { 22373, 64151 },
            { 22374, 37458 },
            { 22377, 39598 },
            { 22378, 37592 },
            { 22399, 39602 },
            { 22402, 36994 },
            { 22408, 39600 },
            { 22409, 39603 },
            { 22411, 35934 },
            { 22419, 39604 },
            { 22432, 39605 },
            { 22434, 36163 },
            { 22435, 35423 },
            { 22436, 39607 },
            { 22442, 39608 },
            { 22444, 64152 },
            { 22448, 39609 },
            { 22451, 39606 },
            { 22464, 39599 },
            { 22467, 39610 },
            { 22470, 39611 },
            { 22471, 64154 },
            { 22472, 64153 },
            { 22475, 38532 },
            { 22478, 36841 },
            { 22482, 39613 },
            { 22483, 39614 },
            { 22484, 39612 },
            { 22486, 39616 },
            { 22492, 37975 },
            { 22495, 35046 },
            { 22496, 38261 },
            { 22499, 39617 },
            { 22516, 36859 },
            { 22519, 36535 },
            { 22521, 38012 },
            { 22522, 35566 },
            { 22524, 36329 },
            { 22528, 38520 },
            { 22530, 37808 },
            { 22533, 35992 },
            { 22534, 37325 },
            { 22538, 39615 },
            { 22539, 39618 },
            { 22549, 37314 },
            { 22553, 39619 },
            { 22557, 39620 },
            { 22561, 39622 },
            { 22564, 37607 },
            { 22570, 35500 },
            { 22575, 60063 },
            { 22576, 35201 },
            { 22577, 38385 },
            { 22580, 36842 },
            { 22581, 37735 },
            { 22586, 36324 },
            { 22589, 39628 },
            { 22592, 38331 },
            { 22593, 38875 },
            { 22602, 35314 },
            { 22603, 39624 },
            { 22609, 37209 },
            { 22610, 39627 },
            { 22612, 37763 },
            { 22615, 37736 },
            { 22616, 37764 },
            { 22617, 38071 },
            { 22618, 37579 },
            { 22622, 36295 },
            { 22626, 39623 },
            { 22633, 35222 },
            { 22635, 37717 },
            { 22640, 39625 },
            { 22642, 39621 },
            { 22645, 36975 },
            { 22649, 39629 },
            { 22654, 36717 },
            { 22659, 35755 },
            { 22661, 39630 },
            { 22675, 38374 },
            { 22679, 37277 },
            { 22684, 37572 },
            { 22686, 64157 },
            { 22687, 39632 },
            { 22696, 38510 },
            { 22699, 39633 },
            { 22702, 39638 },
            { 22706, 64158 },
            { 22707, 38317 },
            { 22712, 39637 },
            { 22713, 39631 },
            { 22714, 39634 },
            { 22715, 39636 },
            { 22718, 36260 },
            { 22721, 38343 },
            { 22725, 39639 },
            { 22727, 37476 },
            { 22730, 35315 },
            { 22732, 36843 },
            { 22737, 39641 },
            { 22739, 39640 },
            { 22741, 36232 },
            { 22743, 39642 },
            { 22744, 39644 },
            { 22745, 39643 },
            { 22748, 39646 },
            { 22750, 39635 },
            { 22751, 39648 },
            { 22756, 39647 },
            { 22757, 39645 },
            { 22763, 36461 },
            { 22764, 36976 },
            { 22766, 37235 },
            { 22767, 39649 },
            { 22768, 37050 },
            { 22769, 35051 },
            { 22770, 38020 },
            { 22775, 37593 },
            { 22777, 39651 },
            { 22778, 39650 },
            { 22779, 39652 },
            { 22780, 39653 },
            { 22781, 39654 },
            { 22786, 39655 },
            { 22793, 38351 },
            { 22794, 39656 },
            { 22795, 64159 },
            { 22799, 35268 },
            { 22800, 39657 },
            { 22805, 38747 },
            { 22806, 35407 },
            { 22808, 39367 },
            { 22809, 36711 },
            { 22810, 37309 },
            { 22811, 39658 },
            { 22812, 38633 },
            { 22818, 38578 },
            { 22821, 39660 },
            { 22823, 37349 },
            { 22825, 37718 },
            { 22826, 37310 },
            { 22827, 38262 },
            { 22828, 39661 },
            { 22829, 39662 },
            { 22830, 35227 },
            { 22833, 36536 },
            { 22834, 39663 },
            { 22839, 35022 },
            { 22840, 39664 },
            { 22846, 39665 },
            { 22852, 35202 },
            { 22855, 35567 },
            { 22856, 37854 },
            { 22857, 38386 },
            { 22862, 39669 },
            { 22863, 37236 },
            { 22864, 39668 },
            { 22865, 35935 },
            { 22867, 64160 },
            { 22868, 38522 },
            { 22869, 39667 },
            { 22871, 37765 },
            { 22872, 39671 },
            { 22874, 39670 },
            { 22875, 64161 },
            { 22877, 64162 },
            { 22880, 39673 },
            { 22882, 39672 },
            { 22883, 64163 },
            { 22885, 35228 },
            { 22887, 39674 },
            { 22888, 36775 },
            { 22889, 39676 },
            { 22890, 37444 },
            { 22892, 39675 },
            { 22894, 38321 },
            { 22899, 36759 },
            { 22900, 37754 },
            { 22904, 39744 },
            { 22909, 36164 },
            { 22913, 39745 },
            { 22914, 37952 },
            { 22915, 38108 },
            { 22916, 38607 },
            { 22922, 37956 },
            { 22925, 39754 },
            { 22931, 35671 },
            { 22934, 38756 },
            { 22937, 38573 },
            { 22939, 39850 },
            { 22941, 39746 },
            { 22947, 39749 },
            { 22948, 64164 },
            { 22949, 37315 },
            { 22952, 38487 },
            { 22956, 37737 },
            { 22962, 39750 },
            { 22969, 38533 },
            { 22970, 64165 },
            { 22971, 36296 },
            { 22974, 36776 },
            { 22982, 39751 },
            { 22985, 36463 },
            { 22987, 36462 },
            { 22992, 34999 },
            { 22993, 36038 },
            { 22995, 37033 },
            { 22996, 35023 },
            { 23001, 39755 },
            { 23002, 39756 },
            { 23004, 39753 },
            { 23013, 35159 },
            { 23014, 35501 },
            { 23016, 39752 },
            { 23018, 38595 },
            { 23019, 38224 },
            { 23030, 34982 },
            { 23035, 35063 },
            { 23039, 36464 },
            { 23041, 35024 },
            { 23043, 34977 },
            { 23049, 39761 },
            { 23057, 39759 },
            { 23064, 38586 },
            { 23066, 39762 },
            { 23068, 39760 },
            { 23071, 39758 },
            { 23072, 36944 },
            { 23077, 39757 },
            { 23081, 38360 },
            { 23087, 36066 },
            { 23093, 39766 },
            { 23094, 39767 },
            { 23100, 36777 },
            { 23104, 39763 },
            { 23105, 38987 },
            { 23110, 37995 },
            { 23113, 39765 },
            { 23130, 36261 },
            { 23138, 39768 },
            { 23142, 38263 },
            { 23146, 39769 },
            { 23148, 39764 },
            { 23167, 38585 },
            { 23186, 38013 },
            { 23194, 39770 },
            { 23195, 38225 },
            { 23228, 39771 },
            { 23229, 39775 },
            { 23230, 39772 },
            { 23233, 35269 },
            { 23234, 39774 },
            { 23241, 36537 },
            { 23243, 39773 },
            { 23244, 35993 },
            { 23248, 39787 },
            { 23254, 39780 },
            { 23255, 39777 },
            { 23265, 37508 },
            { 23267, 39776 },
            { 23270, 39778 },
            { 23273, 39779 },
            { 23290, 39781 },
            { 23291, 39782 },
            { 23305, 35568 },
            { 23307, 39784 },
            { 23308, 39783 },
            { 23318, 39785 },
            { 23330, 36844 },
            { 23338, 39788 },
            { 23340, 37594 },
            { 23344, 35172 },
            { 23346, 39786 },
            { 23350, 39789 },
            { 23358, 39790 },
            { 23360, 39793 },
            { 23363, 39791 },
            { 23365, 39792 },
            { 23376, 36465 },
            { 23377, 39794 },
            { 23380, 36165 },
            { 23381, 39795 },
            { 23382, 64166 },
            { 23383, 36506 },
            { 23384, 37302 },
            { 23386, 39796 },
            { 23387, 39797 },
            { 23388, 36473 },
            { 23389, 36166 },
            { 23391, 38608 },
            { 23395, 35655 },
            { 23396, 36039 },
            { 23397, 39798 },
            { 23398, 35447 },
            { 23401, 39799 },
            { 23403, 37303 },
            { 23408, 39800 },
            { 23409, 39841 },
            { 23411, 39801 },
            { 23413, 39802 },
            { 23416, 39803 },
            { 23418, 39805 },
            { 23424, 39806 },
            { 23427, 39808 },
            { 23429, 37358 },
            { 23431, 35142 },
            { 23432, 36583 },
            { 23433, 35008 },
            { 23435, 37238 },
            { 23436, 35502 },
            { 23437, 36531 },
            { 23439, 36167 },
            { 23445, 37766 },
            { 23447, 36672 },
            { 23448, 35503 },
            { 23449, 37512 },
            { 23450, 37608 },
            { 23451, 34998 },
            { 23452, 35672 },
            { 23453, 38387 },
            { 23455, 36544 },
            { 23458, 35697 },
            { 23459, 37097 },
            { 23460, 36538 },
            { 23461, 38727 },
            { 23462, 39809 },
            { 23470, 35707 },
            { 23472, 36297 },
            { 23475, 35409 },
            { 23476, 35203 },
            { 23477, 36778 },
            { 23478, 35270 },
            { 23480, 39810 },
            { 23481, 38757 },
            { 23487, 36712 },
            { 23488, 64167 },
            { 23490, 36578 },
            { 23491, 39811 },
            { 23492, 35569 },
            { 23493, 37840 },
            { 23494, 38567 },
            { 23495, 39812 },
            { 23497, 39813 },
            { 23500, 38264 },
            { 23504, 39815 },
            { 23506, 35494 },
            { 23507, 35829 },
            { 23508, 39814 },
            { 23512, 64169 },
            { 23515, 35504 },
            { 23517, 36945 },
            { 23518, 39819 },
            { 23519, 36416 },
            { 23521, 35271 },
            { 23522, 39818 },
            { 23524, 39816 },
            { 23525, 39820 },
            { 23526, 39817 },
            { 23527, 37962 },
            { 23528, 40651 },
            { 23529, 36946 },
            { 23531, 39821 },
            { 23532, 64170 },
            { 23534, 38846 },
            { 23536, 39822 },
            { 23539, 39824 },
            { 23541, 37534 },
            { 23542, 39823 },
            { 23544, 37025 },
            { 23546, 36507 },
            { 23550, 37326 },
            { 23551, 36597 },
            { 23553, 38293 },
            { 23554, 37098 },
            { 23556, 36555 },
            { 23557, 39825 },
            { 23558, 36779 },
            { 23559, 39826 },
            { 23560, 39827 },
            { 23561, 35025 },
            { 23562, 37304 },
            { 23563, 36977 },
            { 23565, 39828 },
            { 23566, 37809 },
            { 23567, 36780 },
            { 23569, 36781 },
            { 23571, 39829 },
            { 23574, 37099 },
            { 23578, 36782 },
            { 23582, 64171 },
            { 23584, 39830 },
            { 23586, 39831 },
            { 23588, 38622 },
            { 23592, 39832 },
            { 23597, 35780 },
            { 23601, 36673 },
            { 23608, 39833 },
            { 23609, 39834 },
            { 23610, 36570 },
            { 23611, 36939 },
            { 23612, 37874 },
            { 23613, 36979 },
            { 23614, 38134 },
            { 23615, 37953 },
            { 23616, 35783 },
            { 23617, 39835 },
            { 23621, 35727 },
            { 23622, 39836 },
            { 23624, 35836 },
            { 23626, 37837 },
            { 23627, 35246 },
            { 23629, 36466 },
            { 23630, 39837 },
            { 23631, 39840 },
            { 23632, 39839 },
            { 23633, 35835 },
            { 23635, 39838 },
            { 23637, 37719 },
            { 23646, 37294 },
            { 23648, 37738 },
            { 23649, 36550 },
            { 23652, 37239 },
            { 23653, 38810 },
            { 23660, 39842 },
            { 23662, 39843 },
            { 23663, 37844 },
            { 23665, 36434 },
            { 23670, 39845 },
            { 23673, 39846 },
            { 23692, 39847 },
            { 23696, 35570 },
            { 23697, 39848 },
            { 23700, 39849 },
            { 23713, 35242 },
            { 23718, 64172 },
            { 23720, 37210 },
            { 23721, 35554 },
            { 23723, 39851 },
            { 23724, 38566 },
            { 23729, 37328 },
            { 23731, 35448 },
            { 23734, 39853 },
            { 23735, 39855 },
            { 23736, 35549 },
            { 23738, 64173 },
            { 23739, 39852 },
            { 23740, 39854 },
            { 23742, 39857 },
            { 23749, 39856 },
            { 23751, 39858 },
            { 23769, 39859 },
            { 23776, 37819 },
            { 23777, 35756 },
            { 23784, 35299 },
            { 23785, 39860 },
            { 23786, 39865 },
            { 23789, 39863 },
            { 23791, 38389 },
            { 23792, 38388 },
            { 23797, 64174 },
            { 23798, 37767 },
            { 23802, 39862 },
            { 23803, 36723 },
            { 23805, 39861 },
            { 23815, 37010 },
            { 23819, 39866 },
            { 23822, 36328 },
            { 23825, 39872 },
            { 23828, 39873 },
            { 23829, 39867 },
            { 23830, 35410 },
            { 23831, 39868 },
            { 23832, 39877 },
            { 23833, 39876 },
            { 23834, 39875 },
            { 23835, 39871 },
            { 23839, 39870 },
            { 23842, 39874 },
            { 23847, 64175 },
            { 23849, 38390 },
            { 23874, 64178 },
            { 23883, 39881 },
            { 23884, 39878 },
            { 23886, 39880 },
            { 23888, 38802 },
            { 23890, 39879 },
            { 23891, 64176 },
            { 23900, 39869 },
            { 23913, 37011 },
            { 23916, 39882 },
            { 23917, 64179 },
            { 23919, 36277 },
            { 23923, 39883 },
            { 23926, 39884 },
            { 23938, 39887 },
            { 23940, 39886 },
            { 23943, 39885 },
            { 23947, 37768 },
            { 23948, 39864 },
            { 23952, 39893 },
            { 23965, 39889 },
            { 23970, 39888 },
            { 23980, 39890 },
            { 23982, 39891 },
            { 23991, 39894 },
            { 23992, 64180 },
            { 23993, 64181 },
            { 23994, 38884 },
            { 23996, 39895 },
            { 23997, 39892 },
            { 24009, 39896 },
            { 24012, 35550 },
            { 24013, 39897 },
            { 24016, 64182 },
            { 24018, 39899 },
            { 24019, 39898 },
            { 24022, 39900 },
            { 24027, 39901 },
            { 24029, 37100 },
            { 24030, 36674 },
            { 24033, 36740 },
            { 24035, 37251 },
            { 24037, 36168 },
            { 24038, 36278 },
            { 24039, 36169 },
            { 24040, 35728 },
            { 24043, 39902 },
            { 24046, 36279 },
            { 24049, 36040 },
            { 24050, 39903 },
            { 24051, 38564 },
            { 24052, 37986 },
            { 24053, 39904 },
            { 24055, 36170 },
            { 24059, 35498 },
            { 24061, 37446 },
            { 24062, 35792 },
            { 24066, 36467 },
            { 24067, 38266 },
            { 24070, 38079 },
            { 24075, 39905 },
            { 24076, 35571 },
            { 24081, 39908 },
            { 24086, 37535 },
            { 24089, 39907 },
            { 24090, 39906 },
            { 24091, 39909 },
            { 24093, 37609 },
            { 24101, 36995 },
            { 24107, 36468 },
            { 24109, 37064 },
            { 24111, 37329 },
            { 24112, 35649 },
            { 24115, 37536 },
            { 24118, 39910 },
            { 24119, 39911 },
            { 24120, 36845 },
            { 24125, 38488 },
            { 24128, 39914 },
            { 24131, 39913 },
            { 24132, 39912 },
            { 24133, 38301 },
            { 24135, 39921 },
            { 24140, 38521 },
            { 24142, 39915 },
            { 24148, 39917 },
            { 24149, 38539 },
            { 24151, 39916 },
            { 24159, 39918 },
            { 24161, 38054 },
            { 24162, 39919 },
            { 24163, 38332 },
            { 24164, 39920 },
            { 24178, 35505 },
            { 24179, 38333 },
            { 24180, 37966 },
            { 24181, 39922 },
            { 24182, 39923 },
            { 24184, 36171 },
            { 24185, 35506 },
            { 24186, 39924 },
            { 24187, 36022 },
            { 24188, 38755 },
            { 24189, 38728 },
            { 24190, 35572 },
            { 24191, 39926 },
            { 24193, 37537 },
            { 24195, 36172 },
            { 24196, 36783 },
            { 24199, 38109 },
            { 24202, 36784 },
            { 24207, 36760 },
            { 24213, 37610 },
            { 24214, 38391 },
            { 24215, 37720 },
            { 24218, 36173 },
            { 24220, 38267 },
            { 24224, 39927 },
            { 24230, 37752 },
            { 24231, 36288 },
            { 24235, 36041 },
            { 24237, 37611 },
            { 24245, 35009 },
            { 24246, 36750 },
            { 24247, 36174 },
            { 24248, 38758 },
            { 24257, 39928 },
            { 24258, 39929 },
            { 24259, 38000 },
            { 24264, 39930 },
            { 24265, 38901 },
            { 24266, 38988 },
            { 24271, 39932 },
            { 24272, 39931 },
            { 24275, 35430 },
            { 24278, 40000 },
            { 24282, 40003 },
            { 24283, 40004 },
            { 24285, 40002 },
            { 24287, 38239 },
            { 24288, 36785 },
            { 24289, 40006 },
            { 24290, 40005 },
            { 24291, 40001 },
            { 24296, 40007 },
            { 24297, 40008 },
            { 24300, 40009 },
            { 24304, 40012 },
            { 24305, 40010 },
            { 24307, 40011 },
            { 24308, 40013 },
            { 24310, 35204 },
            { 24311, 37612 },
            { 24312, 40014 },
            { 24314, 35994 },
            { 24315, 35316 },
            { 24316, 37973 },
            { 24318, 40015 },
            { 24319, 37881 },
            { 24321, 38361 },
            { 24323, 40016 },
            { 24324, 38989 },
            { 24329, 40017 },
            { 24330, 38334 },
            { 24331, 40020 },
            { 24332, 39071 },
            { 24333, 39087 },
            { 24335, 36526 },
            { 24336, 37875 },
            { 24337, 40021 },
            { 24339, 35708 },
            { 24340, 37538 },
            { 24341, 35064 },
            { 24342, 40022 },
            { 24343, 38308 },
            { 24344, 36175 },
            { 24347, 37487 },
            { 24351, 37613 },
            { 24353, 64183 },
            { 24357, 38637 },
            { 24358, 36023 },
            { 24359, 36042 },
            { 24361, 40023 },
            { 24365, 40024 },
            { 24367, 40030 },
            { 24369, 36579 },
            { 24372, 64184 },
            { 24373, 37539 },
            { 24375, 35757 },
            { 24376, 40025 },
            { 24380, 38218 },
            { 24382, 37477 },
            { 24385, 40026 },
            { 24389, 64103 },
            { 24392, 40027 },
            { 24394, 35758 },
            { 24396, 40028 },
            { 24398, 40029 },
            { 24401, 40031 },
            { 24403, 37782 },
            { 24406, 40032 },
            { 24407, 40033 },
            { 24409, 40034 },
            { 24412, 40019 },
            { 24413, 40018 },
            { 24417, 40035 },
            { 24418, 35936 },
            { 24422, 38214 },
            { 24423, 64185 },
            { 24425, 36298 },
            { 24426, 38230 },
            { 24427, 37540 },
            { 24428, 38250 },
            { 24429, 40036 },
            { 24432, 36786 },
            { 24433, 35173 },
            { 24435, 40037 },
            { 24439, 40038 },
            { 24441, 38640 },
            { 24444, 38110 },
            { 24447, 40041 },
            { 24448, 35229 },
            { 24449, 37034 },
            { 24450, 40040 },
            { 24451, 40039 },
            { 24452, 35937 },
            { 24453, 37330 },
            { 24455, 40045 },
            { 24456, 40043 },
            { 24458, 40042 },
            { 24459, 38821 },
            { 24460, 36067 },
            { 24464, 36761 },
            { 24465, 40044 },
            { 24466, 37739 },
            { 24467, 36701 },
            { 24471, 37822 },
            { 24472, 40048 },
            { 24473, 40047 },
            { 24478, 40046 },
            { 24480, 40049 },
            { 24481, 36068 },
            { 24488, 40050 },
            { 24489, 38300 },
            { 24490, 36730 },
            { 24493, 40051 },
            { 24494, 38135 },
            { 24499, 37823 },
            { 24500, 37541 },
            { 24503, 64186 },
            { 24505, 37711 },
            { 24508, 40052 },
            { 24509, 35658 },
            { 24515, 36947 },
            { 24517, 38219 },
            { 24524, 35573 },
            { 24525, 37957 },
            { 24534, 40053 },
            { 24535, 36469 },
            { 24536, 38489 },
            { 24537, 38490 },
            { 24540, 35230 },
            { 24541, 40058 },
            { 24542, 64187 },
            { 24544, 37513 },
            { 24548, 40055 },
            { 24555, 35317 },
            { 24560, 40107 },
            { 24561, 40057 },
            { 24565, 37967 },
            { 24568, 40056 },
            { 24571, 40054 },
            { 24573, 36250 },
            { 24575, 40060 },
            { 24590, 40067 },
            { 24591, 40073 },
            { 24592, 40065 },
            { 24594, 37755 },
            { 24597, 40070 },
            { 24598, 38268 },
            { 24601, 40064 },
            { 24603, 40069 },
            { 24604, 38885 },
            { 24605, 36470 },
            { 24608, 37331 },
            { 24609, 40061 },
            { 24613, 35709 },
            { 24614, 40072 },
            { 24615, 37035 },
            { 24616, 35205 },
            { 24617, 40066 },
            { 24618, 35318 },
            { 24619, 40071 },
            { 24623, 35759 },
            { 24625, 40068 },
            { 24634, 40074 },
            { 24641, 40076 },
            { 24642, 40086 },
            { 24643, 40084 },
            { 24646, 40081 },
            { 24650, 40080 },
            { 24651, 38902 },
            { 24653, 40082 },
            { 24656, 35760 },
            { 24658, 36176 },
            { 24661, 36762 },
            { 24665, 40089 },
            { 24666, 40075 },
            { 24669, 64188 },
            { 24671, 40079 },
            { 24672, 40062 },
            { 24674, 35320 },
            { 24675, 40083 },
            { 24676, 40085 },
            { 24677, 37488 },
            { 24680, 36262 },
            { 24681, 35254 },
            { 24682, 40077 },
            { 24683, 40088 },
            { 24684, 40087 },
            { 24685, 35761 },
            { 24687, 37287 },
            { 24688, 35462 },
            { 24693, 35938 },
            { 24695, 40078 },
            { 24705, 40090 },
            { 24707, 40093 },
            { 24708, 40095 },
            { 24709, 64189 },
            { 24713, 36539 },
            { 24714, 64190 },
            { 24715, 40101 },
            { 24716, 37614 },
            { 24717, 40091 },
            { 24722, 40099 },
            { 24724, 35319 },
            { 24726, 40097 },
            { 24727, 40098 },
            { 24730, 40094 },
            { 24731, 40096 },
            { 24735, 36069 },
            { 24736, 38729 },
            { 24739, 35507 },
            { 24742, 35192 },
            { 24743, 40100 },
            { 24745, 37977 },
            { 24746, 34987 },
            { 24754, 38111 },
            { 24755, 40059 },
            { 24756, 40106 },
            { 24757, 40110 },
            { 24758, 38627 },
            { 24760, 40103 },
            { 24764, 37769 },
            { 24765, 40108 },
            { 24773, 36846 },
            { 24774, 40109 },
            { 24775, 37845 },
            { 24785, 39014 },
            { 24787, 40105 },
            { 24789, 64192 },
            { 24792, 40111 },
            { 24794, 36251 },
            { 24796, 37065 },
            { 24798, 64191 },
            { 24799, 35026 },
            { 24800, 40104 },
            { 24801, 40102 },
            { 24803, 37241 },
            { 24807, 40092 },
            { 24808, 36435 },
            { 24816, 37316 },
            { 24817, 40123 },
            { 24818, 64194 },
            { 24819, 37242 },
            { 24820, 40118 },
            { 24822, 40115 },
            { 24823, 40116 },
            { 24825, 36580 },
            { 24826, 40119 },
            { 24827, 40122 },
            { 24832, 40117 },
            { 24833, 36676 },
            { 24835, 40120 },
            { 24838, 40114 },
            { 24840, 38650 },
            { 24841, 38649 },
            { 24845, 40124 },
            { 24846, 40125 },
            { 24847, 35027 },
            { 24849, 64195 },
            { 24853, 40113 },
            { 24858, 35824 },
            { 24859, 34980 },
            { 24863, 35508 },
            { 24864, 64193 },
            { 24865, 40121 },
            { 24871, 40129 },
            { 24872, 40128 },
            { 24876, 40133 },
            { 24880, 64197 },
            { 24884, 40134 },
            { 24887, 64196 },
            { 24892, 40132 },
            { 24893, 40135 },
            { 24894, 40127 },
            { 24895, 40131 },
            { 24898, 40136 },
            { 24900, 40137 },
            { 24903, 40126 },
            { 24904, 36508 },
            { 24906, 40130 },
            { 24907, 37332 },
            { 24908, 36177 },
            { 24909, 40112 },
            { 24910, 36948 },
            { 24915, 40150 },
            { 24917, 38375 },
            { 24920, 40140 },
            { 24921, 40141 },
            { 24922, 40142 },
            { 24925, 40149 },
            { 24927, 40148 },
            { 24930, 38557 },
            { 24931, 35509 },
            { 24933, 40146 },
            { 24935, 35940 },
            { 24936, 35411 },
            { 24939, 40143 },
            { 24942, 38838 },
            { 24943, 40145 },
            { 24944, 35028 },
            { 24945, 40147 },
            { 24947, 40138 },
            { 24948, 40144 },
            { 24949, 40151 },
            { 24950, 35939 },
            { 24951, 40139 },
            { 24958, 38780 },
            { 24962, 38730 },
            { 24967, 40154 },
            { 24970, 40158 },
            { 24974, 37278 },
            { 24976, 38903 },
            { 24977, 40159 },
            { 24980, 40156 },
            { 24982, 40153 },
            { 24984, 64198 },
            { 24985, 40152 },
            { 24986, 40157 },
            { 24996, 38318 },
            { 24999, 37810 },
            { 25001, 35941 },
            { 25003, 40160 },
            { 25004, 40155 },
            { 25006, 40161 },
            { 25010, 35995 },
            { 25014, 35247 },
            { 25018, 40169 },
            { 25022, 35510 },
            { 25027, 40167 },
            { 25030, 40168 },
            { 25031, 36263 },
            { 25032, 40166 },
            { 25033, 40164 },
            { 25034, 40163 },
            { 25035, 40170 },
            { 25036, 40162 },
            { 25037, 40172 },
            { 25040, 35321 },
            { 25059, 40174 },
            { 25062, 40173 },
            { 25074, 37542 },
            { 25076, 40177 },
            { 25078, 40175 },
            { 25079, 40165 },
            { 25080, 35996 },
            { 25082, 40176 },
            { 25084, 40180 },
            { 25085, 40179 },
            { 25086, 40181 },
            { 25087, 40178 },
            { 25088, 40182 },
            { 25096, 40183 },
            { 25097, 40184 },
            { 25098, 38376 },
            { 25100, 40186 },
            { 25101, 40185 },
            { 25102, 36702 },
            { 25104, 37036 },
            { 25105, 35300 },
            { 25106, 35322 },
            { 25107, 64199 },
            { 25108, 40187 },
            { 25110, 35005 },
            { 25114, 37066 },
            { 25115, 40188 },
            { 25117, 59073 },
            { 25118, 40256 },
            { 25119, 35969 },
            { 25121, 40257 },
            { 25126, 37101 },
            { 25130, 40258 },
            { 25134, 40259 },
            { 25135, 35673 },
            { 25136, 40260 },
            { 25138, 40261 },
            { 25139, 40262 },
            { 25140, 37333 },
            { 25144, 36043 },
            { 25147, 38623 },
            { 25151, 38491 },
            { 25152, 36746 },
            { 25153, 40263 },
            { 25159, 37102 },
            { 25160, 59323 },
            { 25161, 38112 },
            { 25163, 36584 },
            { 25165, 36299 },
            { 25166, 40264 },
            { 25171, 37317 },
            { 25173, 38309 },
            { 25176, 37359 },
            { 25179, 40267 },
            { 25182, 40265 },
            { 25184, 40268 },
            { 25187, 40266 },
            { 25192, 40269 },
            { 25198, 38319 },
            { 25201, 34997 },
            { 25206, 38269 },
            { 25209, 38113 },
            { 25212, 40270 },
            { 25214, 40273 },
            { 25215, 36787 },
            { 25216, 35674 },
            { 25218, 40271 },
            { 25219, 40278 },
            { 25220, 36788 },
            { 25225, 40272 },
            { 25226, 37987 },
            { 25233, 38781 },
            { 25234, 40274 },
            { 25235, 40275 },
            { 25236, 40279 },
            { 25237, 37770 },
            { 25238, 40276 },
            { 25239, 36178 },
            { 25240, 37084 },
            { 25243, 40293 },
            { 25244, 38066 },
            { 25246, 37360 },
            { 25254, 64200 },
            { 25259, 38114 },
            { 25260, 40363 },
            { 25265, 38392 },
            { 25269, 37615 },
            { 25273, 38549 },
            { 25275, 40282 },
            { 25276, 35231 },
            { 25277, 37514 },
            { 25282, 40291 },
            { 25285, 37459 },
            { 25286, 40285 },
            { 25287, 40292 },
            { 25288, 40287 },
            { 25289, 40294 },
            { 25290, 40290 },
            { 25292, 40289 },
            { 25293, 38031 },
            { 25295, 40283 },
            { 25296, 35323 },
            { 25297, 40281 },
            { 25298, 35729 },
            { 25299, 37361 },
            { 25300, 40277 },
            { 25303, 40280 },
            { 25304, 36179 },
            { 25305, 37081 },
            { 25307, 36789 },
            { 25308, 40288 },
            { 25309, 38001 },
            { 25312, 35730 },
            { 25313, 35431 },
            { 25324, 35463 },
            { 25325, 36928 },
            { 25326, 40296 },
            { 25327, 40301 },
            { 25329, 40297 },
            { 25331, 35997 },
            { 25333, 40302 },
            { 25334, 36417 },
            { 25335, 36233 },
            { 25342, 36677 },
            { 25343, 40284 },
            { 25345, 36509 },
            { 25346, 40299 },
            { 25351, 36471 },
            { 25352, 40300 },
            { 25353, 35010 },
            { 25356, 40295 },
            { 25361, 37543 },
            { 25369, 35731 },
            { 25375, 35762 },
            { 25383, 40298 },
            { 25384, 34981 },
            { 25387, 36289 },
            { 25391, 36949 },
            { 25402, 37616 },
            { 25405, 38098 },
            { 25406, 40304 },
            { 25407, 37245 },
            { 25417, 37288 },
            { 25420, 36426 },
            { 25421, 40305 },
            { 25423, 40307 },
            { 25424, 40303 },
            { 25429, 38367 },
            { 25431, 37563 },
            { 25436, 37243 },
            { 25447, 38393 },
            { 25448, 36556 },
            { 25449, 40320 },
            { 25451, 40318 },
            { 25454, 37016 },
            { 25458, 35998 },
            { 25462, 40312 },
            { 25463, 36791 },
            { 25466, 37862 },
            { 25467, 37968 },
            { 25472, 40310 },
            { 25475, 37244 },
            { 25480, 36598 },
            { 25481, 40315 },
            { 25484, 36790 },
            { 25486, 40309 },
            { 25487, 40314 },
            { 25490, 38002 },
            { 25494, 40308 },
            { 25496, 35904 },
            { 25499, 35452 },
            { 25503, 40316 },
            { 25504, 38825 },
            { 25505, 36300 },
            { 25506, 37460 },
            { 25507, 40313 },
            { 25509, 37082 },
            { 25511, 36180 },
            { 25512, 36996 },
            { 25513, 35206 },
            { 25514, 37211 },
            { 25515, 40311 },
            { 25516, 35684 },
            { 25522, 35942 },
            { 25524, 37581 },
            { 25525, 40317 },
            { 25531, 37246 },
            { 25534, 40321 },
            { 25536, 40323 },
            { 25539, 37301 },
            { 25540, 40329 },
            { 25542, 40324 },
            { 25545, 40326 },
            { 25551, 38240 },
            { 25552, 37617 },
            { 25554, 40327 },
            { 25558, 38731 },
            { 25562, 38759 },
            { 25563, 35511 },
            { 25569, 34988 },
            { 25571, 40325 },
            { 25577, 40322 },
            { 25582, 35574 },
            { 25588, 35207 },
            { 25589, 64201 },
            { 25590, 40328 },
            { 25594, 38760 },
            { 25606, 40332 },
            { 25613, 37305 },
            { 25615, 40339 },
            { 25619, 40333 },
            { 25622, 40330 },
            { 25623, 40337 },
            { 25628, 40306 },
            { 25638, 40334 },
            { 25640, 40338 },
            { 25644, 38080 },
            { 25645, 37771 },
            { 25652, 40331 },
            { 25654, 40335 },
            { 25658, 35943 },
            { 25662, 36335 },
            { 25666, 37083 },
            { 25678, 40343 },
            { 25688, 37701 },
            { 25696, 64202 },
            { 25703, 40340 },
            { 25705, 38528 },
            { 25711, 40341 },
            { 25718, 40342 },
            { 25720, 38604 },
            { 25722, 37024 },
            { 25731, 35970 },
            { 25736, 40349 },
            { 25746, 36436 },
            { 25747, 40346 },
            { 25749, 40345 },
            { 25754, 37969 },
            { 25757, 64203 },
            { 25758, 37811 },
            { 25764, 37712 },
            { 25765, 40347 },
            { 25769, 40348 },
            { 25771, 38287 },
            { 25773, 37988 },
            { 25774, 36418 },
            { 25776, 37103 },
            { 25778, 38511 },
            { 25785, 35432 },
            { 25787, 40355 },
            { 25788, 40350 },
            { 25793, 38761 },
            { 25794, 40357 },
            { 25797, 40353 },
            { 25799, 40354 },
            { 25805, 37248 },
            { 25806, 64204 },
            { 25810, 40352 },
            { 25812, 40286 },
            { 25816, 40356 },
            { 25818, 40351 },
            { 25824, 40361 },
            { 25825, 40362 },
            { 25826, 37702 },
            { 25827, 40364 },
            { 25830, 36419 },
            { 25831, 40359 },
            { 25836, 35675 },
            { 25839, 40365 },
            { 25841, 40358 },
            { 25842, 40369 },
            { 25844, 40368 },
            { 25846, 40367 },
            { 25850, 40370 },
            { 25853, 40372 },
            { 25854, 36847 },
            { 25856, 40371 },
            { 25861, 40375 },
            { 25880, 40373 },
            { 25884, 40374 },
            { 25885, 40336 },
            { 25891, 40377 },
            { 25892, 40376 },
            { 25898, 40344 },
            { 25899, 40378 },
            { 25900, 40366 },
            { 25903, 36472 },
            { 25908, 40379 },
            { 25909, 40380 },
            { 25910, 40382 },
            { 25911, 40381 },
            { 25912, 40383 },
            { 25913, 35324 },
            { 25915, 36181 },
            { 25918, 38394 },
            { 25919, 37037 },
            { 25925, 36044 },
            { 25928, 40385 },
            { 25933, 40388 },
            { 25934, 64205 },
            { 25935, 38257 },
            { 25937, 35710 },
            { 25941, 40387 },
            { 25942, 40386 },
            { 25943, 38003 },
            { 25944, 40389 },
            { 25945, 35763 },
            { 25949, 40391 },
            { 25950, 40390 },
            { 25954, 35512 },
            { 25955, 36437 },
            { 25958, 37846 },
            { 25964, 35944 },
            { 25968, 37012 },
            { 25970, 40392 },
            { 25972, 37038 },
            { 25973, 37703 },
            { 25975, 38270 },
            { 25976, 40393 },
            { 25986, 40394 },
            { 25987, 40395 },
            { 25991, 38326 },
            { 25992, 39804 },
            { 25993, 37060 },
            { 25996, 38251 },
            { 25998, 36310 },
            { 26000, 38115 },
            { 26001, 38081 },
            { 26007, 37740 },
            { 26009, 38847 },
            { 26011, 40397 },
            { 26012, 36558 },
            { 26015, 40398 },
            { 26017, 34996 },
            { 26020, 35794 },
            { 26021, 37067 },
            { 26023, 38272 },
            { 26027, 40399 },
            { 26028, 36449 },
            { 26029, 37478 },
            { 26031, 36474 },
            { 26032, 36950 },
            { 26039, 40400 },
            { 26041, 38395 },
            { 26044, 35223 },
            { 26045, 36475 },
            { 26049, 40403 },
            { 26051, 40401 },
            { 26052, 40404 },
            { 26053, 38839 },
            { 26054, 40402 },
            { 26059, 37113 },
            { 26060, 40405 },
            { 26063, 37296 },
            { 26066, 40406 },
            { 26071, 35576 },
            { 26073, 40408 },
            { 26075, 40407 },
            { 26080, 40409 },
            { 26081, 40410 },
            { 26082, 35577 },
            { 26085, 37882 },
            { 26086, 37461 },
            { 26087, 35724 },
            { 26088, 36476 },
            { 26089, 37249 },
            { 26092, 36731 },
            { 26093, 34990 },
            { 26097, 40411 },
            { 26106, 35232 },
            { 26107, 40415 },
            { 26112, 64206 },
            { 26114, 36182 },
            { 26115, 40414 },
            { 26118, 36265 },
            { 26119, 36792 },
            { 26121, 64209 },
            { 26122, 40413 },
            { 26124, 36793 },
            { 26126, 38590 },
            { 26127, 36264 },
            { 26131, 35029 },
            { 26132, 37068 },
            { 26133, 64207 },
            { 26140, 40420 },
            { 26142, 64211 },
            { 26143, 37039 },
            { 26144, 35174 },
            { 26148, 64212 },
            { 26149, 36724 },
            { 26151, 38534 },
            { 26152, 36336 },
            { 26157, 36794 },
            { 26158, 64210 },
            { 26159, 37029 },
            { 26161, 64099 },
            { 26164, 40419 },
            { 26165, 40417 },
            { 26166, 40418 },
            { 26171, 64208 },
            { 26172, 37515 },
            { 26175, 40517 },
            { 26177, 40424 },
            { 26178, 36510 },
            { 26179, 36183 },
            { 26180, 40422 },
            { 26185, 40423 },
            { 26187, 36951 },
            { 26191, 40421 },
            { 26194, 36430 },
            { 26199, 64214 },
            { 26201, 64215 },
            { 26205, 40426 },
            { 26206, 40425 },
            { 26207, 40430 },
            { 26210, 40431 },
            { 26212, 40427 },
            { 26213, 64213 },
            { 26214, 35393 },
            { 26215, 40428 },
            { 26216, 40429 },
            { 26217, 38099 },
            { 26222, 38273 },
            { 26223, 35945 },
            { 26224, 40432 },
            { 26227, 64217 },
            { 26228, 37040 },
            { 26230, 36795 },
            { 26234, 37489 },
            { 26241, 35781 },
            { 26243, 40433 },
            { 26244, 40437 },
            { 26247, 35273 },
            { 26248, 40434 },
            { 26249, 40436 },
            { 26254, 40435 },
            { 26257, 36747 },
            { 26262, 37479 },
            { 26263, 35011 },
            { 26264, 40438 },
            { 26265, 64218 },
            { 26269, 40439 },
            { 26272, 64219 },
            { 26274, 37544 },
            { 26278, 38895 },
            { 26283, 36450 },
            { 26286, 38377 },
            { 26290, 64220 },
            { 26292, 38492 },
            { 26296, 40513 },
            { 26297, 40441 },
            { 26300, 40444 },
            { 26302, 40443 },
            { 26303, 64221 },
            { 26305, 40440 },
            { 26308, 40512 },
            { 26311, 37852 },
            { 26313, 40442 },
            { 26326, 40514 },
            { 26329, 36748 },
            { 26330, 40515 },
            { 26332, 38762 },
            { 26333, 38040 },
            { 26336, 40516 },
            { 26342, 40518 },
            { 26345, 40519 },
            { 26352, 40520 },
            { 26354, 35784 },
            { 26355, 35175 },
            { 26356, 36184 },
            { 26357, 40521 },
            { 26359, 40522 },
            { 26360, 36753 },
            { 26361, 37250 },
            { 26362, 64222 },
            { 26363, 64102 },
            { 26364, 39382 },
            { 26365, 37213 },
            { 26366, 37212 },
            { 26367, 37334 },
            { 26368, 36293 },
            { 26371, 39152 },
            { 26376, 35982 },
            { 26377, 38732 },
            { 26379, 38396 },
            { 26381, 38302 },
            { 26382, 64223 },
            { 26383, 40523 },
            { 26388, 36337 },
            { 26389, 37565 },
            { 26390, 40524 },
            { 26391, 38990 },
            { 26395, 38493 },
            { 26397, 37545 },
            { 26398, 40525 },
            { 26399, 35578 },
            { 26406, 40526 },
            { 26407, 40527 },
            { 26408, 38616 },
            { 26410, 38562 },
            { 26411, 38550 },
            { 26412, 38523 },
            { 26413, 36420 },
            { 26414, 40529 },
            { 26417, 36585 },
            { 26420, 38512 },
            { 26422, 40531 },
            { 26423, 40534 },
            { 26424, 40533 },
            { 26426, 35575 },
            { 26429, 35712 },
            { 26431, 40530 },
            { 26433, 40532 },
            { 26438, 40535 },
            { 26441, 37017 },
            { 26446, 38811 },
            { 26447, 35015 },
            { 26448, 36318 },
            { 26449, 37306 },
            { 26451, 36571 },
            { 26454, 36849 },
            { 26457, 40538 },
            { 26460, 37741 },
            { 26462, 40536 },
            { 26463, 37289 },
            { 26464, 40537 },
            { 26465, 36848 },
            { 26466, 38619 },
            { 26467, 40539 },
            { 26468, 40540 },
            { 26469, 38792 },
            { 26470, 64225 },
            { 26474, 40545 },
            { 26477, 36185 },
            { 26479, 38004 },
            { 26480, 40542 },
            { 26481, 37772 },
            { 26482, 40412 },
            { 26483, 40416 },
            { 26485, 35694 },
            { 26487, 37990 },
            { 26492, 40544 },
            { 26494, 36796 },
            { 26495, 38082 },
            { 26501, 40550 },
            { 26503, 38136 },
            { 26505, 40541 },
            { 26507, 40547 },
            { 26508, 40546 },
            { 26512, 37069 },
            { 26517, 38541 },
            { 26519, 38865 },
            { 26522, 38535 },
            { 26524, 35274 },
            { 26525, 36477 },
            { 26528, 39015 },
            { 26529, 40549 },
            { 26530, 37013 },
            { 26534, 40548 },
            { 26537, 40543 },
            { 26543, 36045 },
            { 26547, 40555 },
            { 26548, 40553 },
            { 26550, 35275 },
            { 26551, 40551 },
            { 26552, 40557 },
            { 26553, 40563 },
            { 26555, 64226 },
            { 26560, 64228 },
            { 26561, 37318 },
            { 26564, 38335 },
            { 26566, 40565 },
            { 26570, 38209 },
            { 26574, 40564 },
            { 26575, 38032 },
            { 26576, 38494 },
            { 26577, 35513 },
            { 26579, 37109 },
            { 26580, 36703 },
            { 26584, 37585 },
            { 26586, 38733 },
            { 26589, 40560 },
            { 26590, 40559 },
            { 26594, 40561 },
            { 26596, 40558 },
            { 26599, 40566 },
            { 26601, 40556 },
            { 26604, 40554 },
            { 26606, 40562 },
            { 26607, 40552 },
            { 26609, 37516 },
            { 26611, 38646 },
            { 26612, 36548 },
            { 26613, 36338 },
            { 26619, 36280 },
            { 26622, 38543 },
            { 26623, 35424 },
            { 26625, 64229 },
            { 26626, 37580 },
            { 26627, 37832 },
            { 26628, 35176 },
            { 26643, 37104 },
            { 26646, 37042 },
            { 26647, 35913 },
            { 26654, 40568 },
            { 26657, 36186 },
            { 26658, 35484 },
            { 26665, 40570 },
            { 26666, 35476 },
            { 26667, 40577 },
            { 26674, 40573 },
            { 26676, 37105 },
            { 26680, 35434 },
            { 26681, 36266 },
            { 26684, 35433 },
            { 26685, 36301 },
            { 26688, 40571 },
            { 26689, 35973 },
            { 26690, 35946 },
            { 26691, 37773 },
            { 26692, 64230 },
            { 26694, 40569 },
            { 26696, 35012 },
            { 26701, 40572 },
            { 26702, 40574 },
            { 26704, 35787 },
            { 26705, 35915 },
            { 26706, 64227 },
            { 26707, 35514 },
            { 26708, 35690 },
            { 26713, 40578 },
            { 26716, 36343 },
            { 26717, 38545 },
            { 26719, 36438 },
            { 26723, 40579 },
            { 26727, 38223 },
            { 26740, 40591 },
            { 26742, 35249 },
            { 26743, 40580 },
            { 26750, 40597 },
            { 26751, 40581 },
            { 26753, 38848 },
            { 26755, 40588 },
            { 26757, 38014 },
            { 26765, 40596 },
            { 26767, 40583 },
            { 26771, 34994 },
            { 26772, 40585 },
            { 26775, 36187 },
            { 26779, 40587 },
            { 26781, 40586 },
            { 26783, 40582 },
            { 26784, 40593 },
            { 26786, 36797 },
            { 26790, 39659 },
            { 26791, 36070 },
            { 26792, 38812 },
            { 26797, 40584 },
            { 26799, 37618 },
            { 26800, 35394 },
            { 26801, 36267 },
            { 26803, 40576 },
            { 26805, 40592 },
            { 26806, 35457 },
            { 26809, 40590 },
            { 26810, 40594 },
            { 26812, 37774 },
            { 26820, 35580 },
            { 26822, 40624 },
            { 26824, 64100 },
            { 26825, 38599 },
            { 26826, 40599 },
            { 26827, 35579 },
            { 26829, 40606 },
            { 26831, 64231 },
            { 26834, 38495 },
            { 26836, 40607 },
            { 26837, 40609 },
            { 26839, 40613 },
            { 26840, 40601 },
            { 26842, 37449 },
            { 26847, 37775 },
            { 26848, 40617 },
            { 26849, 40604 },
            { 26851, 40614 },
            { 26855, 40608 },
            { 26862, 36952 },
            { 26863, 40618 },
            { 26866, 37041 },
            { 26873, 40616 },
            { 26874, 35515 },
            { 26880, 39023 },
            { 26881, 40598 },
            { 26884, 40612 },
            { 26885, 35030 },
            { 26888, 40600 },
            { 26891, 38584 },
            { 26892, 40605 },
            { 26893, 36929 },
            { 26894, 37573 },
            { 26895, 40595 },
            { 26898, 40611 },
            { 26905, 37018 },
            { 26906, 40621 },
            { 26907, 35473 },
            { 26908, 35999 },
            { 26913, 40623 },
            { 26914, 40602 },
            { 26915, 40622 },
            { 26917, 40615 },
            { 26918, 40603 },
            { 26920, 40619 },
            { 26922, 40620 },
            { 26928, 40637 },
            { 26932, 37836 },
            { 26934, 40610 },
            { 26937, 40633 },
            { 26941, 40635 },
            { 26943, 37590 },
            { 26954, 38763 },
            { 26963, 38294 },
            { 26964, 40630 },
            { 26965, 37320 },
            { 26969, 40636 },
            { 26970, 37214 },
            { 26972, 40627 },
            { 26973, 40640 },
            { 26974, 40639 },
            { 26976, 37869 },
            { 26977, 40638 },
            { 26978, 37864 },
            { 26984, 64233 },
            { 26986, 40642 },
            { 26987, 40629 },
            { 26989, 35782 },
            { 26990, 40632 },
            { 26991, 36732 },
            { 26995, 38016 },
            { 26996, 40634 },
            { 26997, 35785 },
            { 26999, 40626 },
            { 27000, 40628 },
            { 27001, 40625 },
            { 27004, 38991 },
            { 27005, 35449 },
            { 27006, 40631 },
            { 27009, 40641 },
            { 27010, 35412 },
            { 27018, 36325 },
            { 27022, 35196 },
            { 27025, 40658 },
            { 27028, 38992 },
            { 27029, 40661 },
            { 27032, 64235 },
            { 27035, 36953 },
            { 27036, 40660 },
            { 27040, 40659 },
            { 27047, 40656 },
            { 27054, 40644 },
            { 27057, 40673 },
            { 27058, 40643 },
            { 27060, 40662 },
            { 27067, 40654 },
            { 27070, 40649 },
            { 27071, 40646 },
            { 27073, 40647 },
            { 27075, 40655 },
            { 27079, 60064 },
            { 27082, 40652 },
            { 27083, 36188 },
            { 27084, 37574 },
            { 27085, 37252 },
            { 27086, 40650 },
            { 27088, 40645 },
            { 27091, 40648 },
            { 27096, 38764 },
            { 27097, 38538 },
            { 27101, 40653 },
            { 27102, 40663 },
            { 27106, 64236 },
            { 27111, 40671 },
            { 27112, 40664 },
            { 27115, 40677 },
            { 27117, 40675 },
            { 27122, 40670 },
            { 27129, 40669 },
            { 27131, 37582 },
            { 27133, 37253 },
            { 27135, 40667 },
            { 27138, 40665 },
            { 27141, 40672 },
            { 27146, 40678 },
            { 27147, 38131 },
            { 27148, 40684 },
            { 27154, 40679 },
            { 27155, 40682 },
            { 27156, 40676 },
            { 27159, 37524 },
            { 27161, 38231 },
            { 27163, 40666 },
            { 27166, 40674 },
            { 27167, 36798 },
            { 27169, 38605 },
            { 27170, 40694 },
            { 27171, 40681 },
            { 27177, 36000 },
            { 27178, 35233 },
            { 27179, 35454 },
            { 27182, 40657 },
            { 27184, 64237 },
            { 27189, 36799 },
            { 27190, 40686 },
            { 27192, 40693 },
            { 27193, 36599 },
            { 27194, 35474 },
            { 27197, 37453 },
            { 27204, 40683 },
            { 27206, 64239 },
            { 27207, 40688 },
            { 27208, 40692 },
            { 27211, 35764 },
            { 27224, 35691 },
            { 27225, 40690 },
            { 27231, 35648 },
            { 27233, 37833 },
            { 27234, 40689 },
            { 27238, 40691 },
            { 27243, 64238 },
            { 27250, 40685 },
            { 27251, 64240 },
            { 27256, 40687 },
            { 27262, 64241 },
            { 27263, 35456 },
            { 27264, 37480 },
            { 27268, 40698 },
            { 27277, 40696 },
            { 27278, 36071 },
            { 27280, 40695 },
            { 27287, 40768 },
            { 27292, 40567 },
            { 27296, 40697 },
            { 27298, 40699 },
            { 27299, 40700 },
            { 27306, 40779 },
            { 27308, 40775 },
            { 27310, 40589 },
            { 27315, 40774 },
            { 27320, 40773 },
            { 27323, 40770 },
            { 27329, 40680 },
            { 27330, 40772 },
            { 27331, 40771 },
            { 27345, 40777 },
            { 27347, 38981 },
            { 27354, 40780 },
            { 27355, 35833 },
            { 27358, 40776 },
            { 27359, 40778 },
            { 27362, 64242 },
            { 27364, 64243 },
            { 27368, 38053 },
            { 27370, 40781 },
            { 27386, 40785 },
            { 27387, 40782 },
            { 27396, 38803 },
            { 27397, 40783 },
            { 27402, 40668 },
            { 27410, 40786 },
            { 27414, 40787 },
            { 27421, 35156 },
            { 27423, 40789 },
            { 27424, 35975 },
            { 27425, 36511 },
            { 27427, 35795 },
            { 27431, 35234 },
            { 27442, 38782 },
            { 27447, 40791 },
            { 27448, 40790 },
            { 27449, 40793 },
            { 27450, 35676 },
            { 27453, 35796 },
            { 27454, 35516 },
            { 27459, 40796 },
            { 27463, 40795 },
            { 27465, 40797 },
            { 27468, 35276 },
            { 27470, 37462 },
            { 27472, 40798 },
            { 27475, 35517 },
            { 27476, 40800 },
            { 27481, 40799 },
            { 27483, 40801 },
            { 27487, 40802 },
            { 27489, 40803 },
            { 27490, 36478 },
            { 27491, 37043 },
            { 27492, 36255 },
            { 27494, 38288 },
            { 27497, 38368 },
            { 27498, 39011 },
            { 27503, 36501 },
            { 27507, 36302 },
            { 27508, 38896 },
            { 27512, 40804 },
            { 27513, 40805 },
            { 27515, 36480 },
            { 27519, 40806 },
            { 27520, 40807 },
            { 27523, 40809 },
            { 27524, 40808 },
            { 27526, 38519 },
            { 27529, 36733 },
            { 27530, 36586 },
            { 27531, 36451 },
            { 27533, 40810 },
            { 27541, 40812 },
            { 27542, 36930 },
            { 27544, 40811 },
            { 27550, 40813 },
            { 27556, 40814 },
            { 27562, 40815 },
            { 27563, 40816 },
            { 27567, 40817 },
            { 27569, 40819 },
            { 27570, 40818 },
            { 27571, 40820 },
            { 27572, 35235 },
            { 27573, 37481 },
            { 27575, 40821 },
            { 27578, 36421 },
            { 27579, 35435 },
            { 27580, 40822 },
            { 27583, 37729 },
            { 27584, 39626 },
            { 27589, 35650 },
            { 27590, 40823 },
            { 27595, 40824 },
            { 27597, 38378 },
            { 27598, 38536 },
            { 27602, 37829 },
            { 27603, 40825 },
            { 27604, 38116 },
            { 27606, 64244 },
            { 27608, 38137 },
            { 27611, 38609 },
            { 27615, 40826 },
            { 27627, 40828 },
            { 27628, 40827 },
            { 27631, 40830 },
            { 27635, 40829 },
            { 27656, 40833 },
            { 27663, 36481 },
            { 27665, 38575 },
            { 27667, 40834 },
            { 27668, 40835 },
            { 27671, 35651 },
            { 27675, 40836 },
            { 27683, 40838 },
            { 27684, 40837 },
            { 27700, 36997 },
            { 27703, 38232 },
            { 27704, 35177 },
            { 27710, 38083 },
            { 27711, 64245 },
            { 27712, 37619 },
            { 27713, 36704 },
            { 27714, 35713 },
            { 27726, 38084 },
            { 27728, 36524 },
            { 27733, 40840 },
            { 27735, 35518 },
            { 27738, 35224 },
            { 27740, 64246 },
            { 27741, 37872 },
            { 27742, 40839 },
            { 27743, 36189 },
            { 27744, 37490 },
            { 27746, 40841 },
            { 27752, 40849 },
            { 27754, 40842 },
            { 27759, 64248 },
            { 27760, 37311 },
            { 27762, 35714 },
            { 27763, 40850 },
            { 27770, 35976 },
            { 27773, 35652 },
            { 27774, 40848 },
            { 27777, 40846 },
            { 27778, 40843 },
            { 27779, 38784 },
            { 27782, 64247 },
            { 27784, 37566 },
            { 27788, 37847 },
            { 27789, 40844 },
            { 27792, 40852 },
            { 27794, 40851 },
            { 27795, 35906 },
            { 27798, 35243 },
            { 27801, 36281 },
            { 27802, 40845 },
            { 27803, 40847 },
            { 27809, 38518 },
            { 27810, 37362 },
            { 27819, 38551 },
            { 27822, 40860 },
            { 27825, 40861 },
            { 27827, 35277 },
            { 27832, 38310 },
            { 27833, 38651 },
            { 27834, 40863 },
            { 27835, 36513 },
            { 27836, 36800 },
            { 27837, 40856 },
            { 27838, 40862 },
            { 27839, 35208 },
            { 27841, 35765 },
            { 27844, 40853 },
            { 27845, 40858 },
            { 27849, 37106 },
            { 27850, 38033 },
            { 27852, 38117 },
            { 27859, 40855 },
            { 27861, 38464 },
            { 27863, 40857 },
            { 27865, 40866 },
            { 27866, 64249 },
            { 27867, 40864 },
            { 27869, 40859 },
            { 27873, 38465 },
            { 27874, 37991 },
            { 27875, 35715 },
            { 27877, 37700 },
            { 27880, 37517 },
            { 27882, 40867 },
            { 27887, 40865 },
            { 27888, 37335 },
            { 27889, 40854 },
            { 27891, 35178 },
            { 27908, 64250 },
            { 27915, 38765 },
            { 27916, 40878 },
            { 27922, 40877 },
            { 27927, 37108 },
            { 27929, 40874 },
            { 27931, 38796 },
            { 27934, 37812 },
            { 27935, 40868 },
            { 27941, 37571 },
            { 27945, 35179 },
            { 27946, 36190 },
            { 27947, 40871 },
            { 27954, 36678 },
            { 27955, 40876 },
            { 27957, 40875 },
            { 27958, 40870 },
            { 27960, 40873 },
            { 27963, 35464 },
            { 27965, 40872 },
            { 27966, 37992 },
            { 27969, 38828 },
            { 27972, 36850 },
            { 27973, 37107 },
            { 27993, 40884 },
            { 27994, 40882 },
            { 27996, 38252 },
            { 28003, 40879 },
            { 28004, 40881 },
            { 28006, 35161 },
            { 28009, 36191 },
            { 28010, 38993 },
            { 28012, 35420 },
            { 28014, 38274 },
            { 28015, 64252 },
            { 28020, 38785 },
            { 28023, 35395 },
            { 28024, 36954 },
            { 28025, 40883 },
            { 28037, 40888 },
            { 28039, 64251 },
            { 28040, 36801 },
            { 28044, 38735 },
            { 28046, 40885 },
            { 28051, 40880 },
            { 28053, 40886 },
            { 28054, 64320 },
            { 28057, 38876 },
            { 28059, 37779 },
            { 28060, 37824 },
            { 28076, 64321 },
            { 28079, 35413 },
            { 28082, 35188 },
            { 28085, 40892 },
            { 28088, 40895 },
            { 28092, 38849 },
            { 28096, 38788 },
            { 28101, 40902 },
            { 28102, 40896 },
            { 28103, 40893 },
            { 28107, 38866 },
            { 28108, 40899 },
            { 28111, 64322 },
            { 28113, 36713 },
            { 28114, 40901 },
            { 28117, 40906 },
            { 28120, 37777 },
            { 28121, 40904 },
            { 28126, 40898 },
            { 28129, 37463 },
            { 28132, 40905 },
            { 28134, 40894 },
            { 28136, 40900 },
            { 28138, 40907 },
            { 28139, 35066 },
            { 28140, 40897 },
            { 28142, 40908 },
            { 28145, 36955 },
            { 28146, 64324 },
            { 28147, 36734 },
            { 28149, 38307 },
            { 28151, 36268 },
            { 28152, 64323 },
            { 28153, 40889 },
            { 28154, 40903 },
            { 28155, 37721 },
            { 28156, 64325 },
            { 28165, 37044 },
            { 28167, 35465 },
            { 28168, 36303 },
            { 28169, 36802 },
            { 28170, 40891 },
            { 28171, 36705 },
            { 28179, 35947 },
            { 28181, 40890 },
            { 28185, 40912 },
            { 28186, 36749 },
            { 28187, 36024 },
            { 28189, 40927 },
            { 28191, 40921 },
            { 28192, 35732 },
            { 28193, 37742 },
            { 28195, 40916 },
            { 28196, 40925 },
            { 28197, 34989 },
            { 28198, 35153 },
            { 28199, 64328 },
            { 28201, 35255 },
            { 28203, 40918 },
            { 28204, 37290 },
            { 28205, 40909 },
            { 28206, 40911 },
            { 28207, 36192 },
            { 28216, 40928 },
            { 28217, 64326 },
            { 28218, 40923 },
            { 28220, 64329 },
            { 28222, 40915 },
            { 28227, 40922 },
            { 28234, 38569 },
            { 28237, 40920 },
            { 28238, 40924 },
            { 28246, 36046 },
            { 28248, 36803 },
            { 28251, 37464 },
            { 28252, 64327 },
            { 28255, 40914 },
            { 28263, 38734 },
            { 28267, 40917 },
            { 28270, 40910 },
            { 28271, 37778 },
            { 28274, 40913 },
            { 28278, 40919 },
            { 28286, 39024 },
            { 28287, 36540 },
            { 28288, 38558 },
            { 28290, 40929 },
            { 28300, 38060 },
            { 28303, 40941 },
            { 28304, 36025 },
            { 28310, 36736 },
            { 28312, 40931 },
            { 28316, 38829 },
            { 28317, 36193 },
            { 28319, 40944 },
            { 28322, 35052 },
            { 28325, 40942 },
            { 28330, 40930 },
            { 28335, 40936 },
            { 28338, 40938 },
            { 28342, 38766 },
            { 28343, 40933 },
            { 28346, 37709 },
            { 28349, 40935 },
            { 28351, 64330 },
            { 28354, 40943 },
            { 28356, 40937 },
            { 28357, 38597 },
            { 28361, 40932 },
            { 28363, 36512 },
            { 28364, 40956 },
            { 28369, 35466 },
            { 28371, 40934 },
            { 28372, 40939 },
            { 28373, 40940 },
            { 28381, 37354 },
            { 28382, 37336 },
            { 28396, 40948 },
            { 28399, 40954 },
            { 28402, 40952 },
            { 28404, 37704 },
            { 28407, 57410 },
            { 28408, 40949 },
            { 28414, 40950 },
            { 28415, 40926 },
            { 28417, 35737 },
            { 28418, 38233 },
            { 28422, 36541 },
            { 28425, 36247 },
            { 28431, 38994 },
            { 28433, 40946 },
            { 28435, 57409 },
            { 28436, 35209 },
            { 28437, 37254 },
            { 28448, 38041 },
            { 28450, 35519 },
            { 28451, 38904 },
            { 28459, 38559 },
            { 28460, 37584 },
            { 28465, 40953 },
            { 28466, 40955 },
            { 28472, 37201 },
            { 28478, 57408 },
            { 28479, 40951 },
            { 28481, 40945 },
            { 28485, 35521 },
            { 28500, 35977 },
            { 28504, 57422 },
            { 28507, 57417 },
            { 28508, 37110 },
            { 28511, 35459 },
            { 28516, 36737 },
            { 28518, 57426 },
            { 28525, 57419 },
            { 28526, 37546 },
            { 28527, 57416 },
            { 28528, 37591 },
            { 28532, 57451 },
            { 28536, 57413 },
            { 28538, 57412 },
            { 28540, 57421 },
            { 28544, 57415 },
            { 28545, 57414 },
            { 28546, 57420 },
            { 28548, 37023 },
            { 28550, 57411 },
            { 28552, 64331 },
            { 28558, 57423 },
            { 28561, 57424 },
            { 28567, 35520 },
            { 28577, 57429 },
            { 28579, 57428 },
            { 28580, 57430 },
            { 28586, 57433 },
            { 28593, 37730 },
            { 28595, 57427 },
            { 28597, 64332 },
            { 28601, 57431 },
            { 28608, 35971 },
            { 28609, 37367 },
            { 28610, 57425 },
            { 28611, 37978 },
            { 28614, 57432 },
            { 28628, 57437 },
            { 28629, 57435 },
            { 28632, 57438 },
            { 28635, 57441 },
            { 28639, 57434 },
            { 28640, 36234 },
            { 28641, 37959 },
            { 28644, 40887 },
            { 28651, 38804 },
            { 28652, 57436 },
            { 28654, 57440 },
            { 28655, 37363 },
            { 28657, 57439 },
            { 28659, 57418 },
            { 28661, 64333 },
            { 28662, 59529 },
            { 28666, 57444 },
            { 28670, 57448 },
            { 28673, 57446 },
            { 28677, 64334 },
            { 28679, 64335 },
            { 28681, 57442 },
            { 28683, 57443 },
            { 28687, 57447 },
            { 28689, 57445 },
            { 28693, 38253 },
            { 28696, 57453 },
            { 28698, 57450 },
            { 28699, 57449 },
            { 28701, 57452 },
            { 28702, 37842 },
            { 28703, 57454 },
            { 28710, 37525 },
            { 28711, 37355 },
            { 28712, 64336 },
            { 28716, 37027 },
            { 28720, 57455 },
            { 28722, 57457 },
            { 28734, 57456 },
            { 28748, 40947 },
            { 28753, 57458 },
            { 28760, 37861 },
            { 28771, 57459 },
            { 28779, 35278 },
            { 28783, 37780 },
            { 28784, 35396 },
            { 28792, 35716 },
            { 28796, 36572 },
            { 28797, 36304 },
            { 28805, 64337 },
            { 28809, 38982 },
            { 28810, 36998 },
            { 28814, 35210 },
            { 28818, 57461 },
            { 28825, 57460 },
            { 28843, 64338 },
            { 28844, 57464 },
            { 28845, 37465 },
            { 28846, 57467 },
            { 28847, 57462 },
            { 28851, 57466 },
            { 28856, 57465 },
            { 28857, 37727 },
            { 28858, 35031 },
            { 28859, 64098 },
            { 28872, 38899 },
            { 28875, 57469 },
            { 28879, 35143 },
            { 28889, 57472 },
            { 28893, 57470 },
            { 28895, 57468 },
            { 28913, 57463 },
            { 28921, 38466 },
            { 28925, 57474 },
            { 28932, 64340 },
            { 28937, 57473 },
            { 28943, 64339 },
            { 28948, 35211 },
            { 28953, 57476 },
            { 28954, 38320 },
            { 28956, 57475 },
            { 28961, 38579 },
            { 28966, 36805 },
            { 28982, 37202 },
            { 28988, 36804 },
            { 28998, 64342 },
            { 28999, 64343 },
            { 29001, 38905 },
            { 29004, 57482 },
            { 29006, 37111 },
            { 29013, 57478 },
            { 29014, 57483 },
            { 29017, 35212 },
            { 29020, 64341 },
            { 29026, 57481 },
            { 29028, 38017 },
            { 29029, 57477 },
            { 29030, 57480 },
            { 29031, 36806 },
            { 29033, 38095 },
            { 29036, 57484 },
            { 29038, 36559 },
            { 29053, 37112 },
            { 29060, 57487 },
            { 29064, 57479 },
            { 29066, 35910 },
            { 29071, 57485 },
            { 29076, 38767 },
            { 29077, 57488 },
            { 29081, 60068 },
            { 29087, 36718 },
            { 29096, 57489 },
            { 29100, 57490 },
            { 29105, 37965 },
            { 29113, 57492 },
            { 29118, 57493 },
            { 29121, 64345 },
            { 29123, 37970 },
            { 29128, 37781 },
            { 29129, 57495 },
            { 29134, 57497 },
            { 29136, 38867 },
            { 29138, 57494 },
            { 29140, 57496 },
            { 29141, 35213 },
            { 29143, 57491 },
            { 29151, 39546 },
            { 29152, 57498 },
            { 29157, 37255 },
            { 29158, 36439 },
            { 29159, 57500 },
            { 29164, 57499 },
            { 29165, 36931 },
            { 29166, 39383 },
            { 29173, 57501 },
            { 29177, 57503 },
            { 29179, 57486 },
            { 29180, 57502 },
            { 29182, 64346 },
            { 29183, 57504 },
            { 29190, 38042 },
            { 29197, 57505 },
            { 29200, 57506 },
            { 29211, 57507 },
            { 29224, 57508 },
            { 29226, 37596 },
            { 29228, 57510 },
            { 29229, 57509 },
            { 29232, 57511 },
            { 29234, 57512 },
            { 29237, 36573 },
            { 29238, 38275 },
            { 29242, 38634 },
            { 29243, 57513 },
            { 29244, 57514 },
            { 29245, 37237 },
            { 29246, 36514 },
            { 29247, 57515 },
            { 29248, 57516 },
            { 29254, 57517 },
            { 29255, 38352 },
            { 29256, 38085 },
            { 29259, 57518 },
            { 29260, 38006 },
            { 29266, 37547 },
            { 29272, 57519 },
            { 29273, 35301 },
            { 29275, 35725 },
            { 29277, 38596 },
            { 29279, 38580 },
            { 29281, 35250 },
            { 29282, 38995 },
            { 29287, 38513 },
            { 29289, 38312 },
            { 29298, 37045 },
            { 29300, 57520 },
            { 29305, 37825 },
            { 29309, 36001 },
            { 29310, 57521 },
            { 29312, 36306 },
            { 29313, 57523 },
            { 29314, 57522 },
            { 29319, 57524 },
            { 29330, 57525 },
            { 29334, 57526 },
            { 29344, 35677 },
            { 29346, 57527 },
            { 29351, 57528 },
            { 29356, 36002 },
            { 29359, 38086 },
            { 29361, 64347 },
            { 29362, 57530 },
            { 29366, 36851 },
            { 29369, 57529 },
            { 29374, 64348 },
            { 29378, 35766 },
            { 29379, 57531 },
            { 29380, 57533 },
            { 29382, 57532 },
            { 29390, 57534 },
            { 29392, 36047 },
            { 29394, 57535 },
            { 29399, 35815 },
            { 29401, 37215 },
            { 29403, 36253 },
            { 29408, 57537 },
            { 29409, 57538 },
            { 29410, 57536 },
            { 29417, 36587 },
            { 29420, 37830 },
            { 29421, 35767 },
            { 29431, 57540 },
            { 29432, 37451 },
            { 29433, 57539 },
            { 29436, 38996 },
            { 29437, 38018 },
            { 29450, 57543 },
            { 29462, 57545 },
            { 29463, 57542 },
            { 29467, 38610 },
            { 29468, 57544 },
            { 29469, 57546 },
            { 29471, 38850 },
            { 29476, 64349 },
            { 29477, 57550 },
            { 29481, 57549 },
            { 29482, 37526 },
            { 29483, 37964 },
            { 29486, 36003 },
            { 29487, 57548 },
            { 29492, 57547 },
            { 29494, 38736 },
            { 29495, 38737 },
            { 29502, 57551 },
            { 29503, 35214 },
            { 29508, 36246 },
            { 29509, 36482 },
            { 29518, 57552 },
            { 29519, 57553 },
            { 29527, 57555 },
            { 29539, 36706 },
            { 29544, 57557 },
            { 29546, 57556 },
            { 29552, 57558 },
            { 29554, 35436 },
            { 29557, 57560 },
            { 29559, 64351 },
            { 29560, 57559 },
            { 29562, 57562 },
            { 29563, 57561 },
            { 29572, 36026 },
            { 29575, 38822 },
            { 29577, 35786 },
            { 29579, 35236 },
            { 29590, 35816 },
            { 29609, 35551 },
            { 29618, 38886 },
            { 29619, 57564 },
            { 29627, 57566 },
            { 29629, 64352 },
            { 29632, 57567 },
            { 29634, 35279 },
            { 29640, 57563 },
            { 29641, 64353 },
            { 29642, 36440 },
            { 29645, 37567 },
            { 29646, 57565 },
            { 29650, 64356 },
            { 29654, 64354 },
            { 29662, 57570 },
            { 29664, 36588 },
            { 29667, 64355 },
            { 29669, 57568 },
            { 29674, 35933 },
            { 29677, 38087 },
            { 29678, 57569 },
            { 29681, 57596 },
            { 29685, 64358 },
            { 29688, 57575 },
            { 29694, 36027 },
            { 29699, 35717 },
            { 29701, 57572 },
            { 29702, 38813 },
            { 29703, 64357 },
            { 29705, 38830 },
            { 29730, 37364 },
            { 29733, 57574 },
            { 29734, 64359 },
            { 29737, 64361 },
            { 29738, 64360 },
            { 29742, 64362 },
            { 29746, 57576 },
            { 29747, 38868 },
            { 29748, 35797 },
            { 29749, 38138 },
            { 29750, 37993 },
            { 29754, 57577 },
            { 29759, 57579 },
            { 29761, 57582 },
            { 29781, 57578 },
            { 29785, 57581 },
            { 29786, 36072 },
            { 29787, 35180 },
            { 29788, 57583 },
            { 29790, 37008 },
            { 29791, 57580 },
            { 29792, 38874 },
            { 29794, 64363 },
            { 29795, 57586 },
            { 29796, 60066 },
            { 29801, 57584 },
            { 29802, 57587 },
            { 29807, 57573 },
            { 29808, 57585 },
            { 29811, 36282 },
            { 29814, 57588 },
            { 29822, 57589 },
            { 29827, 38814 },
            { 29833, 64364 },
            { 29835, 57590 },
            { 29854, 57591 },
            { 29855, 64365 },
            { 29858, 57571 },
            { 29863, 57592 },
            { 29872, 35522 },
            { 29885, 36515 },
            { 29898, 57593 },
            { 29903, 57594 },
            { 29908, 57595 },
            { 29916, 35162 },
            { 29920, 57664 },
            { 29922, 38234 },
            { 29923, 57665 },
            { 29926, 35490 },
            { 29927, 57666 },
            { 29929, 57667 },
            { 29934, 57668 },
            { 29936, 57670 },
            { 29937, 57671 },
            { 29938, 57669 },
            { 29942, 38258 },
            { 29943, 57673 },
            { 29944, 57672 },
            { 29953, 64366 },
            { 29955, 57675 },
            { 29956, 57674 },
            { 29957, 57676 },
            { 29964, 57677 },
            { 29965, 57679 },
            { 29966, 57678 },
            { 29969, 36249 },
            { 29971, 57681 },
            { 29973, 57680 },
            { 29976, 35523 },
            { 29978, 36978 },
            { 29980, 37723 },
            { 29982, 57682 },
            { 29983, 37046 },
            { 29987, 36441 },
            { 29989, 35225 },
            { 29990, 57683 },
            { 29992, 38768 },
            { 29995, 38369 },
            { 29996, 57684 },
            { 29999, 64168 },
            { 30000, 37731 },
            { 30001, 38738 },
            { 30002, 36194 },
            { 30003, 36956 },
            { 30007, 37482 },
            { 30008, 39346 },
            { 30010, 37548 },
            { 30011, 35302 },
            { 30012, 57685 },
            { 30020, 57686 },
            { 30022, 57691 },
            { 30025, 57689 },
            { 30026, 57688 },
            { 30027, 40384 },
            { 30028, 35397 },
            { 30029, 57687 },
            { 30031, 35032 },
            { 30033, 38056 },
            { 30036, 38088 },
            { 30041, 38831 },
            { 30042, 57692 },
            { 30043, 57690 },
            { 30044, 37499 },
            { 30045, 37028 },
            { 30048, 38057 },
            { 30050, 38220 },
            { 30052, 57694 },
            { 30053, 38826 },
            { 30054, 35948 },
            { 30055, 57695 },
            { 30057, 57693 },
            { 30058, 38100 },
            { 30059, 57696 },
            { 30061, 57697 },
            { 30063, 64367 },
            { 30064, 35033 },
            { 30067, 36852 },
            { 30068, 57702 },
            { 30070, 57699 },
            { 30071, 37867 },
            { 30072, 57698 },
            { 30079, 35653 },
            { 30082, 57705 },
            { 30086, 57700 },
            { 30087, 57701 },
            { 30089, 57704 },
            { 30090, 57703 },
            { 30091, 38212 },
            { 30094, 37217 },
            { 30095, 37216 },
            { 30097, 35678 },
            { 30100, 57706 },
            { 30106, 57707 },
            { 30109, 57708 },
            { 30115, 57710 },
            { 30117, 57709 },
            { 30123, 35189 },
            { 30129, 57718 },
            { 30130, 38118 },
            { 30131, 57712 },
            { 30133, 57714 },
            { 30136, 57716 },
            { 30137, 36957 },
            { 30140, 57717 },
            { 30141, 57715 },
            { 30142, 36542 },
            { 30146, 57711 },
            { 30147, 57713 },
            { 30149, 38241 },
            { 30151, 36807 },
            { 30154, 57720 },
            { 30157, 57719 },
            { 30162, 57721 },
            { 30164, 36516 },
            { 30165, 36269 },
            { 30168, 37783 },
            { 30169, 57722 },
            { 30171, 37577 },
            { 30174, 57724 },
            { 30178, 38815 },
            { 30179, 57723 },
            { 30185, 37257 },
            { 30192, 57730 },
            { 30194, 57732 },
            { 30195, 57733 },
            { 30196, 37491 },
            { 30202, 57731 },
            { 30204, 57728 },
            { 30206, 57725 },
            { 30207, 57726 },
            { 30209, 57729 },
            { 30217, 57736 },
            { 30219, 57734 },
            { 30221, 57735 },
            { 30239, 57737 },
            { 30240, 57739 },
            { 30241, 57740 },
            { 30242, 57741 },
            { 30244, 57742 },
            { 30247, 57738 },
            { 30256, 57744 },
            { 30260, 57743 },
            { 30267, 57745 },
            { 30274, 38851 },
            { 30278, 57748 },
            { 30279, 57746 },
            { 30280, 57747 },
            { 30284, 35552 },
            { 30290, 38652 },
            { 30294, 38344 },
            { 30296, 57750 },
            { 30300, 57749 },
            { 30305, 57751 },
            { 30306, 57752 },
            { 30311, 57756 },
            { 30312, 57753 },
            { 30313, 57754 },
            { 30314, 57755 },
            { 30316, 57757 },
            { 30320, 57758 },
            { 30322, 57759 },
            { 30326, 57760 },
            { 30328, 57761 },
            { 30330, 38061 },
            { 30331, 37743 },
            { 30332, 57762 },
            { 30333, 38034 },
            { 30334, 38227 },
            { 30336, 57763 },
            { 30338, 64368 },
            { 30339, 57764 },
            { 30340, 37705 },
            { 30342, 35398 },
            { 30343, 36195 },
            { 30344, 57765 },
            { 30347, 57766 },
            { 30350, 57767 },
            { 30352, 36424 },
            { 30355, 57769 },
            { 30358, 57768 },
            { 30361, 57770 },
            { 30362, 57771 },
            { 30363, 64371 },
            { 30364, 64369 },
            { 30366, 64370 },
            { 30374, 64372 },
            { 30382, 38119 },
            { 30384, 57772 },
            { 30388, 57773 },
            { 30391, 60041 },
            { 30392, 57774 },
            { 30393, 57775 },
            { 30394, 57776 },
            { 30399, 36429 },
            { 30402, 57777 },
            { 30403, 38005 },
            { 30406, 38526 },
            { 30408, 35181 },
            { 30410, 35190 },
            { 30413, 57778 },
            { 30418, 57780 },
            { 30422, 57779 },
            { 30423, 37776 },
            { 30427, 37047 },
            { 30428, 40792 },
            { 30430, 57781 },
            { 30431, 38591 },
            { 30433, 57782 },
            { 30435, 35524 },
            { 30436, 38101 },
            { 30437, 57783 },
            { 30439, 57784 },
            { 30442, 57785 },
            { 30446, 38618 },
            { 30450, 38611 },
            { 30452, 37564 },
            { 30456, 37258 },
            { 30459, 57787 },
            { 30462, 36738 },
            { 30465, 36808 },
            { 30468, 57790 },
            { 30471, 57789 },
            { 30472, 57788 },
            { 30473, 38139 },
            { 30475, 35525 },
            { 30476, 36007 },
            { 30491, 57796 },
            { 30494, 57793 },
            { 30495, 36958 },
            { 30496, 38576 },
            { 30500, 57792 },
            { 30501, 57794 },
            { 30502, 57795 },
            { 30505, 57791 },
            { 30519, 57797 },
            { 30520, 57798 },
            { 30522, 37549 },
            { 30524, 35553 },
            { 30528, 37509 },
            { 30534, 64374 },
            { 30535, 57799 },
            { 30554, 57800 },
            { 30555, 57803 },
            { 30561, 36999 },
            { 30563, 37826 },
            { 30565, 57804 },
            { 30566, 38514 },
            { 30568, 57801 },
            { 30571, 57802 },
            { 30585, 57807 },
            { 30590, 57806 },
            { 30591, 57805 },
            { 30603, 57809 },
            { 30606, 57808 },
            { 30609, 57810 },
            { 30622, 57812 },
            { 30624, 57811 },
            { 30629, 38347 },
            { 30636, 36725 },
            { 30637, 38852 },
            { 30640, 57813 },
            { 30643, 37813 },
            { 30646, 57814 },
            { 30649, 57815 },
            { 30651, 57819 },
            { 30652, 57817 },
            { 30653, 57818 },
            { 30655, 57816 },
            { 30663, 57820 },
            { 30669, 57821 },
            { 30679, 57822 },
            { 30682, 57823 },
            { 30683, 38581 },
            { 30684, 57824 },
            { 30690, 38638 },
            { 30691, 57825 },
            { 30693, 37485 },
            { 30695, 38026 },
            { 30697, 35817 },
            { 30701, 37466 },
            { 30702, 57826 },
            { 30703, 35768 },
            { 30707, 37070 },
            { 30716, 57827 },
            { 30722, 36283 },
            { 30732, 57828 },
            { 30738, 57829 },
            { 30740, 36004 },
            { 30741, 36307 },
            { 30752, 57831 },
            { 30753, 64376 },
            { 30757, 37749 },
            { 30758, 36308 },
            { 30759, 35693 },
            { 30770, 38467 },
            { 30772, 37994 },
            { 30778, 37750 },
            { 30783, 36219 },
            { 30789, 57833 },
            { 30798, 64377 },
            { 30813, 36809 },
            { 30820, 64378 },
            { 30827, 38832 },
            { 30828, 36196 },
            { 30831, 36005 },
            { 30834, 38049 },
            { 30836, 57835 },
            { 30842, 64379 },
            { 30844, 57837 },
            { 30849, 36073 },
            { 30854, 57836 },
            { 30855, 37620 },
            { 30860, 57839 },
            { 30861, 35414 },
            { 30862, 57834 },
            { 30865, 38120 },
            { 30867, 35151 },
            { 30869, 36330 },
            { 30871, 39025 },
            { 30874, 57838 },
            { 30883, 57840 },
            { 30887, 38345 },
            { 30889, 37079 },
            { 30890, 57842 },
            { 30895, 57843 },
            { 30901, 57841 },
            { 30906, 35437 },
            { 30908, 57849 },
            { 30910, 57848 },
            { 30913, 36517 },
            { 30917, 57850 },
            { 30918, 57845 },
            { 30922, 57851 },
            { 30923, 57846 },
            { 30928, 38102 },
            { 30929, 57844 },
            { 30932, 57847 },
            { 30938, 57921 },
            { 30951, 57920 },
            { 30952, 38529 },
            { 30956, 57852 },
            { 30959, 35049 },
            { 30964, 57923 },
            { 30973, 57922 },
            { 30977, 36810 },
            { 30983, 57924 },
            { 30990, 37218 },
            { 30993, 57926 },
            { 30994, 57925 },
            { 31001, 57927 },
            { 31014, 57830 },
            { 31018, 57832 },
            { 31019, 57929 },
            { 31020, 57928 },
            { 31024, 64380 },
            { 31034, 36518 },
            { 31036, 38887 },
            { 31038, 36560 },
            { 31040, 57930 },
            { 31041, 35926 },
            { 31047, 35679 },
            { 31048, 35654 },
            { 31049, 36483 },
            { 31056, 38739 },
            { 31059, 57936 },
            { 31061, 57935 },
            { 31062, 37219 },
            { 31063, 57932 },
            { 31066, 57934 },
            { 31069, 36714 },
            { 31070, 36959 },
            { 31071, 57933 },
            { 31072, 57931 },
            { 31074, 37961 },
            { 31077, 36811 },
            { 31080, 38235 },
            { 31085, 36309 },
            { 31095, 37784 },
            { 31098, 57937 },
            { 31103, 57938 },
            { 31104, 57960 },
            { 31105, 35798 },
            { 31108, 39004 },
            { 31109, 37204 },
            { 31114, 57939 },
            { 31117, 35280 },
            { 31118, 37621 },
            { 31119, 38303 },
            { 31124, 64385 },
            { 31131, 64387 },
            { 31133, 57940 },
            { 31142, 35738 },
            { 31143, 57941 },
            { 31146, 57943 },
            { 31150, 57944 },
            { 31152, 37960 },
            { 31155, 57945 },
            { 31161, 57946 },
            { 31162, 57947 },
            { 31165, 35799 },
            { 31166, 35281 },
            { 31167, 37827 },
            { 31168, 36679 },
            { 31169, 36484 },
            { 31177, 57948 },
            { 31179, 36680 },
            { 31185, 35272 },
            { 31186, 38242 },
            { 31189, 57949 },
            { 31192, 38121 },
            { 31199, 37220 },
            { 31201, 57952 },
            { 31203, 57953 },
            { 31204, 38025 },
            { 31206, 36960 },
            { 31207, 57950 },
            { 31209, 37505 },
            { 31212, 57951 },
            { 31216, 36812 },
            { 31227, 35034 },
            { 31232, 35656 },
            { 31240, 57954 },
            { 31243, 37622 },
            { 31245, 57955 },
            { 31246, 37061 },
            { 31252, 38571 },
            { 31255, 38210 },
            { 31256, 57956 },
            { 31257, 57957 },
            { 31258, 37492 },
            { 31260, 38853 },
            { 31263, 57959 },
            { 31264, 57958 },
            { 31278, 36589 },
            { 31281, 57961 },
            { 31282, 35054 },
            { 31287, 57964 },
            { 31291, 57962 },
            { 31292, 35282 },
            { 31293, 35949 },
            { 31294, 57963 },
            { 31295, 36197 },
            { 31296, 36242 },
            { 31298, 38372 },
            { 31299, 57965 },
            { 31302, 38515 },
            { 31305, 57967 },
            { 31309, 37071 },
            { 31310, 35182 },
            { 31311, 35256 },
            { 31312, 34986 },
            { 31319, 57966 },
            { 31329, 57968 },
            { 31330, 57969 },
            { 31331, 36853 },
            { 31337, 57970 },
            { 31339, 35438 },
            { 31344, 57972 },
            { 31348, 35978 },
            { 31350, 35718 },
            { 31353, 57973 },
            { 31354, 35827 },
            { 31357, 57974 },
            { 31359, 37114 },
            { 31361, 37835 },
            { 31363, 37086 },
            { 31364, 36339 },
            { 31368, 57975 },
            { 31378, 37506 },
            { 31379, 37259 },
            { 31381, 57977 },
            { 31382, 57979 },
            { 31383, 57976 },
            { 31384, 57978 },
            { 31391, 35905 },
            { 31401, 57980 },
            { 31402, 35909 },
            { 31406, 35719 },
            { 31407, 38769 },
            { 31408, 57982 },
            { 31414, 57984 },
            { 31418, 35149 },
            { 31423, 57987 },
            { 31427, 35478 },
            { 31428, 57986 },
            { 31429, 57985 },
            { 31431, 57989 },
            { 31432, 57981 },
            { 31434, 57990 },
            { 31435, 38823 },
            { 31437, 57991 },
            { 31439, 57992 },
            { 31441, 64388 },
            { 31442, 39666 },
            { 31443, 57994 },
            { 31445, 57993 },
            { 31449, 57995 },
            { 31450, 57996 },
            { 31452, 38835 },
            { 31453, 57997 },
            { 31455, 59629 },
            { 31456, 36813 },
            { 31457, 57998 },
            { 31458, 57999 },
            { 31459, 36726 },
            { 31461, 37814 },
            { 31462, 58000 },
            { 31463, 64389 },
            { 31466, 37447 },
            { 31467, 64391 },
            { 31469, 58001 },
            { 31471, 37467 },
            { 31472, 58002 },
            { 31478, 35747 },
            { 31480, 39262 },
            { 31481, 37500 },
            { 31482, 36529 },
            { 31487, 35526 },
            { 31490, 58003 },
            { 31492, 58016 },
            { 31494, 58006 },
            { 31496, 35720 },
            { 31498, 58005 },
            { 31499, 58018 },
            { 31503, 58004 },
            { 31505, 36814 },
            { 31512, 58008 },
            { 31513, 58009 },
            { 31515, 37706 },
            { 31518, 58010 },
            { 31520, 35453 },
            { 31525, 36985 },
            { 31526, 38276 },
            { 31528, 58012 },
            { 31532, 37350 },
            { 31539, 58007 },
            { 31541, 58011 },
            { 31542, 58013 },
            { 31545, 36345 },
            { 31557, 58020 },
            { 31558, 38221 },
            { 31560, 38052 },
            { 31561, 37785 },
            { 31563, 35800 },
            { 31564, 58019 },
            { 31565, 58017 },
            { 31567, 38067 },
            { 31568, 58014 },
            { 31569, 37501 },
            { 31570, 37787 },
            { 31572, 37786 },
            { 31574, 36340 },
            { 31581, 58038 },
            { 31589, 58022 },
            { 31591, 58024 },
            { 31596, 58027 },
            { 31598, 58028 },
            { 31600, 58025 },
            { 31601, 58026 },
            { 31604, 58023 },
            { 31605, 58021 },
            { 31610, 58015 },
            { 31622, 38349 },
            { 31623, 35283 },
            { 31627, 58035 },
            { 31629, 58032 },
            { 31631, 58037 },
            { 31634, 58036 },
            { 31636, 38035 },
            { 31637, 38565 },
            { 31639, 36442 },
            { 31640, 58030 },
            { 31641, 58039 },
            { 31642, 58034 },
            { 31644, 58033 },
            { 31645, 58029 },
            { 31646, 64392 },
            { 31647, 58031 },
            { 31649, 35527 },
            { 31658, 37468 },
            { 31661, 37115 },
            { 31665, 38048 },
            { 31668, 58044 },
            { 31672, 38050 },
            { 31680, 37087 },
            { 31681, 58041 },
            { 31684, 38093 },
            { 31686, 58045 },
            { 31687, 38353 },
            { 31689, 37498 },
            { 31691, 58040 },
            { 31692, 58042 },
            { 31695, 58043 },
            { 31709, 58046 },
            { 31712, 36546 },
            { 31716, 37828 },
            { 31717, 58051 },
            { 31718, 58050 },
            { 31721, 58047 },
            { 31725, 38997 },
            { 31731, 58056 },
            { 31734, 58060 },
            { 31735, 58057 },
            { 31744, 58053 },
            { 31751, 58054 },
            { 31757, 58059 },
            { 31761, 58048 },
            { 31762, 39379 },
            { 31763, 58055 },
            { 31764, 58049 },
            { 31767, 58058 },
            { 31775, 58064 },
            { 31777, 35528 },
            { 31779, 58061 },
            { 31783, 58062 },
            { 31786, 58063 },
            { 31787, 58066 },
            { 31799, 58065 },
            { 31800, 38132 },
            { 31805, 58067 },
            { 31806, 38906 },
            { 31807, 38379 },
            { 31808, 58072 },
            { 31811, 58069 },
            { 31820, 58068 },
            { 31821, 37072 },
            { 31823, 58071 },
            { 31824, 58073 },
            { 31828, 58070 },
            { 31830, 58077 },
            { 31832, 58074 },
            { 31839, 58075 },
            { 31840, 58052 },
            { 31844, 58076 },
            { 31845, 58078 },
            { 31852, 58079 },
            { 31859, 38340 },
            { 31861, 58080 },
            { 31870, 38624 },
            { 31873, 35788 },
            { 31874, 35912 },
            { 31875, 58081 },
            { 31881, 38322 },
            { 31883, 37000 },
            { 31885, 38574 },
            { 31888, 58082 },
            { 31890, 38833 },
            { 31893, 38036 },
            { 31895, 37221 },
            { 31896, 37971 },
            { 31899, 36716 },
            { 31903, 35006 },
            { 31905, 58087 },
            { 31906, 58085 },
            { 31908, 58083 },
            { 31909, 35487 },
            { 31911, 36815 },
            { 31912, 58088 },
            { 31915, 58086 },
            { 31917, 58084 },
            { 31918, 58092 },
            { 31921, 58091 },
            { 31922, 58090 },
            { 31923, 58089 },
            { 31929, 58093 },
            { 31933, 58094 },
            { 31934, 37048 },
            { 31936, 58095 },
            { 31938, 58097 },
            { 31941, 58096 },
            { 31946, 36048 },
            { 31950, 37207 },
            { 31954, 58099 },
            { 31958, 37788 },
            { 31960, 58098 },
            { 31964, 58100 },
            { 31966, 38323 },
            { 31967, 37260 },
            { 31968, 36198 },
            { 31970, 58101 },
            { 31975, 38854 },
            { 31983, 58103 },
            { 31986, 58104 },
            { 31988, 58105 },
            { 31990, 58106 },
            { 31992, 36485 },
            { 31994, 58107 },
            { 31995, 35950 },
            { 31998, 35722 },
            { 32000, 35657 },
            { 32002, 58176 },
            { 32004, 38641 },
            { 32005, 36199 },
            { 32006, 58108 },
            { 32010, 58179 },
            { 32011, 38628 },
            { 32013, 37979 },
            { 32016, 38226 },
            { 32020, 36739 },
            { 32021, 58178 },
            { 32023, 36561 },
            { 32024, 36200 },
            { 32025, 36486 },
            { 32026, 35721 },
            { 32027, 38324 },
            { 32028, 58177 },
            { 32032, 37222 },
            { 32033, 38497 },
            { 32034, 36341 },
            { 32043, 36487 },
            { 32044, 37595 },
            { 32046, 58182 },
            { 32047, 38877 },
            { 32048, 36311 },
            { 32050, 58183 },
            { 32051, 36961 },
            { 32053, 58185 },
            { 32057, 36816 },
            { 32058, 36270 },
            { 32063, 58184 },
            { 32066, 36681 },
            { 32067, 36028 },
            { 32068, 37223 },
            { 32069, 58180 },
            { 32070, 58186 },
            { 32072, 64394 },
            { 32075, 58181 },
            { 32076, 35951 },
            { 32078, 58189 },
            { 32079, 58193 },
            { 32080, 35979 },
            { 32086, 58188 },
            { 32091, 58197 },
            { 32092, 64395 },
            { 32094, 36201 },
            { 32097, 38797 },
            { 32098, 35002 },
            { 32099, 58194 },
            { 32102, 35723 },
            { 32104, 58191 },
            { 32110, 58192 },
            { 32113, 37789 },
            { 32114, 58190 },
            { 32115, 58187 },
            { 32117, 35399 },
            { 32118, 37090 },
            { 32121, 36006 },
            { 32125, 58199 },
            { 32137, 58196 },
            { 32143, 58198 },
            { 32147, 58195 },
            { 32153, 35952 },
            { 32154, 37297 },
            { 32155, 58200 },
            { 32156, 37262 },
            { 32159, 58213 },
            { 32160, 64397 },
            { 32162, 58209 },
            { 32163, 58203 },
            { 32171, 58207 },
            { 32172, 36600 },
            { 32173, 35035 },
            { 32174, 58202 },
            { 32175, 58210 },
            { 32176, 58214 },
            { 32177, 36202 },
            { 32178, 38612 },
            { 32180, 37588 },
            { 32181, 58204 },
            { 32183, 64396 },
            { 32184, 58212 },
            { 32186, 58201 },
            { 32187, 37469 },
            { 32189, 58206 },
            { 32190, 35003 },
            { 32191, 38600 },
            { 32199, 58205 },
            { 32202, 35801 },
            { 32203, 38122 },
            { 32207, 37261 },
            { 32209, 38862 },
            { 32210, 36751 },
            { 32213, 58254 },
            { 32214, 64398 },
            { 32216, 58215 },
            { 32218, 37116 },
            { 32220, 58211 },
            { 32221, 58216 },
            { 32222, 58218 },
            { 32224, 37623 },
            { 32225, 58221 },
            { 32228, 58217 },
            { 32232, 38354 },
            { 32233, 35529 },
            { 32236, 38601 },
            { 32239, 35036 },
            { 32242, 58220 },
            { 32244, 38907 },
            { 32251, 58219 },
            { 32257, 35215 },
            { 32260, 37866 },
            { 32261, 58222 },
            { 32265, 58229 },
            { 32266, 58223 },
            { 32267, 58230 },
            { 32274, 58226 },
            { 32283, 38043 },
            { 32286, 36552 },
            { 32287, 58228 },
            { 32289, 58225 },
            { 32290, 58231 },
            { 32291, 58224 },
            { 32294, 36707 },
            { 32299, 38468 },
            { 32302, 36715 },
            { 32305, 58227 },
            { 32306, 58240 },
            { 32309, 58235 },
            { 32311, 58238 },
            { 32313, 58236 },
            { 32314, 58241 },
            { 32315, 58234 },
            { 32317, 58208 },
            { 32318, 37073 },
            { 32321, 38089 },
            { 32323, 58237 },
            { 32326, 58232 },
            { 32330, 37184 },
            { 32331, 35953 },
            { 32333, 36682 },
            { 32338, 64399 },
            { 32340, 36932 },
            { 32341, 37205 },
            { 32342, 58244 },
            { 32345, 58246 },
            { 32346, 58247 },
            { 32349, 58243 },
            { 32350, 58245 },
            { 32358, 58233 },
            { 32359, 58242 },
            { 32361, 58250 },
            { 32362, 58249 },
            { 32365, 38554 },
            { 32368, 35914 },
            { 32377, 58248 },
            { 32379, 58252 },
            { 32380, 58251 },
            { 32381, 58255 },
            { 32383, 58257 },
            { 32386, 36443 },
            { 32387, 58253 },
            { 32392, 58258 },
            { 32393, 58259 },
            { 32394, 64092 },
            { 32396, 58260 },
            { 32398, 58266 },
            { 32399, 37722 },
            { 32400, 58262 },
            { 32402, 58261 },
            { 32403, 58263 },
            { 32404, 58264 },
            { 32406, 58265 },
            { 32411, 58267 },
            { 32412, 58268 },
            { 32566, 35530 },
            { 32568, 58269 },
            { 32570, 58270 },
            { 32581, 58271 },
            { 32583, 64400 },
            { 32588, 58272 },
            { 32589, 58273 },
            { 32590, 58274 },
            { 32592, 58275 },
            { 32593, 58276 },
            { 32596, 58278 },
            { 32597, 58277 },
            { 32600, 58279 },
            { 32607, 58280 },
            { 32608, 58281 },
            { 32615, 58284 },
            { 32616, 58282 },
            { 32617, 58283 },
            { 32618, 36319 },
            { 32619, 35954 },
            { 32622, 37493 },
            { 32624, 38065 },
            { 32626, 36752 },
            { 32629, 37996 },
            { 32631, 38123 },
            { 32632, 58285 },
            { 32633, 40171 },
            { 32642, 58286 },
            { 32643, 58288 },
            { 32645, 38789 },
            { 32646, 58287 },
            { 32647, 58290 },
            { 32648, 58289 },
            { 32650, 38770 },
            { 32652, 58291 },
            { 32654, 38140 },
            { 32660, 58292 },
            { 32666, 58295 },
            { 32669, 58294 },
            { 32670, 58293 },
            { 32673, 64401 },
            { 32675, 58296 },
            { 32676, 35921 },
            { 32680, 37185 },
            { 32681, 35680 },
            { 32686, 58300 },
            { 32687, 58297 },
            { 32690, 58298 },
            { 32694, 58301 },
            { 32696, 58302 },
            { 32697, 58299 },
            { 32701, 35144 },
            { 32705, 35237 },
            { 32709, 58304 },
            { 32710, 58305 },
            { 32714, 58306 },
            { 32716, 38786 },
            { 32722, 36683 },
            { 32724, 58308 },
            { 32725, 58307 },
            { 32736, 37001 },
            { 32737, 58309 },
            { 32742, 58310 },
            { 32745, 58311 },
            { 32747, 35555 },
            { 32752, 35531 },
            { 32755, 58312 },
            { 32761, 58313 },
            { 32763, 38524 },
            { 32764, 38787 },
            { 32768, 38771 },
            { 32769, 38998 },
            { 32771, 36204 },
            { 32772, 58316 },
            { 32773, 36562 },
            { 32774, 58315 },
            { 32779, 58317 },
            { 32780, 36519 },
            { 32784, 37327 },
            { 32786, 58318 },
            { 32789, 36203 },
            { 32791, 38613 },
            { 32792, 58319 },
            { 32793, 58320 },
            { 32796, 58321 },
            { 32801, 58322 },
            { 32808, 58323 },
            { 32819, 36520 },
            { 32822, 38635 },
            { 32827, 58325 },
            { 32829, 37470 },
            { 32831, 58324 },
            { 32838, 58327 },
            { 32842, 58326 },
            { 32850, 58328 },
            { 32854, 37049 },
            { 32856, 58329 },
            { 32858, 58330 },
            { 32862, 38327 },
            { 32863, 58331 },
            { 32865, 37263 },
            { 32866, 58332 },
            { 32872, 58333 },
            { 32879, 38908 },
            { 32880, 58336 },
            { 32882, 58335 },
            { 32883, 58334 },
            { 32884, 37550 },
            { 32886, 58337 },
            { 32887, 36933 },
            { 32889, 58338 },
            { 32893, 58339 },
            { 32894, 38999 },
            { 32895, 58340 },
            { 32900, 58341 },
            { 32901, 58343 },
            { 32902, 58342 },
            { 32903, 38051 },
            { 32905, 37879 },
            { 32907, 39005 },
            { 32908, 38055 },
            { 32915, 58345 },
            { 32918, 36817 },
            { 32920, 38217 },
            { 32922, 58346 },
            { 32923, 58344 },
            { 32925, 35532 },
            { 32929, 36050 },
            { 32930, 36488 },
            { 32933, 38124 },
            { 32937, 36008 },
            { 32938, 38498 },
            { 32940, 58349 },
            { 32941, 58347 },
            { 32943, 36205 },
            { 32945, 36206 },
            { 32946, 35047 },
            { 32948, 36326 },
            { 32954, 38008 },
            { 32963, 35037 },
            { 32964, 58354 },
            { 32966, 37471 },
            { 32972, 38007 },
            { 32974, 37337 },
            { 32982, 58356 },
            { 32985, 58352 },
            { 32986, 58355 },
            { 32987, 58350 },
            { 32989, 58353 },
            { 32990, 38469 },
            { 32993, 36051 },
            { 32996, 35067 },
            { 32997, 58351 },
            { 33007, 58358 },
            { 33009, 58359 },
            { 33012, 37815 },
            { 33016, 35769 },
            { 33020, 58437 },
            { 33021, 37980 },
            { 33026, 36489 },
            { 33029, 35770 },
            { 33030, 37062 },
            { 33031, 39013 },
            { 33032, 38572 },
            { 33033, 58357 },
            { 33034, 37074 },
            { 33050, 35698 },
            { 33051, 58360 },
            { 33059, 58362 },
            { 33065, 58361 },
            { 33071, 58363 },
            { 33073, 37445 },
            { 33075, 37981 },
            { 33081, 37551 },
            { 33086, 58434 },
            { 33094, 58433 },
            { 33099, 58364 },
            { 33102, 36980 },
            { 33104, 38277 },
            { 33105, 58436 },
            { 33107, 58435 },
            { 33108, 36207 },
            { 33109, 39026 },
            { 33119, 58452 },
            { 33125, 58440 },
            { 33126, 58441 },
            { 33131, 36590 },
            { 33134, 58439 },
            { 33136, 36248 },
            { 33137, 58438 },
            { 33140, 58442 },
            { 33144, 37552 },
            { 33145, 38304 },
            { 33146, 37186 },
            { 33151, 37338 },
            { 33152, 58446 },
            { 33154, 58447 },
            { 33155, 58443 },
            { 33160, 58444 },
            { 33162, 58445 },
            { 33167, 36208 },
            { 33171, 58453 },
            { 33173, 58449 },
            { 33178, 38278 },
            { 33180, 38540 },
            { 33181, 38215 },
            { 33184, 58448 },
            { 33187, 58451 },
            { 33188, 58450 },
            { 33192, 38499 },
            { 33193, 58454 },
            { 33200, 58455 },
            { 33203, 37206 },
            { 33205, 58456 },
            { 33208, 58458 },
            { 33210, 58462 },
            { 33213, 58459 },
            { 33214, 58457 },
            { 33215, 37982 },
            { 33216, 58460 },
            { 33218, 58461 },
            { 33222, 35248 },
            { 33224, 58468 },
            { 33225, 58463 },
            { 33229, 58464 },
            { 33233, 58465 },
            { 33235, 37279 },
            { 33240, 58467 },
            { 33241, 58466 },
            { 33242, 58469 },
            { 33247, 58470 },
            { 33248, 58471 },
            { 33251, 36962 },
            { 33253, 35303 },
            { 33255, 58472 },
            { 33256, 38869 },
            { 33258, 36521 },
            { 33261, 36684 },
            { 33267, 36490 },
            { 33268, 37494 },
            { 33274, 58473 },
            { 33275, 58474 },
            { 33276, 35152 },
            { 33278, 58475 },
            { 33281, 58476 },
            { 33282, 58477 },
            { 33285, 58478 },
            { 33287, 58479 },
            { 33288, 35771 },
            { 33289, 40360 },
            { 33290, 58480 },
            { 33292, 37091 },
            { 33293, 58481 },
            { 33294, 36553 },
            { 33296, 58482 },
            { 33298, 39086 },
            { 33302, 58483 },
            { 33303, 38364 },
            { 33304, 35546 },
            { 33307, 37187 },
            { 33308, 36727 },
            { 33310, 38289 },
            { 33311, 36685 },
            { 33321, 58484 },
            { 33322, 36209 },
            { 33323, 58485 },
            { 33324, 38090 },
            { 33326, 58500 },
            { 33331, 58487 },
            { 33333, 37319 },
            { 33334, 38037 },
            { 33335, 36029 },
            { 33336, 58486 },
            { 33337, 37188 },
            { 33344, 58488 },
            { 33351, 37624 },
            { 33368, 58490 },
            { 33369, 58489 },
            { 33370, 58492 },
            { 33373, 58491 },
            { 33375, 58493 },
            { 33378, 58496 },
            { 33380, 58494 },
            { 33382, 35533 },
            { 33384, 58497 },
            { 33386, 58498 },
            { 33387, 58499 },
            { 33390, 36271 },
            { 33391, 38855 },
            { 33393, 58501 },
            { 33394, 36934 },
            { 33398, 35216 },
            { 33399, 58502 },
            { 33400, 58503 },
            { 33406, 58504 },
            { 33419, 35056 },
            { 33421, 58505 },
            { 33426, 58506 },
            { 33433, 38279 },
            { 33437, 36549 },
            { 33439, 58508 },
            { 33445, 35400 },
            { 33446, 34992 },
            { 33451, 58507 },
            { 33452, 58510 },
            { 33453, 37997 },
            { 33455, 36963 },
            { 33457, 35284 },
            { 33459, 38470 },
            { 33464, 35964 },
            { 33465, 35802 },
            { 33467, 58509 },
            { 33469, 35304 },
            { 33477, 35489 },
            { 33489, 35217 },
            { 33490, 58514 },
            { 33491, 38888 },
            { 33492, 37339 },
            { 33495, 38243 },
            { 33497, 58526 },
            { 33499, 35285 },
            { 33500, 58524 },
            { 33502, 58522 },
            { 33503, 58513 },
            { 33505, 58511 },
            { 33507, 58512 },
            { 33509, 36577 },
            { 33510, 35818 },
            { 33511, 37527 },
            { 33515, 37839 },
            { 33521, 35184 },
            { 33523, 58516 },
            { 33524, 58515 },
            { 33529, 58521 },
            { 33530, 58517 },
            { 33531, 58520 },
            { 33537, 64403 },
            { 33538, 38606 },
            { 33539, 58519 },
            { 33540, 35286 },
            { 33541, 35485 },
            { 33542, 58523 },
            { 33545, 58525 },
            { 33550, 35955 },
            { 33558, 58529 },
            { 33559, 58538 },
            { 33560, 58539 },
            { 33564, 34985 },
            { 33571, 58546 },
            { 33576, 35055 },
            { 33579, 58537 },
            { 33583, 58536 },
            { 33585, 58531 },
            { 33586, 58530 },
            { 33588, 58528 },
            { 33589, 58527 },
            { 33590, 37507 },
            { 33592, 37369 },
            { 33593, 58533 },
            { 33600, 58532 },
            { 33605, 58535 },
            { 33609, 37264 },
            { 33610, 35956 },
            { 33615, 35168 },
            { 33616, 58534 },
            { 33618, 36210 },
            { 33624, 37265 },
            { 33634, 64404 },
            { 33651, 58552 },
            { 33653, 58553 },
            { 33655, 35287 },
            { 33659, 35244 },
            { 33660, 58550 },
            { 33663, 64405 },
            { 33669, 58540 },
            { 33671, 58548 },
            { 33673, 58555 },
            { 33674, 58549 },
            { 33678, 58547 },
            { 33683, 58518 },
            { 33686, 58545 },
            { 33690, 58541 },
            { 33694, 35534 },
            { 33695, 58543 },
            { 33696, 58554 },
            { 33698, 58544 },
            { 33704, 58556 },
            { 33706, 58542 },
            { 33707, 38044 },
            { 33713, 38793 },
            { 33717, 58551 },
            { 33725, 58573 },
            { 33729, 58565 },
            { 33733, 37019 },
            { 33735, 64406 },
            { 33738, 35685 },
            { 33740, 35803 },
            { 33742, 58560 },
            { 33747, 35289 },
            { 33750, 36818 },
            { 33752, 58563 },
            { 33756, 36312 },
            { 33759, 37744 },
            { 33760, 58568 },
            { 33769, 38380 },
            { 33771, 58559 },
            { 33775, 35288 },
            { 33776, 36052 },
            { 33777, 38216 },
            { 33778, 58569 },
            { 33780, 58557 },
            { 33782, 64407 },
            { 33783, 58566 },
            { 33787, 58576 },
            { 33789, 58561 },
            { 33795, 58562 },
            { 33796, 37816 },
            { 33799, 58567 },
            { 33803, 58564 },
            { 33804, 38471 },
            { 33805, 58570 },
            { 33806, 35038 },
            { 33811, 58558 },
            { 33824, 58572 },
            { 33826, 58571 },
            { 33833, 38027 },
            { 33834, 58578 },
            { 33836, 58589 },
            { 33841, 35486 },
            { 33845, 58592 },
            { 33848, 58574 },
            { 33852, 58579 },
            { 33853, 38798 },
            { 33862, 58588 },
            { 33864, 64408 },
            { 33865, 38772 },
            { 33870, 38824 },
            { 33879, 37528 },
            { 33883, 35467 },
            { 33889, 38290 },
            { 33890, 58594 },
            { 33891, 37791 },
            { 33894, 34991 },
            { 33897, 58587 },
            { 33899, 58583 },
            { 33900, 37266 },
            { 33901, 58577 },
            { 33902, 58585 },
            { 33903, 58590 },
            { 33905, 37963 },
            { 33909, 34984 },
            { 33911, 58582 },
            { 33913, 58591 },
            { 33914, 38296 },
            { 33922, 58586 },
            { 33924, 58581 },
            { 33931, 36819 },
            { 33936, 36686 },
            { 33940, 36522 },
            { 33945, 38614 },
            { 33948, 38246 },
            { 33951, 58597 },
            { 33953, 58606 },
            { 33965, 58584 },
            { 33970, 35479 },
            { 33972, 64409 },
            { 33976, 36854 },
            { 33977, 58595 },
            { 33979, 58600 },
            { 33980, 37267 },
            { 33983, 58596 },
            { 33985, 58603 },
            { 33988, 37502 },
            { 33990, 58604 },
            { 33993, 38773 },
            { 33994, 58593 },
            { 33995, 35415 },
            { 33997, 58599 },
            { 34000, 58602 },
            { 34001, 38570 },
            { 34006, 58605 },
            { 34009, 58598 },
            { 34010, 58601 },
            { 34012, 64096 },
            { 34028, 38472 },
            { 34030, 38976 },
            { 34036, 58609 },
            { 34044, 58616 },
            { 34047, 58608 },
            { 34048, 36545 },
            { 34054, 58575 },
            { 34065, 38348 },
            { 34067, 38560 },
            { 34068, 58615 },
            { 34069, 58614 },
            { 34071, 58610 },
            { 34072, 58611 },
            { 34074, 35157 },
            { 34079, 58613 },
            { 34081, 58607 },
            { 34086, 37587 },
            { 34092, 58612 },
            { 34093, 35068 },
            { 34101, 37280 },
            { 34109, 38337 },
            { 34112, 58617 },
            { 34113, 58688 },
            { 34115, 38103 },
            { 34120, 58620 },
            { 34121, 36820 },
            { 34122, 36551 },
            { 34123, 58690 },
            { 34126, 35772 },
            { 34131, 64410 },
            { 34133, 58691 },
            { 34135, 38297 },
            { 34136, 58619 },
            { 34137, 64411 },
            { 34138, 58580 },
            { 34147, 58618 },
            { 34152, 39022 },
            { 34153, 37792 },
            { 34154, 38291 },
            { 34155, 64412 },
            { 34157, 58698 },
            { 34167, 58704 },
            { 34174, 58705 },
            { 34176, 58692 },
            { 34180, 38038 },
            { 34183, 58702 },
            { 34184, 58694 },
            { 34186, 58696 },
            { 34192, 58706 },
            { 34193, 58695 },
            { 34196, 58699 },
            { 34199, 35218 },
            { 34201, 37859 },
            { 34203, 58700 },
            { 34204, 58703 },
            { 34212, 58693 },
            { 34214, 37189 },
            { 34216, 58697 },
            { 34217, 36422 },
            { 34218, 36964 },
            { 34219, 35919 },
            { 34220, 38642 },
            { 34222, 38647 },
            { 34223, 36754 },
            { 34224, 64414 },
            { 34233, 58710 },
            { 34234, 58708 },
            { 34241, 39021 },
            { 34249, 58707 },
            { 34253, 38805 },
            { 34255, 58709 },
            { 34256, 58711 },
            { 34261, 58712 },
            { 34268, 58715 },
            { 34269, 58713 },
            { 34276, 37793 },
            { 34277, 58714 },
            { 34281, 38091 },
            { 34282, 58701 },
            { 34295, 36755 },
            { 34297, 58716 },
            { 34298, 58721 },
            { 34299, 37268 },
            { 34302, 58720 },
            { 34306, 58689 },
            { 34310, 58722 },
            { 34311, 37224 },
            { 34314, 58717 },
            { 34315, 58719 },
            { 34323, 58718 },
            { 34326, 40784 },
            { 34327, 40769 },
            { 34330, 58724 },
            { 34338, 58723 },
            { 34349, 38806 },
            { 34351, 57786 },
            { 34352, 58725 },
            { 34367, 58726 },
            { 34381, 58727 },
            { 34382, 36053 },
            { 34384, 35699 },
            { 34388, 58729 },
            { 34389, 39292 },
            { 34394, 35733 },
            { 34396, 38840 },
            { 34398, 35825 },
            { 34399, 58730 },
            { 34407, 58731 },
            { 34411, 37518 },
            { 34417, 58732 },
            { 34425, 37880 },
            { 34427, 35000 },
            { 34442, 35297 },
            { 34443, 58737 },
            { 34444, 58738 },
            { 34451, 58733 },
            { 34453, 36444 },
            { 34467, 58734 },
            { 34468, 37985 },
            { 34473, 58735 },
            { 34474, 58736 },
            { 34475, 58746 },
            { 34479, 58740 },
            { 34480, 58743 },
            { 34486, 58739 },
            { 34500, 58741 },
            { 34502, 58742 },
            { 34503, 36566 },
            { 34505, 58744 },
            { 34507, 37472 },
            { 34509, 35957 },
            { 34510, 35425 },
            { 34516, 58747 },
            { 34521, 35422 },
            { 34523, 58753 },
            { 34526, 58748 },
            { 34527, 58752 },
            { 34532, 38072 },
            { 34537, 58749 },
            { 34540, 58750 },
            { 34541, 38247 },
            { 34542, 38104 },
            { 34543, 58754 },
            { 34552, 37371 },
            { 34553, 58764 },
            { 34555, 58760 },
            { 34558, 35305 },
            { 34560, 58758 },
            { 34562, 38473 },
            { 34563, 58759 },
            { 34566, 58756 },
            { 34568, 58757 },
            { 34569, 58762 },
            { 34570, 58765 },
            { 34573, 58763 },
            { 34577, 58761 },
            { 34578, 58755 },
            { 34584, 37495 },
            { 34586, 58772 },
            { 34588, 38568 },
            { 34597, 58770 },
            { 34601, 58771 },
            { 34612, 58766 },
            { 34615, 58768 },
            { 34619, 58769 },
            { 34623, 58767 },
            { 34633, 37092 },
            { 34635, 39000 },
            { 34636, 58776 },
            { 34638, 58777 },
            { 34643, 58783 },
            { 34645, 36937 },
            { 34647, 58779 },
            { 34649, 58782 },
            { 34655, 58774 },
            { 34656, 58773 },
            { 34659, 58784 },
            { 34662, 35290 },
            { 34664, 58780 },
            { 34666, 58785 },
            { 34670, 58781 },
            { 34676, 58778 },
            { 34678, 37553 },
            { 34680, 58775 },
            { 34687, 38024 },
            { 34690, 58789 },
            { 34701, 38746 },
            { 34719, 58788 },
            { 34722, 58787 },
            { 34731, 58796 },
            { 34735, 58790 },
            { 34739, 58798 },
            { 34746, 38790 },
            { 34747, 58801 },
            { 34749, 58792 },
            { 34752, 58793 },
            { 34756, 58797 },
            { 34758, 58800 },
            { 34759, 58799 },
            { 34763, 58791 },
            { 34768, 58794 },
            { 34770, 58811 },
            { 34784, 58804 },
            { 34799, 58802 },
            { 34802, 58803 },
            { 34806, 58808 },
            { 34807, 58809 },
            { 34809, 35401 },
            { 34811, 35681 },
            { 34814, 58807 },
            { 34821, 58786 },
            { 34823, 64417 },
            { 34829, 58806 },
            { 34830, 58810 },
            { 34831, 58805 },
            { 34833, 58812 },
            { 34837, 58814 },
            { 34838, 58813 },
            { 34849, 58816 },
            { 34850, 58815 },
            { 34851, 58745 },
            { 34855, 58820 },
            { 34865, 58817 },
            { 34870, 58818 },
            { 34873, 58819 },
            { 34875, 58821 },
            { 34880, 35980 },
            { 34882, 58823 },
            { 34884, 58822 },
            { 34886, 36687 },
            { 34892, 36211 },
            { 34893, 40869 },
            { 34898, 58824 },
            { 34899, 36720 },
            { 34903, 35416 },
            { 34905, 58825 },
            { 34907, 35185 },
            { 34909, 36821 },
            { 34910, 58826 },
            { 34913, 36212 },
            { 34914, 58827 },
            { 34915, 35039 },
            { 34920, 38236 },
            { 34923, 58828 },
            { 34928, 37002 },
            { 34930, 58835 },
            { 34933, 58832 },
            { 34935, 37519 },
            { 34941, 58833 },
            { 34942, 58830 },
            { 34943, 35804 },
            { 34945, 58829 },
            { 34946, 58836 },
            { 34952, 35925 },
            { 34955, 37340 },
            { 34957, 58842 },
            { 34962, 58838 },
            { 34966, 37299 },
            { 34967, 58837 },
            { 34969, 58840 },
            { 34974, 58831 },
            { 34978, 58841 },
            { 34980, 58843 },
            { 34987, 38125 },
            { 34990, 58839 },
            { 34992, 58844 },
            { 34993, 58846 },
            { 34996, 36049 },
            { 34997, 58834 },
            { 34999, 35007 },
            { 35007, 58845 },
            { 35009, 36313 },
            { 35010, 38900 },
            { 35011, 58847 },
            { 35012, 58848 },
            { 35013, 37269 },
            { 35023, 38816 },
            { 35028, 58849 },
            { 35029, 38740 },
            { 35032, 58850 },
            { 35033, 58851 },
            { 35036, 38370 },
            { 35037, 58852 },
            { 35039, 36286 },
            { 35041, 38817 },
            { 35048, 58857 },
            { 35058, 58858 },
            { 35059, 36822 },
            { 35060, 58856 },
            { 35061, 64418 },
            { 35064, 38791 },
            { 35065, 58853 },
            { 35068, 58855 },
            { 35069, 37051 },
            { 35070, 37022 },
            { 35074, 58854 },
            { 35076, 58859 },
            { 35079, 38305 },
            { 35082, 58861 },
            { 35084, 58860 },
            { 35088, 35468 },
            { 35090, 38474 },
            { 35091, 58862 },
            { 35100, 64093 },
            { 35101, 58874 },
            { 35102, 58864 },
            { 35109, 58865 },
            { 35114, 58866 },
            { 35115, 58867 },
            { 35126, 58871 },
            { 35128, 58872 },
            { 35131, 58870 },
            { 35137, 58868 },
            { 35139, 58863 },
            { 35140, 58869 },
            { 35148, 58873 },
            { 35149, 59573 },
            { 35158, 35238 },
            { 35166, 58876 },
            { 35167, 35805 },
            { 35168, 58875 },
            { 35172, 58945 },
            { 35174, 58944 },
            { 35178, 58947 },
            { 35181, 58946 },
            { 35183, 58948 },
            { 35186, 36688 },
            { 35188, 58949 },
            { 35191, 58950 },
            { 35198, 58951 },
            { 35199, 37052 },
            { 35201, 38774 },
            { 35203, 58952 },
            { 35206, 38306 },
            { 35207, 37989 },
            { 35208, 58953 },
            { 35210, 58954 },
            { 35211, 36009 },
            { 35215, 35659 },
            { 35219, 58955 },
            { 35222, 36491 },
            { 35223, 37984 },
            { 35224, 58956 },
            { 35226, 35439 },
            { 35233, 58957 },
            { 35238, 58959 },
            { 35239, 38807 },
            { 35241, 58958 },
            { 35242, 36965 },
            { 35244, 58960 },
            { 35247, 58961 },
            { 35250, 58962 },
            { 35251, 35535 },
            { 35258, 58963 },
            { 35261, 58964 },
            { 35263, 58965 },
            { 35264, 58966 },
            { 35282, 35440 },
            { 35290, 58967 },
            { 35292, 58968 },
            { 35293, 58969 },
            { 35299, 35312 },
            { 35302, 36935 },
            { 35303, 58970 },
            { 35316, 58971 },
            { 35320, 58972 },
            { 35328, 36030 },
            { 35330, 37625 },
            { 35331, 58973 },
            { 35336, 35958 },
            { 35338, 36981 },
            { 35340, 58976 },
            { 35342, 37794 },
            { 35344, 58975 },
            { 35346, 64419 },
            { 35347, 35920 },
            { 35350, 58974 },
            { 35351, 37365 },
            { 35352, 35660 },
            { 35355, 58977 },
            { 35357, 58978 },
            { 35359, 36823 },
            { 35363, 35981 },
            { 35365, 58979 },
            { 35370, 38475 },
            { 35373, 37085 },
            { 35377, 35734 },
            { 35379, 38643 },
            { 35380, 37225 },
            { 35382, 58980 },
            { 35383, 64420 },
            { 35386, 36966 },
            { 35387, 37520 },
            { 35388, 36824 },
            { 35393, 58981 },
            { 35398, 58984 },
            { 35400, 58985 },
            { 35408, 36284 },
            { 35409, 37312 },
            { 35410, 58983 },
            { 35412, 36825 },
            { 35413, 38237 },
            { 35419, 58982 },
            { 35422, 36492 },
            { 35424, 35186 },
            { 35426, 58989 },
            { 35427, 35959 },
            { 35430, 36494 },
            { 35433, 36493 },
            { 35435, 39020 },
            { 35436, 58988 },
            { 35437, 58987 },
            { 35438, 37190 },
            { 35440, 35692 },
            { 35441, 39010 },
            { 35442, 35417 },
            { 35443, 36826 },
            { 35449, 64421 },
            { 35452, 58986 },
            { 35458, 58991 },
            { 35460, 58992 },
            { 35461, 58990 },
            { 35463, 36054 },
            { 35465, 38751 },
            { 35468, 36495 },
            { 35469, 37958 },
            { 35473, 58995 },
            { 35475, 37054 },
            { 35477, 37473 },
            { 35480, 38741 },
            { 35482, 58998 },
            { 35486, 36074 },
            { 35488, 37053 },
            { 35489, 58994 },
            { 35491, 58999 },
            { 35492, 36075 },
            { 35493, 58996 },
            { 35494, 58997 },
            { 35495, 64422 },
            { 35496, 58993 },
            { 35500, 37088 },
            { 35501, 37831 },
            { 35504, 37454 },
            { 35506, 35291 },
            { 35513, 38126 },
            { 35516, 35682 },
            { 35518, 64423 },
            { 35519, 37554 },
            { 35522, 59002 },
            { 35524, 59000 },
            { 35527, 37483 },
            { 35531, 37055 },
            { 35532, 35536 },
            { 35533, 59001 },
            { 35535, 36986 },
            { 35538, 38856 },
            { 35542, 39007 },
            { 35546, 59003 },
            { 35547, 59015 },
            { 35548, 37555 },
            { 35550, 59014 },
            { 35551, 64424 },
            { 35552, 59011 },
            { 35553, 59019 },
            { 35554, 59012 },
            { 35556, 59008 },
            { 35558, 37626 },
            { 35559, 59006 },
            { 35563, 59004 },
            { 35565, 38720 },
            { 35566, 36496 },
            { 35569, 59009 },
            { 35571, 59005 },
            { 35574, 64426 },
            { 35575, 59013 },
            { 35576, 36756 },
            { 35578, 36031 },
            { 35582, 37368 },
            { 35584, 38500 },
            { 35585, 35193 },
            { 35586, 35040 },
            { 35588, 37795 },
            { 35591, 59017 },
            { 35596, 59016 },
            { 35598, 37860 },
            { 35600, 59021 },
            { 35604, 59010 },
            { 35606, 59020 },
            { 35607, 59022 },
            { 35609, 36010 },
            { 35610, 59018 },
            { 35611, 36213 },
            { 35613, 36563 },
            { 35616, 59023 },
            { 35617, 38775 },
            { 35622, 59026 },
            { 35624, 59029 },
            { 35627, 59027 },
            { 35628, 38228 },
            { 35635, 59024 },
            { 35641, 35806 },
            { 35646, 59028 },
            { 35649, 59030 },
            { 35657, 59034 },
            { 35660, 59031 },
            { 35662, 59033 },
            { 35663, 59032 },
            { 35667, 64427 },
            { 35670, 59035 },
            { 35672, 36527 },
            { 35674, 59037 },
            { 35675, 59036 },
            { 35676, 38280 },
            { 35679, 59039 },
            { 35686, 35960 },
            { 35691, 59038 },
            { 35692, 59040 },
            { 35695, 59041 },
            { 35696, 35683 },
            { 35697, 58303 },
            { 35698, 36855 },
            { 35700, 59042 },
            { 35703, 36076 },
            { 35709, 59043 },
            { 35711, 64428 },
            { 35712, 59044 },
            { 35715, 36445 },
            { 35722, 40396 },
            { 35724, 59045 },
            { 35726, 59046 },
            { 35728, 36689 },
            { 35730, 59047 },
            { 35731, 59048 },
            { 35734, 59049 },
            { 35737, 59050 },
            { 35738, 59051 },
            { 35895, 37450 },
            { 35898, 59052 },
            { 35903, 59054 },
            { 35905, 59053 },
            { 35910, 37796 },
            { 35912, 59055 },
            { 35914, 38476 },
            { 35916, 59056 },
            { 35918, 59057 },
            { 35920, 59058 },
            { 35925, 59059 },
            { 35930, 37848 },
            { 35937, 36827 },
            { 35938, 59060 },
            { 35946, 36235 },
            { 35947, 39084 },
            { 35948, 59061 },
            { 35960, 59062 },
            { 35961, 38238 },
            { 35962, 59063 },
            { 35964, 59071 },
            { 35970, 59064 },
            { 35973, 59066 },
            { 35977, 59065 },
            { 35978, 59067 },
            { 35980, 38501 },
            { 35981, 59068 },
            { 35982, 59069 },
            { 35988, 59070 },
            { 35992, 59072 },
            { 35997, 35404 },
            { 35998, 37605 },
            { 36000, 38281 },
            { 36001, 36320 },
            { 36002, 36214 },
            { 36007, 38254 },
            { 36008, 35293 },
            { 36009, 38092 },
            { 36010, 59075 },
            { 36011, 35537 },
            { 36012, 37075 },
            { 36013, 59074 },
            { 36014, 59079 },
            { 36015, 37529 },
            { 36016, 38625 },
            { 36018, 59077 },
            { 36019, 59078 },
            { 36020, 35661 },
            { 36022, 59080 },
            { 36023, 38019 },
            { 36024, 37341 },
            { 36027, 38127 },
            { 36028, 37724 },
            { 36029, 59076 },
            { 36031, 38502 },
            { 36032, 35306 },
            { 36033, 59082 },
            { 36034, 38983 },
            { 36035, 37568 },
            { 36036, 39012 },
            { 36039, 36497 },
            { 36040, 59081 },
            { 36042, 37295 },
            { 36045, 59098 },
            { 36046, 37191 },
            { 36049, 37878 },
            { 36051, 38255 },
            { 36058, 59085 },
            { 36059, 36446 },
            { 36060, 36498 },
            { 36062, 36828 },
            { 36064, 38021 },
            { 36066, 36011 },
            { 36067, 59084 },
            { 36068, 59083 },
            { 36070, 38282 },
            { 36074, 36543 },
            { 36077, 37745 },
            { 36080, 64429 },
            { 36084, 64430 },
            { 36090, 59087 },
            { 36091, 59088 },
            { 36092, 36215 },
            { 36093, 59086 },
            { 36100, 59089 },
            { 36101, 59090 },
            { 36103, 59092 },
            { 36104, 37281 },
            { 36106, 59091 },
            { 36107, 35556 },
            { 36109, 59094 },
            { 36111, 59093 },
            { 36112, 59095 },
            { 36114, 64431 },
            { 36115, 59097 },
            { 36116, 59099 },
            { 36118, 59100 },
            { 36196, 37076 },
            { 36198, 36557 },
            { 36199, 59101 },
            { 36203, 35441 },
            { 36205, 59102 },
            { 36208, 37270 },
            { 36209, 59103 },
            { 36211, 59104 },
            { 36212, 38283 },
            { 36214, 64432 },
            { 36215, 35662 },
            { 36225, 59105 },
            { 36229, 37556 },
            { 36234, 35194 },
            { 36249, 59106 },
            { 36259, 36591 },
            { 36264, 37014 },
            { 36275, 37291 },
            { 36282, 59109 },
            { 36286, 59108 },
            { 36290, 59107 },
            { 36299, 59115 },
            { 36300, 59113 },
            { 36303, 59110 },
            { 36310, 59112 },
            { 36314, 59111 },
            { 36315, 59114 },
            { 36317, 35735 },
            { 36319, 59118 },
            { 36321, 37077 },
            { 36323, 59119 },
            { 36328, 36055 },
            { 36330, 59116 },
            { 36331, 59117 },
            { 36335, 38984 },
            { 36339, 37557 },
            { 36341, 37192 },
            { 36348, 59120 },
            { 36351, 59123 },
            { 36360, 59121 },
            { 36361, 59122 },
            { 36362, 38776 },
            { 36367, 37797 },
            { 36368, 59126 },
            { 36381, 59124 },
            { 36382, 59125 },
            { 36383, 59127 },
            { 36394, 59208 },
            { 36400, 59130 },
            { 36404, 59131 },
            { 36405, 59129 },
            { 36418, 59128 },
            { 36420, 37627 },
            { 36423, 59200 },
            { 36424, 59204 },
            { 36425, 59201 },
            { 36426, 59132 },
            { 36428, 59202 },
            { 36432, 59203 },
            { 36437, 59210 },
            { 36441, 59205 },
            { 36447, 37078 },
            { 36448, 59207 },
            { 36451, 59209 },
            { 36452, 59206 },
            { 36466, 59212 },
            { 36468, 36690 },
            { 36470, 59211 },
            { 36476, 59213 },
            { 36481, 59214 },
            { 36484, 59217 },
            { 36485, 59216 },
            { 36487, 59215 },
            { 36490, 59219 },
            { 36491, 59218 },
            { 36493, 38644 },
            { 36497, 59221 },
            { 36499, 59220 },
            { 36500, 59222 },
            { 36505, 59223 },
            { 36513, 59225 },
            { 36522, 59224 },
            { 36523, 36967 },
            { 36524, 59226 },
            { 36527, 35819 },
            { 36528, 59227 },
            { 36529, 59229 },
            { 36542, 59230 },
            { 36549, 59231 },
            { 36550, 59228 },
            { 36552, 59232 },
            { 36554, 36564 },
            { 36555, 59233 },
            { 36556, 35663 },
            { 36557, 35922 },
            { 36559, 64434 },
            { 36562, 36012 },
            { 36571, 59234 },
            { 36575, 37870 },
            { 36578, 37725 },
            { 36579, 59235 },
            { 36587, 59238 },
            { 36600, 36530 },
            { 36603, 59237 },
            { 36604, 59236 },
            { 36605, 35961 },
            { 36606, 59239 },
            { 36611, 35442 },
            { 36613, 59241 },
            { 36617, 36314 },
            { 36618, 59240 },
            { 36620, 59249 },
            { 36626, 59243 },
            { 36627, 59245 },
            { 36628, 38371 },
            { 36629, 59242 },
            { 36633, 59244 },
            { 36635, 59248 },
            { 36636, 59246 },
            { 36637, 35664 },
            { 36639, 59247 },
            { 36646, 59250 },
            { 36649, 38009 },
            { 36650, 38870 },
            { 36655, 36691 },
            { 36659, 59251 },
            { 36664, 38721 },
            { 36665, 59253 },
            { 36667, 59252 },
            { 36670, 59256 },
            { 36671, 38752 },
            { 36674, 59255 },
            { 36676, 35469 },
            { 36677, 59254 },
            { 36678, 59259 },
            { 36681, 59258 },
            { 36684, 59257 },
            { 36685, 37713 },
            { 36686, 59260 },
            { 36695, 59261 },
            { 36700, 59262 },
            { 36703, 36236 },
            { 36705, 35908 },
            { 36706, 59264 },
            { 36707, 59265 },
            { 36708, 59266 },
            { 36763, 36968 },
            { 36764, 59267 },
            { 36766, 36523 },
            { 36767, 59268 },
            { 36771, 59269 },
            { 36775, 39327 },
            { 36776, 39326 },
            { 36781, 59270 },
            { 36782, 58256 },
            { 36783, 59271 },
            { 36784, 37443 },
            { 36785, 36938 },
            { 36786, 37983 },
            { 36791, 59272 },
            { 36794, 38355 },
            { 36795, 37586 },
            { 36796, 36254 },
            { 36799, 37448 },
            { 36802, 35145 },
            { 36804, 38552 },
            { 36805, 36982 },
            { 36814, 35965 },
            { 36817, 35807 },
            { 36820, 38356 },
            { 36826, 59273 },
            { 36834, 59275 },
            { 36837, 59274 },
            { 36838, 35294 },
            { 36841, 37876 },
            { 36842, 59276 },
            { 36843, 38039 },
            { 36845, 37714 },
            { 36847, 59277 },
            { 36848, 36721 },
            { 36852, 59279 },
            { 36855, 38592 },
            { 36856, 59294 },
            { 36857, 59281 },
            { 36858, 59282 },
            { 36861, 37575 },
            { 36864, 37342 },
            { 36865, 37271 },
            { 36867, 37798 },
            { 36869, 59280 },
            { 36870, 35700 },
            { 36875, 59289 },
            { 36877, 59286 },
            { 36878, 59299 },
            { 36879, 37799 },
            { 36880, 37504 },
            { 36881, 59283 },
            { 36883, 37628 },
            { 36884, 37746 },
            { 36885, 59284 },
            { 36886, 59288 },
            { 36887, 36992 },
            { 36889, 38023 },
            { 36890, 37578 },
            { 36893, 37056 },
            { 36894, 59287 },
            { 36895, 37292 },
            { 36896, 37282 },
            { 36897, 59285 },
            { 36898, 34983 },
            { 36899, 38977 },
            { 36903, 59290 },
            { 36910, 37343 },
            { 36913, 36692 },
            { 36914, 36969 },
            { 36917, 59292 },
            { 36918, 59291 },
            { 36920, 35053 },
            { 36921, 59293 },
            { 36924, 38222 },
            { 36926, 59301 },
            { 36929, 37849 },
            { 36930, 37003 },
            { 36933, 37496 },
            { 36935, 35830 },
            { 36937, 59300 },
            { 36938, 38742 },
            { 36939, 35166 },
            { 36941, 38357 },
            { 36942, 35295 },
            { 36943, 59295 },
            { 36944, 59296 },
            { 36945, 59297 },
            { 36946, 59298 },
            { 36947, 37817 },
            { 36948, 37442 },
            { 36949, 35041 },
            { 36950, 59302 },
            { 36952, 59303 },
            { 36953, 60065 },
            { 36956, 37307 },
            { 36958, 59304 },
            { 36960, 35219 },
            { 36961, 37227 },
            { 36963, 36013 },
            { 36965, 38777 },
            { 36967, 64437 },
            { 36968, 59305 },
            { 36969, 37707 },
            { 36973, 37272 },
            { 36974, 36565 },
            { 36975, 59306 },
            { 36978, 59309 },
            { 36981, 36741 },
            { 36982, 59307 },
            { 36983, 37194 },
            { 36984, 37193 },
            { 36986, 35042 },
            { 36988, 38857 },
            { 36989, 59311 },
            { 36991, 38128 },
            { 36992, 59313 },
            { 36993, 59312 },
            { 36994, 59310 },
            { 36995, 57988 },
            { 36996, 35538 },
            { 36999, 59278 },
            { 37001, 59315 },
            { 37002, 59314 },
            { 37007, 59316 },
            { 37009, 38743 },
            { 37027, 37855 },
            { 37030, 38477 },
            { 37032, 59317 },
            { 37034, 36567 },
            { 37039, 59318 },
            { 37041, 59319 },
            { 37045, 59320 },
            { 37048, 37696 },
            { 37057, 35048 },
            { 37066, 36216 },
            { 37070, 39001 },
            { 37083, 59324 },
            { 37086, 64438 },
            { 37089, 35923 },
            { 37090, 59321 },
            { 37092, 59322 },
            { 37096, 38292 },
            { 37101, 35443 },
            { 37109, 38744 },
            { 37111, 35773 },
            { 37117, 37747 },
            { 37122, 59325 },
            { 37138, 59326 },
            { 37141, 64440 },
            { 37145, 59327 },
            { 37159, 64441 },
            { 37165, 37697 },
            { 37168, 59329 },
            { 37170, 59328 },
            { 37193, 37841 },
            { 37194, 59330 },
            { 37195, 36693 },
            { 37196, 36574 },
            { 37197, 38010 },
            { 37198, 37521 },
            { 37202, 36592 },
            { 37204, 37004 },
            { 37206, 59331 },
            { 37208, 59332 },
            { 37218, 36988 },
            { 37219, 59333 },
            { 37221, 59334 },
            { 37225, 59335 },
            { 37226, 38799 },
            { 37228, 36694 },
            { 37234, 59337 },
            { 37235, 59336 },
            { 37237, 36217 },
            { 37239, 36243 },
            { 37240, 36447 },
            { 37250, 59340 },
            { 37255, 36742 },
            { 37257, 59339 },
            { 37259, 59338 },
            { 37261, 37351 },
            { 37264, 36077 },
            { 37266, 37057 },
            { 37271, 38062 },
            { 37276, 36696 },
            { 37282, 59341 },
            { 37284, 36829 },
            { 37290, 59344 },
            { 37291, 59342 },
            { 37295, 59343 },
            { 37300, 59346 },
            { 37301, 59345 },
            { 37304, 36856 },
            { 37306, 59347 },
            { 37312, 59348 },
            { 37313, 59349 },
            { 37318, 38094 },
            { 37319, 36305 },
            { 37320, 36575 },
            { 37321, 59350 },
            { 37323, 59351 },
            { 37324, 38818 },
            { 37325, 36708 },
            { 37326, 38636 },
            { 37327, 38858 },
            { 37328, 59352 },
            { 37329, 35808 },
            { 37334, 59353 },
            { 37335, 64443 },
            { 37336, 37698 },
            { 37338, 64442 },
            { 37339, 59356 },
            { 37340, 35480 },
            { 37341, 36970 },
            { 37342, 64444 },
            { 37343, 59354 },
            { 37345, 59355 },
            { 37347, 37598 },
            { 37348, 64447 },
            { 37349, 64448 },
            { 37350, 38516 },
            { 37351, 35834 },
            { 37357, 64445 },
            { 37358, 64446 },
            { 37365, 59358 },
            { 37366, 59359 },
            { 37372, 59357 },
            { 37375, 59361 },
            { 37382, 64449 },
            { 37386, 64451 },
            { 37389, 37853 },
            { 37390, 35426 },
            { 37392, 64450 },
            { 37393, 59365 },
            { 37396, 59362 },
            { 37397, 59364 },
            { 37406, 59360 },
            { 37417, 59502 },
            { 37420, 59363 },
            { 37428, 38889 },
            { 37431, 36056 },
            { 37433, 64458 },
            { 37434, 64452 },
            { 37436, 64454 },
            { 37439, 59373 },
            { 37440, 64453 },
            { 37444, 37715 },
            { 37445, 59368 },
            { 37448, 59371 },
            { 37449, 59369 },
            { 37451, 59374 },
            { 37454, 64455 },
            { 37456, 59375 },
            { 37457, 64457 },
            { 37463, 59367 },
            { 37465, 64456 },
            { 37466, 59380 },
            { 37467, 35220 },
            { 37470, 59366 },
            { 37474, 38059 },
            { 37476, 59370 },
            { 37478, 36830 },
            { 37479, 64459 },
            { 37489, 36218 },
            { 37495, 64461 },
            { 37496, 64462 },
            { 37502, 38503 },
            { 37504, 35810 },
            { 37507, 36709 },
            { 37509, 37818 },
            { 37512, 64095 },
            { 37521, 37196 },
            { 37523, 59378 },
            { 37525, 59372 },
            { 37526, 59377 },
            { 37528, 38593 },
            { 37530, 37558 },
            { 37531, 59379 },
            { 37532, 59376 },
            { 37543, 64460 },
            { 37549, 37195 },
            { 37559, 59383 },
            { 37561, 59382 },
            { 37583, 59381 },
            { 37584, 64466 },
            { 37586, 38478 },
            { 37587, 64470 },
            { 37589, 64468 },
            { 37591, 64464 },
            { 37593, 64465 },
            { 37600, 64469 },
            { 37604, 36763 },
            { 37607, 64463 },
            { 37609, 59384 },
            { 37610, 38365 },
            { 37613, 35187 },
            { 37618, 38245 },
            { 37619, 37522 },
            { 37624, 35736 },
            { 37625, 64101 },
            { 37626, 59386 },
            { 37627, 64473 },
            { 37628, 36220 },
            { 37631, 64476 },
            { 37634, 64478 },
            { 37638, 36427 },
            { 37647, 59385 },
            { 37648, 37005 },
            { 37656, 37006 },
            { 37657, 59456 },
            { 37658, 59458 },
            { 37661, 64477 },
            { 37662, 64475 },
            { 37664, 36857 },
            { 37665, 64472 },
            { 37666, 59457 },
            { 37667, 59459 },
            { 37669, 64471 },
            { 37670, 35793 },
            { 37672, 38244 },
            { 37675, 36576 },
            { 37676, 38978 },
            { 37678, 59388 },
            { 37679, 36342 },
            { 37682, 39006 },
            { 37685, 59461 },
            { 37690, 59460 },
            { 37691, 59462 },
            { 37700, 59387 },
            { 37704, 64094 },
            { 37707, 37863 },
            { 37709, 37748 },
            { 37716, 37589 },
            { 37718, 59467 },
            { 37719, 64480 },
            { 37723, 37474 },
            { 37724, 59463 },
            { 37728, 59464 },
            { 37740, 35916 },
            { 37742, 59466 },
            { 37744, 64479 },
            { 37749, 36014 },
            { 37756, 59465 },
            { 37758, 36831 },
            { 37772, 35481 },
            { 37780, 59471 },
            { 37782, 36285 },
            { 37783, 37273 },
            { 37786, 37576 },
            { 37796, 64481 },
            { 37799, 35418 },
            { 37804, 59469 },
            { 37805, 59470 },
            { 37806, 37569 },
            { 37808, 59468 },
            { 37817, 59472 },
            { 37827, 59478 },
            { 37830, 64482 },
            { 37832, 59481 },
            { 37840, 59480 },
            { 37841, 37708 },
            { 37846, 59473 },
            { 37847, 59474 },
            { 37848, 59477 },
            { 37853, 59479 },
            { 37854, 64483 },
            { 37857, 35774 },
            { 37860, 59482 },
            { 37861, 59476 },
            { 37864, 59475 },
            { 37880, 64484 },
            { 37891, 59486 },
            { 37895, 59487 },
            { 37904, 59488 },
            { 37907, 59485 },
            { 37908, 59484 },
            { 37912, 36832 },
            { 37913, 37800 },
            { 37914, 59483 },
            { 37921, 59492 },
            { 37931, 59490 },
            { 37937, 64485 },
            { 37941, 59491 },
            { 37942, 59489 },
            { 37944, 37366 },
            { 37946, 59493 },
            { 37953, 59494 },
            { 37956, 59496 },
            { 37957, 64486 },
            { 37960, 64487 },
            { 37969, 35539 },
            { 37970, 59495 },
            { 37971, 38648 },
            { 37978, 59507 },
            { 37979, 59497 },
            { 37982, 59500 },
            { 37984, 59498 },
            { 37986, 59499 },
            { 37994, 59501 },
            { 38000, 59503 },
            { 38005, 59504 },
            { 38007, 59505 },
            { 38012, 59508 },
            { 38013, 59506 },
            { 38014, 59509 },
            { 38015, 59511 },
            { 38017, 59510 },
            { 38263, 37559 },
            { 38272, 38629 },
            { 38274, 59512 },
            { 38275, 37197 },
            { 38279, 59513 },
            { 38281, 38338 },
            { 38282, 59514 },
            { 38283, 35402 },
            { 38287, 35163 },
            { 38289, 35541 },
            { 38290, 64488 },
            { 38291, 35540 },
            { 38292, 59515 },
            { 38294, 59516 },
            { 38296, 59517 },
            { 38297, 59518 },
            { 38304, 59520 },
            { 38306, 35542 },
            { 38307, 35444 },
            { 38308, 36221 },
            { 38309, 38068 },
            { 38311, 59522 },
            { 38312, 59521 },
            { 38317, 59523 },
            { 38322, 35195 },
            { 38329, 59526 },
            { 38331, 59525 },
            { 38332, 59524 },
            { 38334, 59527 },
            { 38339, 59530 },
            { 38343, 35013 },
            { 38346, 59528 },
            { 38348, 59532 },
            { 38349, 59531 },
            { 38356, 59534 },
            { 38357, 59533 },
            { 38358, 59535 },
            { 38360, 37804 },
            { 38364, 59536 },
            { 38369, 59537 },
            { 38370, 59539 },
            { 38373, 59538 },
            { 38428, 38284 },
            { 38433, 59540 },
            { 38440, 59541 },
            { 38442, 36323 },
            { 38446, 59542 },
            { 38447, 59543 },
            { 38450, 38504 },
            { 38459, 37226 },
            { 38463, 34978 },
            { 38464, 37321 },
            { 38466, 59544 },
            { 38468, 38285 },
            { 38475, 59547 },
            { 38476, 59545 },
            { 38477, 36222 },
            { 38479, 59546 },
            { 38480, 36032 },
            { 38491, 38339 },
            { 38492, 59549 },
            { 38493, 59551 },
            { 38494, 59550 },
            { 38495, 59552 },
            { 38498, 35136 },
            { 38499, 36983 },
            { 38500, 36764 },
            { 38501, 35543 },
            { 38502, 59553 },
            { 38506, 38022 },
            { 38508, 59555 },
            { 38512, 35137 },
            { 38514, 59554 },
            { 38515, 37570 },
            { 38517, 38859 },
            { 38518, 37801 },
            { 38519, 59548 },
            { 38520, 38820 },
            { 38522, 36015 },
            { 38525, 38778 },
            { 38533, 35831 },
            { 38534, 38834 },
            { 38536, 35911 },
            { 38538, 37344 },
            { 38539, 58432 },
            { 38541, 59556 },
            { 38542, 35403 },
            { 38543, 37007 },
            { 38548, 35445 },
            { 38549, 59558 },
            { 38551, 59559 },
            { 38552, 59557 },
            { 38553, 35972 },
            { 38555, 36315 },
            { 38556, 36833 },
            { 38557, 64491 },
            { 38560, 35138 },
            { 38563, 38871 },
            { 38567, 59561 },
            { 38568, 59308 },
            { 38570, 59560 },
            { 38575, 64492 },
            { 38576, 59564 },
            { 38577, 59562 },
            { 38578, 59563 },
            { 38580, 59565 },
            { 38582, 59566 },
            { 38583, 38890 },
            { 38584, 59567 },
            { 38585, 59568 },
            { 38587, 37063 },
            { 38588, 38073 },
            { 38592, 37021 },
            { 38593, 35557 },
            { 38596, 38745 },
            { 38597, 35307 },
            { 38598, 36695 },
            { 38599, 36057 },
            { 38601, 59571 },
            { 38603, 59570 },
            { 38604, 36499 },
            { 38605, 59572 },
            { 38606, 59569 },
            { 38609, 36423 },
            { 38613, 59576 },
            { 38614, 58795 },
            { 38617, 39380 },
            { 38619, 37015 },
            { 38620, 59574 },
            { 38626, 38819 },
            { 38627, 37871 },
            { 38632, 35146 },
            { 38634, 37089 },
            { 38635, 36532 },
            { 38640, 38325 },
            { 38642, 35167 },
            { 38646, 38891 },
            { 38647, 38795 },
            { 38649, 59577 },
            { 38651, 37732 },
            { 38656, 36601 },
            { 38660, 59578 },
            { 38662, 59579 },
            { 38663, 36971 },
            { 38664, 59580 },
            { 38666, 38892 },
            { 38669, 59575 },
            { 38670, 59582 },
            { 38671, 59584 },
            { 38673, 59583 },
            { 38675, 59581 },
            { 38678, 59585 },
            { 38681, 59586 },
            { 38684, 37274 },
            { 38686, 35296 },
            { 38692, 59587 },
            { 38695, 38582 },
            { 38698, 59588 },
            { 38704, 59589 },
            { 38706, 38985 },
            { 38707, 64493 },
            { 38712, 40528 },
            { 38713, 59590 },
            { 38715, 64494 },
            { 38717, 59591 },
            { 38718, 59592 },
            { 38722, 59596 },
            { 38723, 64495 },
            { 38724, 59593 },
            { 38726, 59594 },
            { 38728, 59595 },
            { 38729, 59597 },
            { 38733, 64496 },
            { 38735, 64497 },
            { 38737, 64498 },
            { 38738, 37058 },
            { 38741, 64499 },
            { 38742, 38645 },
            { 38745, 37059 },
            { 38748, 59598 },
            { 38750, 38129 },
            { 38752, 59599 },
            { 38753, 60018 },
            { 38754, 38602 },
            { 38756, 59600 },
            { 38758, 59601 },
            { 38760, 59602 },
            { 38761, 35446 },
            { 38763, 59604 },
            { 38765, 36984 },
            { 38769, 59605 },
            { 38772, 35907 },
            { 38777, 59606 },
            { 38778, 59610 },
            { 38780, 59608 },
            { 38785, 59609 },
            { 38788, 35475 },
            { 38789, 59607 },
            { 38790, 59611 },
            { 38795, 59612 },
            { 38797, 35014 },
            { 38799, 59613 },
            { 38800, 59614 },
            { 38808, 36834 },
            { 38812, 59615 },
            { 38816, 35686 },
            { 38819, 59618 },
            { 38822, 59617 },
            { 38824, 59616 },
            { 38827, 59025 },
            { 38829, 38362 },
            { 38835, 59619 },
            { 38836, 59620 },
            { 38851, 59621 },
            { 38854, 59622 },
            { 38856, 59623 },
            { 38859, 59624 },
            { 38867, 35544 },
            { 38876, 59625 },
            { 38893, 59626 },
            { 38894, 37954 },
            { 38898, 59628 },
            { 38899, 35257 },
            { 38901, 59631 },
            { 38902, 59630 },
            { 38907, 35139 },
            { 38911, 35775 },
            { 38913, 38341 },
            { 38914, 37560 },
            { 38915, 36256 },
            { 38917, 36224 },
            { 38918, 36743 },
            { 38920, 36987 },
            { 38924, 59633 },
            { 38927, 59632 },
            { 38928, 38753 },
            { 38929, 35558 },
            { 38930, 38096 },
            { 38931, 37850 },
            { 38935, 37020 },
            { 38936, 38860 },
            { 38938, 35962 },
            { 38945, 59636 },
            { 38948, 59635 },
            { 38956, 38506 },
            { 38957, 37802 },
            { 38964, 35183 },
            { 38967, 59637 },
            { 38968, 59634 },
            { 38971, 38256 },
            { 38972, 38794 },
            { 38973, 59638 },
            { 38982, 59639 },
            { 38987, 59641 },
            { 38988, 37352 },
            { 38989, 35450 },
            { 38990, 35451 },
            { 38991, 59640 },
            { 38996, 35559 },
            { 38997, 36016 },
            { 38999, 64500 },
            { 39000, 35560 },
            { 39003, 37726 },
            { 39006, 38878 },
            { 39013, 64501 },
            { 39015, 36058 },
            { 39019, 59642 },
            { 39023, 59643 },
            { 39024, 59644 },
            { 39025, 59712 },
            { 39027, 59714 },
            { 39028, 59713 },
            { 39080, 38295 },
            { 39082, 59715 },
            { 39087, 59716 },
            { 39089, 59717 },
            { 39094, 59718 },
            { 39107, 59720 },
            { 39108, 59719 },
            { 39110, 59721 },
            { 39131, 38130 },
            { 39132, 58314 },
            { 39135, 36936 },
            { 39138, 35665 },
            { 39145, 59722 },
            { 39147, 59723 },
            { 39149, 39338 },
            { 39150, 40794 },
            { 39151, 38097 },
            { 39154, 35065 },
            { 39156, 35001 },
            { 39164, 36500 },
            { 39165, 38479 },
            { 39166, 36860 },
            { 39171, 59724 },
            { 39173, 38621 },
            { 39177, 59725 },
            { 39178, 38779 },
            { 39180, 35169 },
            { 39184, 36448 },
            { 39186, 59726 },
            { 39187, 35308 },
            { 39188, 59727 },
            { 39192, 59728 },
            { 39197, 59730 },
            { 39198, 59731 },
            { 39200, 59733 },
            { 39201, 59729 },
            { 39204, 59732 },
            { 39207, 64504 },
            { 39208, 35545 },
            { 39212, 59734 },
            { 39214, 59735 },
            { 39229, 59736 },
            { 39230, 59737 },
            { 39234, 59738 },
            { 39237, 59740 },
            { 39241, 59739 },
            { 39243, 59742 },
            { 39244, 59745 },
            { 39248, 59741 },
            { 39249, 59743 },
            { 39250, 59744 },
            { 39253, 59746 },
            { 39255, 35776 },
            { 39318, 36593 },
            { 39319, 59747 },
            { 39320, 59748 },
            { 39321, 36225 },
            { 39326, 64506 },
            { 39333, 59749 },
            { 39336, 35421 },
            { 39340, 37998 },
            { 39341, 59750 },
            { 39342, 59751 },
            { 39347, 37497 },
            { 39348, 37865 },
            { 39356, 59752 },
            { 39361, 38045 },
            { 39364, 37322 },
            { 39365, 35191 },
            { 39366, 35820 },
            { 39368, 35821 },
            { 39376, 37523 },
            { 39377, 59757 },
            { 39378, 35822 },
            { 39381, 35309 },
            { 39384, 59756 },
            { 39387, 59754 },
            { 39389, 59755 },
            { 39391, 59753 },
            { 39394, 59767 },
            { 39405, 59758 },
            { 39406, 59759 },
            { 39409, 59760 },
            { 39410, 59761 },
            { 39416, 59763 },
            { 39419, 59762 },
            { 39423, 36728 },
            { 39425, 59764 },
            { 39429, 59766 },
            { 39438, 35666 },
            { 39439, 59765 },
            { 39442, 37275 },
            { 39443, 36017 },
            { 39449, 59768 },
            { 39464, 37323 },
            { 39467, 59769 },
            { 39472, 37803 },
            { 39479, 59770 },
            { 39486, 59776 },
            { 39488, 59773 },
            { 39490, 59772 },
            { 39491, 59774 },
            { 39493, 59771 },
            { 39501, 59778 },
            { 39502, 64507 },
            { 39509, 59777 },
            { 39511, 59780 },
            { 39514, 35777 },
            { 39515, 59779 },
            { 39519, 59781 },
            { 39522, 59782 },
            { 39524, 59784 },
            { 39525, 59783 },
            { 39529, 59785 },
            { 39530, 59787 },
            { 39531, 59786 },
            { 39592, 36252 },
            { 39597, 59788 },
            { 39600, 59789 },
            { 39608, 35419 },
            { 39612, 59790 },
            { 39616, 59791 },
            { 39620, 37009 },
            { 39631, 59792 },
            { 39633, 59793 },
            { 39635, 59794 },
            { 39636, 59795 },
            { 39640, 36226 },
            { 39641, 64508 },
            { 39644, 64576 },
            { 39646, 59796 },
            { 39647, 59797 },
            { 39650, 59798 },
            { 39651, 59799 },
            { 39654, 59800 },
            { 39658, 38063 },
            { 39659, 59802 },
            { 39661, 38213 },
            { 39662, 59803 },
            { 39663, 59801 },
            { 39665, 59805 },
            { 39668, 59804 },
            { 39671, 59806 },
            { 39675, 59807 },
            { 39686, 59808 },
            { 39704, 59809 },
            { 39706, 59810 },
            { 39711, 59811 },
            { 39714, 59812 },
            { 39715, 59813 },
            { 39717, 59814 },
            { 39719, 59815 },
            { 39720, 59816 },
            { 39721, 59817 },
            { 39722, 59818 },
            { 39726, 59819 },
            { 39727, 59820 },
            { 39729, 40788 },
            { 39730, 59821 },
            { 39739, 58102 },
            { 39740, 35667 },
            { 39745, 35392 },
            { 39746, 36272 },
            { 39747, 59823 },
            { 39748, 59822 },
            { 39749, 38563 },
            { 39757, 59825 },
            { 39758, 59826 },
            { 39759, 59824 },
            { 39761, 59827 },
            { 39764, 38530 },
            { 39768, 59828 },
            { 39770, 35739 },
            { 39791, 38980 },
            { 39794, 64578 },
            { 39796, 59829 },
            { 39797, 64577 },
            { 39811, 59831 },
            { 39822, 35004 },
            { 39823, 64579 },
            { 39825, 59832 },
            { 39826, 38313 },
            { 39827, 59830 },
            { 39830, 59833 },
            { 39831, 59834 },
            { 39839, 59835 },
            { 39840, 59836 },
            { 39848, 59837 },
            { 39850, 38542 },
            { 39851, 36428 },
            { 39853, 36344 },
            { 39854, 37198 },
            { 39857, 64580 },
            { 39860, 59838 },
            { 39865, 59841 },
            { 39867, 64581 },
            { 39872, 59839 },
            { 39878, 59842 },
            { 39881, 36079 },
            { 39882, 59840 },
            { 39887, 59843 },
            { 39889, 59844 },
            { 39890, 59845 },
            { 39892, 59849 },
            { 39894, 36425 },
            { 39899, 37346 },
            { 39905, 59850 },
            { 39906, 59847 },
            { 39907, 59846 },
            { 39908, 59848 },
            { 39912, 35966 },
            { 39920, 59854 },
            { 39921, 59853 },
            { 39922, 59852 },
            { 39925, 34993 },
            { 39936, 64582 },
            { 39940, 59864 },
            { 39942, 59860 },
            { 39944, 59861 },
            { 39945, 59857 },
            { 39946, 59863 },
            { 39948, 59859 },
            { 39949, 35458 },
            { 39952, 39019 },
            { 39954, 59862 },
            { 39955, 59858 },
            { 39956, 59856 },
            { 39957, 59855 },
            { 39963, 59866 },
            { 39969, 59869 },
            { 39972, 59868 },
            { 39973, 59867 },
            { 39981, 38248 },
            { 39982, 59865 },
            { 39983, 35057 },
            { 39984, 59870 },
            { 39986, 59872 },
            { 39993, 35471 },
            { 39994, 59851 },
            { 39995, 35158 },
            { 39998, 59874 },
            { 40006, 59873 },
            { 40007, 59871 },
            { 40008, 37452 },
            { 40018, 38544 },
            { 40023, 38872 },
            { 40026, 59875 },
            { 40032, 59876 },
            { 40039, 59877 },
            { 40054, 59878 },
            { 40056, 59879 },
            { 40165, 37561 },
            { 40167, 59880 },
            { 40169, 38069 },
            { 40171, 59885 },
            { 40172, 59881 },
            { 40176, 59882 },
            { 40179, 38480 },
            { 40180, 38594 },
            { 40182, 37838 },
            { 40195, 59886 },
            { 40198, 59887 },
            { 40199, 37820 },
            { 40200, 59884 },
            { 40201, 59883 },
            { 40206, 35240 },
            { 40210, 59895 },
            { 40213, 59894 },
            { 40219, 35221 },
            { 40223, 59892 },
            { 40227, 59891 },
            { 40230, 59889 },
            { 40232, 35483 },
            { 40234, 59888 },
            { 40235, 36528 },
            { 40236, 35239 },
            { 40251, 36227 },
            { 40254, 59898 },
            { 40255, 59897 },
            { 40257, 59896 },
            { 40260, 59893 },
            { 40262, 59899 },
            { 40264, 59900 },
            { 40272, 59972 },
            { 40273, 59971 },
            { 40281, 59973 },
            { 40284, 35148 },
            { 40285, 59968 },
            { 40286, 59969 },
            { 40288, 36244 },
            { 40289, 38583 },
            { 40292, 59970 },
            { 40299, 64584 },
            { 40300, 38481 },
            { 40303, 59978 },
            { 40304, 64583 },
            { 40306, 59974 },
            { 40314, 59979 },
            { 40327, 59976 },
            { 40329, 59975 },
            { 40335, 35963 },
            { 40346, 59980 },
            { 40356, 59981 },
            { 40361, 59982 },
            { 40363, 59977 },
            { 40367, 59890 },
            { 40370, 59983 },
            { 40372, 37599 },
            { 40376, 59987 },
            { 40378, 59988 },
            { 40379, 59986 },
            { 40385, 59985 },
            { 40386, 59991 },
            { 40388, 59984 },
            { 40390, 59989 },
            { 40399, 59990 },
            { 40403, 59993 },
            { 40409, 59992 },
            { 40422, 59995 },
            { 40429, 59996 },
            { 40431, 59997 },
            { 40434, 39016 },
            { 40440, 59994 },
            { 40441, 37353 },
            { 40442, 36331 },
            { 40445, 59998 },
            { 40473, 64586 },
            { 40474, 59999 },
            { 40475, 60000 },
            { 40478, 60001 },
            { 40565, 60002 },
            { 40568, 36018 },
            { 40569, 60003 },
            { 40573, 60004 },
            { 40575, 36525 },
            { 40577, 60005 },
            { 40584, 60006 },
            { 40587, 60007 },
            { 40588, 60008 },
            { 40593, 60011 },
            { 40594, 60009 },
            { 40595, 39003 },
            { 40597, 60010 },
            { 40599, 38893 },
            { 40605, 60012 },
            { 40607, 38873 },
            { 40613, 60013 },
            { 40614, 38046 },
            { 40617, 60014 },
            { 40618, 60016 },
            { 40621, 60017 },
            { 40632, 60015 },
            { 40633, 36237 },
            { 40634, 38603 },
            { 40635, 38531 },
            { 40636, 39925 },
            { 40638, 40832 },
            { 40639, 38555 },
            { 40644, 35241 },
            { 40652, 60019 },
            { 40653, 35695 },
            { 40654, 60020 },
            { 40655, 60021 },
            { 40656, 60022 },
            { 40657, 64587 },
            { 40658, 36245 },
            { 40660, 60023 },
            { 40664, 57554 },
            { 40665, 38617 },
            { 40667, 37345 },
            { 40668, 60024 },
            { 40669, 60026 },
            { 40670, 60025 },
            { 40672, 60027 },
            { 40677, 60028 },
            { 40680, 60029 },
            { 40687, 60030 },
            { 40692, 60032 },
            { 40694, 60033 },
            { 40695, 60034 },
            { 40697, 60035 },
            { 40699, 60036 },
            { 40700, 60037 },
            { 40701, 60038 },
            { 40711, 60039 },
            { 40712, 60040 },
            { 40718, 37699 },
            { 40723, 36059 },
            { 40725, 60042 },
            { 40736, 37228 },
            { 40737, 60043 },
            { 40748, 60044 },
            { 40763, 38208 },
            { 40766, 60045 },
            { 40778, 60046 },
            { 40779, 57942 },
            { 40782, 59096 },
            { 40783, 59627 },
            { 40786, 60047 },
            { 40788, 60048 },
            { 40799, 60050 },
            { 40800, 60051 },
            { 40801, 60052 },
            { 40802, 38894 },
            { 40803, 60049 },
            { 40806, 60053 },
            { 40807, 60054 },
            { 40810, 60056 },
            { 40812, 60055 },
            { 40818, 60058 },
            { 40822, 60059 },
            { 40823, 60057 },
            { 40845, 38836 },
            { 40853, 60060 },
            { 40860, 60061 },
            { 40861, 57971 },
            { 40864, 60062 },
            { 57344, 61504 },
            { 57345, 61505 },
            { 57346, 61506 },
            { 57347, 61507 },
            { 57348, 61508 },
            { 57349, 61509 },
            { 57350, 61510 },
            { 57351, 61511 },
            { 57352, 61512 },
            { 57353, 61513 },
            { 57354, 61514 },
            { 57355, 61515 },
            { 57356, 61516 },
            { 57357, 61517 },
            { 57358, 61518 },
            { 57359, 61519 },
            { 57360, 61520 },
            { 57361, 61521 },
            { 57362, 61522 },
            { 57363, 61523 },
            { 57364, 61524 },
            { 57365, 61525 },
            { 57366, 61526 },
            { 57367, 61527 },
            { 57368, 61528 },
            { 57369, 61529 },
            { 57370, 61530 },
            { 57371, 61531 },
            { 57372, 61532 },
            { 57373, 61533 },
            { 57374, 61534 },
            { 57375, 61535 },
            { 57376, 61536 },
            { 57377, 61537 },
            { 57378, 61538 },
            { 57379, 61539 },
            { 57380, 61540 },
            { 57381, 61541 },
            { 57382, 61542 },
            { 57383, 61543 },
            { 57384, 61544 },
            { 57385, 61545 },
            { 57386, 61546 },
            { 57387, 61547 },
            { 57388, 61548 },
            { 57389, 61549 },
            { 57390, 61550 },
            { 57391, 61551 },
            { 57392, 61552 },
            { 57393, 61553 },
            { 57394, 61554 },
            { 57395, 61555 },
            { 57396, 61556 },
            { 57397, 61557 },
            { 57398, 61558 },
            { 57399, 61559 },
            { 57400, 61560 },
            { 57401, 61561 },
            { 57402, 61562 },
            { 57403, 61563 },
            { 57404, 61564 },
            { 57405, 61565 },
            { 57406, 61566 },
            { 57407, 61568 },
            { 57408, 61569 },
            { 57409, 61570 },
            { 57410, 61571 },
            { 57411, 61572 },
            { 57412, 61573 },
            { 57413, 61574 },
            { 57414, 61575 },
            { 57415, 61576 },
            { 57416, 61577 },
            { 57417, 61578 },
            { 57418, 61579 },
            { 57419, 61580 },
            { 57420, 61581 },
            { 57421, 61582 },
            { 57422, 61583 },
            { 57423, 61584 },
            { 57424, 61585 },
            { 57425, 61586 },
            { 57426, 61587 },
            { 57427, 61588 },
            { 57428, 61589 },
            { 57429, 61590 },
            { 57430, 61591 },
            { 57431, 61592 },
            { 57432, 61593 },
            { 57433, 61594 },
            { 57434, 61595 },
            { 57435, 61596 },
            { 57436, 61597 },
            { 57437, 61598 },
            { 57438, 61599 },
            { 57439, 61600 },
            { 57440, 61601 },
            { 57441, 61602 },
            { 57442, 61603 },
            { 57443, 61604 },
            { 57444, 61605 },
            { 57445, 61606 },
            { 57446, 61607 },
            { 57447, 61608 },
            { 57448, 61609 },
            { 57449, 61610 },
            { 57450, 61611 },
            { 57451, 61612 },
            { 57452, 61613 },
            { 57453, 61614 },
            { 57454, 61615 },
            { 57455, 61616 },
            { 57456, 61617 },
            { 57457, 61618 },
            { 57458, 61619 },
            { 57459, 61620 },
            { 57460, 61621 },
            { 57461, 61622 },
            { 57462, 61623 },
            { 57463, 61624 },
            { 57464, 61625 },
            { 57465, 61626 },
            { 57466, 61627 },
            { 57467, 61628 },
            { 57468, 61629 },
            { 57469, 61630 },
            { 57470, 61631 },
            { 57471, 61632 },
            { 57472, 61633 },
            { 57473, 61634 },
            { 57474, 61635 },
            { 57475, 61636 },
            { 57476, 61637 },
            { 57477, 61638 },
            { 57478, 61639 },
            { 57479, 61640 },
            { 57480, 61641 },
            { 57481, 61642 },
            { 57482, 61643 },
            { 57483, 61644 },
            { 57484, 61645 },
            { 57485, 61646 },
            { 57486, 61647 },
            { 57487, 61648 },
            { 57488, 61649 },
            { 57489, 61650 },
            { 57490, 61651 },
            { 57491, 61652 },
            { 57492, 61653 },
            { 57493, 61654 },
            { 57494, 61655 },
            { 57495, 61656 },
            { 57496, 61657 },
            { 57497, 61658 },
            { 57498, 61659 },
            { 57499, 61660 },
            { 57500, 61661 },
            { 57501, 61662 },
            { 57502, 61663 },
            { 57503, 61664 },
            { 57504, 61665 },
            { 57505, 61666 },
            { 57506, 61667 },
            { 57507, 61668 },
            { 57508, 61669 },
            { 57509, 61670 },
            { 57510, 61671 },
            { 57511, 61672 },
            { 57512, 61673 },
            { 57513, 61674 },
            { 57514, 61675 },
            { 57515, 61676 },
            { 57516, 61677 },
            { 57517, 61678 },
            { 57518, 61679 },
            { 57519, 61680 },
            { 57520, 61681 },
            { 57521, 61682 },
            { 57522, 61683 },
            { 57523, 61684 },
            { 57524, 61685 },
            { 57525, 61686 },
            { 57526, 61687 },
            { 57527, 61688 },
            { 57528, 61689 },
            { 57529, 61690 },
            { 57530, 61691 },
            { 57531, 61692 },
            { 57532, 61760 },
            { 57533, 61761 },
            { 57534, 61762 },
            { 57535, 61763 },
            { 57536, 61764 },
            { 57537, 61765 },
            { 57538, 61766 },
            { 57539, 61767 },
            { 57540, 61768 },
            { 57541, 61769 },
            { 57542, 61770 },
            { 57543, 61771 },
            { 57544, 61772 },
            { 57545, 61773 },
            { 57546, 61774 },
            { 57547, 61775 },
            { 57548, 61776 },
            { 57549, 61777 },
            { 57550, 61778 },
            { 57551, 61779 },
            { 57552, 61780 },
            { 57553, 61781 },
            { 57554, 61782 },
            { 57555, 61783 },
            { 57556, 61784 },
            { 57557, 61785 },
            { 57558, 61786 },
            { 57559, 61787 },
            { 57560, 61788 },
            { 57561, 61789 },
            { 57562, 61790 },
            { 57563, 61791 },
            { 57564, 61792 },
            { 57565, 61793 },
            { 57566, 61794 },
            { 57567, 61795 },
            { 57568, 61796 },
            { 57569, 61797 },
            { 57570, 61798 },
            { 57571, 61799 },
            { 57572, 61800 },
            { 57573, 61801 },
            { 57574, 61802 },
            { 57575, 61803 },
            { 57576, 61804 },
            { 57577, 61805 },
            { 57578, 61806 },
            { 57579, 61807 },
            { 57580, 61808 },
            { 57581, 61809 },
            { 57582, 61810 },
            { 57583, 61811 },
            { 57584, 61812 },
            { 57585, 61813 },
            { 57586, 61814 },
            { 57587, 61815 },
            { 57588, 61816 },
            { 57589, 61817 },
            { 57590, 61818 },
            { 57591, 61819 },
            { 57592, 61820 },
            { 57593, 61821 },
            { 57594, 61822 },
            { 57595, 61824 },
            { 57596, 61825 },
            { 57597, 61826 },
            { 57598, 61827 },
            { 57599, 61828 },
            { 57600, 61829 },
            { 57601, 61830 },
            { 57602, 61831 },
            { 57603, 61832 },
            { 57604, 61833 },
            { 57605, 61834 },
            { 57606, 61835 },
            { 57607, 61836 },
            { 57608, 61837 },
            { 57609, 61838 },
            { 57610, 61839 },
            { 57611, 61840 },
            { 57612, 61841 },
            { 57613, 61842 },
            { 57614, 61843 },
            { 57615, 61844 },
            { 57616, 61845 },
            { 57617, 61846 },
            { 57618, 61847 },
            { 57619, 61848 },
            { 57620, 61849 },
            { 57621, 61850 },
            { 57622, 61851 },
            { 57623, 61852 },
            { 57624, 61853 },
            { 57625, 61854 },
            { 57626, 61855 },
            { 57627, 61856 },
            { 57628, 61857 },
            { 57629, 61858 },
            { 57630, 61859 },
            { 57631, 61860 },
            { 57632, 61861 },
            { 57633, 61862 },
            { 57634, 61863 },
            { 57635, 61864 },
            { 57636, 61865 },
            { 57637, 61866 },
            { 57638, 61867 },
            { 57639, 61868 },
            { 57640, 61869 },
            { 57641, 61870 },
            { 57642, 61871 },
            { 57643, 61872 },
            { 57644, 61873 },
            { 57645, 61874 },
            { 57646, 61875 },
            { 57647, 61876 },
            { 57648, 61877 },
            { 57649, 61878 },
            { 57650, 61879 },
            { 57651, 61880 },
            { 57652, 61881 },
            { 57653, 61882 },
            { 57654, 61883 },
            { 57655, 61884 },
            { 57656, 61885 },
            { 57657, 61886 },
            { 57658, 61887 },
            { 57659, 61888 },
            { 57660, 61889 },
            { 57661, 61890 },
            { 57662, 61891 },
            { 57663, 61892 },
            { 57664, 61893 },
            { 57665, 61894 },
            { 57666, 61895 },
            { 57667, 61896 },
            { 57668, 61897 },
            { 57669, 61898 },
            { 57670, 61899 },
            { 57671, 61900 },
            { 57672, 61901 },
            { 57673, 61902 },
            { 57674, 61903 },
            { 57675, 61904 },
            { 57676, 61905 },
            { 57677, 61906 },
            { 57678, 61907 },
            { 57679, 61908 },
            { 57680, 61909 },
            { 57681, 61910 },
            { 57682, 61911 },
            { 57683, 61912 },
            { 57684, 61913 },
            { 57685, 61914 },
            { 57686, 61915 },
            { 57687, 61916 },
            { 57688, 61917 },
            { 57689, 61918 },
            { 57690, 61919 },
            { 57691, 61920 },
            { 57692, 61921 },
            { 57693, 61922 },
            { 57694, 61923 },
            { 57695, 61924 },
            { 57696, 61925 },
            { 57697, 61926 },
            { 57698, 61927 },
            { 57699, 61928 },
            { 57700, 61929 },
            { 57701, 61930 },
            { 57702, 61931 },
            { 57703, 61932 },
            { 57704, 61933 },
            { 57705, 61934 },
            { 57706, 61935 },
            { 57707, 61936 },
            { 57708, 61937 },
            { 57709, 61938 },
            { 57710, 61939 },
            { 57711, 61940 },
            { 57712, 61941 },
            { 57713, 61942 },
            { 57714, 61943 },
            { 57715, 61944 },
            { 57716, 61945 },
            { 57717, 61946 },
            { 57718, 61947 },
            { 57719, 61948 },
            { 57720, 62016 },
            { 57721, 62017 },
            { 57722, 62018 },
            { 57723, 62019 },
            { 57724, 62020 },
            { 57725, 62021 },
            { 57726, 62022 },
            { 57727, 62023 },
            { 57728, 62024 },
            { 57729, 62025 },
            { 57730, 62026 },
            { 57731, 62027 },
            { 57732, 62028 },
            { 57733, 62029 },
            { 57734, 62030 },
            { 57735, 62031 },
            { 57736, 62032 },
            { 57737, 62033 },
            { 57738, 62034 },
            { 57739, 62035 },
            { 57740, 62036 },
            { 57741, 62037 },
            { 57742, 62038 },
            { 57743, 62039 },
            { 57744, 62040 },
            { 57745, 62041 },
            { 57746, 62042 },
            { 57747, 62043 },
            { 57748, 62044 },
            { 57749, 62045 },
            { 57750, 62046 },
            { 57751, 62047 },
            { 57752, 62048 },
            { 57753, 62049 },
            { 57754, 62050 },
            { 57755, 62051 },
            { 57756, 62052 },
            { 57757, 62053 },
            { 57758, 62054 },
            { 57759, 62055 },
            { 57760, 62056 },
            { 57761, 62057 },
            { 57762, 62058 },
            { 57763, 62059 },
            { 57764, 62060 },
            { 57765, 62061 },
            { 57766, 62062 },
            { 57767, 62063 },
            { 57768, 62064 },
            { 57769, 62065 },
            { 57770, 62066 },
            { 57771, 62067 },
            { 57772, 62068 },
            { 57773, 62069 },
            { 57774, 62070 },
            { 57775, 62071 },
            { 57776, 62072 },
            { 57777, 62073 },
            { 57778, 62074 },
            { 57779, 62075 },
            { 57780, 62076 },
            { 57781, 62077 },
            { 57782, 62078 },
            { 57783, 62080 },
            { 57784, 62081 },
            { 57785, 62082 },
            { 57786, 62083 },
            { 57787, 62084 },
            { 57788, 62085 },
            { 57789, 62086 },
            { 57790, 62087 },
            { 57791, 62088 },
            { 57792, 62089 },
            { 57793, 62090 },
            { 57794, 62091 },
            { 57795, 62092 },
            { 57796, 62093 },
            { 57797, 62094 },
            { 57798, 62095 },
            { 57799, 62096 },
            { 57800, 62097 },
            { 57801, 62098 },
            { 57802, 62099 },
            { 57803, 62100 },
            { 57804, 62101 },
            { 57805, 62102 },
            { 57806, 62103 },
            { 57807, 62104 },
            { 57808, 62105 },
            { 57809, 62106 },
            { 57810, 62107 },
            { 57811, 62108 },
            { 57812, 62109 },
            { 57813, 62110 },
            { 57814, 62111 },
            { 57815, 62112 },
            { 57816, 62113 },
            { 57817, 62114 },
            { 57818, 62115 },
            { 57819, 62116 },
            { 57820, 62117 },
            { 57821, 62118 },
            { 57822, 62119 },
            { 57823, 62120 },
            { 57824, 62121 },
            { 57825, 62122 },
            { 57826, 62123 },
            { 57827, 62124 },
            { 57828, 62125 },
            { 57829, 62126 },
            { 57830, 62127 },
            { 57831, 62128 },
            { 57832, 62129 },
            { 57833, 62130 },
            { 57834, 62131 },
            { 57835, 62132 },
            { 57836, 62133 },
            { 57837, 62134 },
            { 57838, 62135 },
            { 57839, 62136 },
            { 57840, 62137 },
            { 57841, 62138 },
            { 57842, 62139 },
            { 57843, 62140 },
            { 57844, 62141 },
            { 57845, 62142 },
            { 57846, 62143 },
            { 57847, 62144 },
            { 57848, 62145 },
            { 57849, 62146 },
            { 57850, 62147 },
            { 57851, 62148 },
            { 57852, 62149 },
            { 57853, 62150 },
            { 57854, 62151 },
            { 57855, 62152 },
            { 57856, 62153 },
            { 57857, 62154 },
            { 57858, 62155 },
            { 57859, 62156 },
            { 57860, 62157 },
            { 57861, 62158 },
            { 57862, 62159 },
            { 57863, 62160 },
            { 57864, 62161 },
            { 57865, 62162 },
            { 57866, 62163 },
            { 57867, 62164 },
            { 57868, 62165 },
            { 57869, 62166 },
            { 57870, 62167 },
            { 57871, 62168 },
            { 57872, 62169 },
            { 57873, 62170 },
            { 57874, 62171 },
            { 57875, 62172 },
            { 57876, 62173 },
            { 57877, 62174 },
            { 57878, 62175 },
            { 57879, 62176 },
            { 57880, 62177 },
            { 57881, 62178 },
            { 57882, 62179 },
            { 57883, 62180 },
            { 57884, 62181 },
            { 57885, 62182 },
            { 57886, 62183 },
            { 57887, 62184 },
            { 57888, 62185 },
            { 57889, 62186 },
            { 57890, 62187 },
            { 57891, 62188 },
            { 57892, 62189 },
            { 57893, 62190 },
            { 57894, 62191 },
            { 57895, 62192 },
            { 57896, 62193 },
            { 57897, 62194 },
            { 57898, 62195 },
            { 57899, 62196 },
            { 57900, 62197 },
            { 57901, 62198 },
            { 57902, 62199 },
            { 57903, 62200 },
            { 57904, 62201 },
            { 57905, 62202 },
            { 57906, 62203 },
            { 57907, 62204 },
            { 57908, 62272 },
            { 57909, 62273 },
            { 57910, 62274 },
            { 57911, 62275 },
            { 57912, 62276 },
            { 57913, 62277 },
            { 57914, 62278 },
            { 57915, 62279 },
            { 57916, 62280 },
            { 57917, 62281 },
            { 57918, 62282 },
            { 57919, 62283 },
            { 57920, 62284 },
            { 57921, 62285 },
            { 57922, 62286 },
            { 57923, 62287 },
            { 57924, 62288 },
            { 57925, 62289 },
            { 57926, 62290 },
            { 57927, 62291 },
            { 57928, 62292 },
            { 57929, 62293 },
            { 57930, 62294 },
            { 57931, 62295 },
            { 57932, 62296 },
            { 57933, 62297 },
            { 57934, 62298 },
            { 57935, 62299 },
            { 57936, 62300 },
            { 57937, 62301 },
            { 57938, 62302 },
            { 57939, 62303 },
            { 57940, 62304 },
            { 57941, 62305 },
            { 57942, 62306 },
            { 57943, 62307 },
            { 57944, 62308 },
            { 57945, 62309 },
            { 57946, 62310 },
            { 57947, 62311 },
            { 57948, 62312 },
            { 57949, 62313 },
            { 57950, 62314 },
            { 57951, 62315 },
            { 57952, 62316 },
            { 57953, 62317 },
            { 57954, 62318 },
            { 57955, 62319 },
            { 57956, 62320 },
            { 57957, 62321 },
            { 57958, 62322 },
            { 57959, 62323 },
            { 57960, 62324 },
            { 57961, 62325 },
            { 57962, 62326 },
            { 57963, 62327 },
            { 57964, 62328 },
            { 57965, 62329 },
            { 57966, 62330 },
            { 57967, 62331 },
            { 57968, 62332 },
            { 57969, 62333 },
            { 57970, 62334 },
            { 57971, 62336 },
            { 57972, 62337 },
            { 57973, 62338 },
            { 57974, 62339 },
            { 57975, 62340 },
            { 57976, 62341 },
            { 57977, 62342 },
            { 57978, 62343 },
            { 57979, 62344 },
            { 57980, 62345 },
            { 57981, 62346 },
            { 57982, 62347 },
            { 57983, 62348 },
            { 57984, 62349 },
            { 57985, 62350 },
            { 57986, 62351 },
            { 57987, 62352 },
            { 57988, 62353 },
            { 57989, 62354 },
            { 57990, 62355 },
            { 57991, 62356 },
            { 57992, 62357 },
            { 57993, 62358 },
            { 57994, 62359 },
            { 57995, 62360 },
            { 57996, 62361 },
            { 57997, 62362 },
            { 57998, 62363 },
            { 57999, 62364 },
            { 58000, 62365 },
            { 58001, 62366 },
            { 58002, 62367 },
            { 58003, 62368 },
            { 58004, 62369 },
            { 58005, 62370 },
            { 58006, 62371 },
            { 58007, 62372 },
            { 58008, 62373 },
            { 58009, 62374 },
            { 58010, 62375 },
            { 58011, 62376 },
            { 58012, 62377 },
            { 58013, 62378 },
            { 58014, 62379 },
            { 58015, 62380 },
            { 58016, 62381 },
            { 58017, 62382 },
            { 58018, 62383 },
            { 58019, 62384 },
            { 58020, 62385 },
            { 58021, 62386 },
            { 58022, 62387 },
            { 58023, 62388 },
            { 58024, 62389 },
            { 58025, 62390 },
            { 58026, 62391 },
            { 58027, 62392 },
            { 58028, 62393 },
            { 58029, 62394 },
            { 58030, 62395 },
            { 58031, 62396 },
            { 58032, 62397 },
            { 58033, 62398 },
            { 58034, 62399 },
            { 58035, 62400 },
            { 58036, 62401 },
            { 58037, 62402 },
            { 58038, 62403 },
            { 58039, 62404 },
            { 58040, 62405 },
            { 58041, 62406 },
            { 58042, 62407 },
            { 58043, 62408 },
            { 58044, 62409 },
            { 58045, 62410 },
            { 58046, 62411 },
            { 58047, 62412 },
            { 58048, 62413 },
            { 58049, 62414 },
            { 58050, 62415 },
            { 58051, 62416 },
            { 58052, 62417 },
            { 58053, 62418 },
            { 58054, 62419 },
            { 58055, 62420 },
            { 58056, 62421 },
            { 58057, 62422 },
            { 58058, 62423 },
            { 58059, 62424 },
            { 58060, 62425 },
            { 58061, 62426 },
            { 58062, 62427 },
            { 58063, 62428 },
            { 58064, 62429 },
            { 58065, 62430 },
            { 58066, 62431 },
            { 58067, 62432 },
            { 58068, 62433 },
            { 58069, 62434 },
            { 58070, 62435 },
            { 58071, 62436 },
            { 58072, 62437 },
            { 58073, 62438 },
            { 58074, 62439 },
            { 58075, 62440 },
            { 58076, 62441 },
            { 58077, 62442 },
            { 58078, 62443 },
            { 58079, 62444 },
            { 58080, 62445 },
            { 58081, 62446 },
            { 58082, 62447 },
            { 58083, 62448 },
            { 58084, 62449 },
            { 58085, 62450 },
            { 58086, 62451 },
            { 58087, 62452 },
            { 58088, 62453 },
            { 58089, 62454 },
            { 58090, 62455 },
            { 58091, 62456 },
            { 58092, 62457 },
            { 58093, 62458 },
            { 58094, 62459 },
            { 58095, 62460 },
            { 58096, 62528 },
            { 58097, 62529 },
            { 58098, 62530 },
            { 58099, 62531 },
            { 58100, 62532 },
            { 58101, 62533 },
            { 58102, 62534 },
            { 58103, 62535 },
            { 58104, 62536 },
            { 58105, 62537 },
            { 58106, 62538 },
            { 58107, 62539 },
            { 58108, 62540 },
            { 58109, 62541 },
            { 58110, 62542 },
            { 58111, 62543 },
            { 58112, 62544 },
            { 58113, 62545 },
            { 58114, 62546 },
            { 58115, 62547 },
            { 58116, 62548 },
            { 58117, 62549 },
            { 58118, 62550 },
            { 58119, 62551 },
            { 58120, 62552 },
            { 58121, 62553 },
            { 58122, 62554 },
            { 58123, 62555 },
            { 58124, 62556 },
            { 58125, 62557 },
            { 58126, 62558 },
            { 58127, 62559 },
            { 58128, 62560 },
            { 58129, 62561 },
            { 58130, 62562 },
            { 58131, 62563 },
            { 58132, 62564 },
            { 58133, 62565 },
            { 58134, 62566 },
            { 58135, 62567 },
            { 58136, 62568 },
            { 58137, 62569 },
            { 58138, 62570 },
            { 58139, 62571 },
            { 58140, 62572 },
            { 58141, 62573 },
            { 58142, 62574 },
            { 58143, 62575 },
            { 58144, 62576 },
            { 58145, 62577 },
            { 58146, 62578 },
            { 58147, 62579 },
            { 58148, 62580 },
            { 58149, 62581 },
            { 58150, 62582 },
            { 58151, 62583 },
            { 58152, 62584 },
            { 58153, 62585 },
            { 58154, 62586 },
            { 58155, 62587 },
            { 58156, 62588 },
            { 58157, 62589 },
            { 58158, 62590 },
            { 58159, 62592 },
            { 58160, 62593 },
            { 58161, 62594 },
            { 58162, 62595 },
            { 58163, 62596 },
            { 58164, 62597 },
            { 58165, 62598 },
            { 58166, 62599 },
            { 58167, 62600 },
            { 58168, 62601 },
            { 58169, 62602 },
            { 58170, 62603 },
            { 58171, 62604 },
            { 58172, 62605 },
            { 58173, 62606 },
            { 58174, 62607 },
            { 58175, 62608 },
            { 58176, 62609 },
            { 58177, 62610 },
            { 58178, 62611 },
            { 58179, 62612 },
            { 58180, 62613 },
            { 58181, 62614 },
            { 58182, 62615 },
            { 58183, 62616 },
            { 58184, 62617 },
            { 58185, 62618 },
            { 58186, 62619 },
            { 58187, 62620 },
            { 58188, 62621 },
            { 58189, 62622 },
            { 58190, 62623 },
            { 58191, 62624 },
            { 58192, 62625 },
            { 58193, 62626 },
            { 58194, 62627 },
            { 58195, 62628 },
            { 58196, 62629 },
            { 58197, 62630 },
            { 58198, 62631 },
            { 58199, 62632 },
            { 58200, 62633 },
            { 58201, 62634 },
            { 58202, 62635 },
            { 58203, 62636 },
            { 58204, 62637 },
            { 58205, 62638 },
            { 58206, 62639 },
            { 58207, 62640 },
            { 58208, 62641 },
            { 58209, 62642 },
            { 58210, 62643 },
            { 58211, 62644 },
            { 58212, 62645 },
            { 58213, 62646 },
            { 58214, 62647 },
            { 58215, 62648 },
            { 58216, 62649 },
            { 58217, 62650 },
            { 58218, 62651 },
            { 58219, 62652 },
            { 58220, 62653 },
            { 58221, 62654 },
            { 58222, 62655 },
            { 58223, 62656 },
            { 58224, 62657 },
            { 58225, 62658 },
            { 58226, 62659 },
            { 58227, 62660 },
            { 58228, 62661 },
            { 58229, 62662 },
            { 58230, 62663 },
            { 58231, 62664 },
            { 58232, 62665 },
            { 58233, 62666 },
            { 58234, 62667 },
            { 58235, 62668 },
            { 58236, 62669 },
            { 58237, 62670 },
            { 58238, 62671 },
            { 58239, 62672 },
            { 58240, 62673 },
            { 58241, 62674 },
            { 58242, 62675 },
            { 58243, 62676 },
            { 58244, 62677 },
            { 58245, 62678 },
            { 58246, 62679 },
            { 58247, 62680 },
            { 58248, 62681 },
            { 58249, 62682 },
            { 58250, 62683 },
            { 58251, 62684 },
            { 58252, 62685 },
            { 58253, 62686 },
            { 58254, 62687 },
            { 58255, 62688 },
            { 58256, 62689 },
            { 58257, 62690 },
            { 58258, 62691 },
            { 58259, 62692 },
            { 58260, 62693 },
            { 58261, 62694 },
            { 58262, 62695 },
            { 58263, 62696 },
            { 58264, 62697 },
            { 58265, 62698 },
            { 58266, 62699 },
            { 58267, 62700 },
            { 58268, 62701 },
            { 58269, 62702 },
            { 58270, 62703 },
            { 58271, 62704 },
            { 58272, 62705 },
            { 58273, 62706 },
            { 58274, 62707 },
            { 58275, 62708 },
            { 58276, 62709 },
            { 58277, 62710 },
            { 58278, 62711 },
            { 58279, 62712 },
            { 58280, 62713 },
            { 58281, 62714 },
            { 58282, 62715 },
            { 58283, 62716 },
            { 58284, 62784 },
            { 58285, 62785 },
            { 58286, 62786 },
            { 58287, 62787 },
            { 58288, 62788 },
            { 58289, 62789 },
            { 58290, 62790 },
            { 58291, 62791 },
            { 58292, 62792 },
            { 58293, 62793 },
            { 58294, 62794 },
            { 58295, 62795 },
            { 58296, 62796 },
            { 58297, 62797 },
            { 58298, 62798 },
            { 58299, 62799 },
            { 58300, 62800 },
            { 58301, 62801 },
            { 58302, 62802 },
            { 58303, 62803 },
            { 58304, 62804 },
            { 58305, 62805 },
            { 58306, 62806 },
            { 58307, 62807 },
            { 58308, 62808 },
            { 58309, 62809 },
            { 58310, 62810 },
            { 58311, 62811 },
            { 58312, 62812 },
            { 58313, 62813 },
            { 58314, 62814 },
            { 58315, 62815 },
            { 58316, 62816 },
            { 58317, 62817 },
            { 58318, 62818 },
            { 58319, 62819 },
            { 58320, 62820 },
            { 58321, 62821 },
            { 58322, 62822 },
            { 58323, 62823 },
            { 58324, 62824 },
            { 58325, 62825 },
            { 58326, 62826 },
            { 58327, 62827 },
            { 58328, 62828 },
            { 58329, 62829 },
            { 58330, 62830 },
            { 58331, 62831 },
            { 58332, 62832 },
            { 58333, 62833 },
            { 58334, 62834 },
            { 58335, 62835 },
            { 58336, 62836 },
            { 58337, 62837 },
            { 58338, 62838 },
            { 58339, 62839 },
            { 58340, 62840 },
            { 58341, 62841 },
            { 58342, 62842 },
            { 58343, 62843 },
            { 58344, 62844 },
            { 58345, 62845 },
            { 58346, 62846 },
            { 58347, 62848 },
            { 58348, 62849 },
            { 58349, 62850 },
            { 58350, 62851 },
            { 58351, 62852 },
            { 58352, 62853 },
            { 58353, 62854 },
            { 58354, 62855 },
            { 58355, 62856 },
            { 58356, 62857 },
            { 58357, 62858 },
            { 58358, 62859 },
            { 58359, 62860 },
            { 58360, 62861 },
            { 58361, 62862 },
            { 58362, 62863 },
            { 58363, 62864 },
            { 58364, 62865 },
            { 58365, 62866 },
            { 58366, 62867 },
            { 58367, 62868 },
            { 58368, 62869 },
            { 58369, 62870 },
            { 58370, 62871 },
            { 58371, 62872 },
            { 58372, 62873 },
            { 58373, 62874 },
            { 58374, 62875 },
            { 58375, 62876 },
            { 58376, 62877 },
            { 58377, 62878 },
            { 58378, 62879 },
            { 58379, 62880 },
            { 58380, 62881 },
            { 58381, 62882 },
            { 58382, 62883 },
            { 58383, 62884 },
            { 58384, 62885 },
            { 58385, 62886 },
            { 58386, 62887 },
            { 58387, 62888 },
            { 58388, 62889 },
            { 58389, 62890 },
            { 58390, 62891 },
            { 58391, 62892 },
            { 58392, 62893 },
            { 58393, 62894 },
            { 58394, 62895 },
            { 58395, 62896 },
            { 58396, 62897 },
            { 58397, 62898 },
            { 58398, 62899 },
            { 58399, 62900 },
            { 58400, 62901 },
            { 58401, 62902 },
            { 58402, 62903 },
            { 58403, 62904 },
            { 58404, 62905 },
            { 58405, 62906 },
            { 58406, 62907 },
            { 58407, 62908 },
            { 58408, 62909 },
            { 58409, 62910 },
            { 58410, 62911 },
            { 58411, 62912 },
            { 58412, 62913 },
            { 58413, 62914 },
            { 58414, 62915 },
            { 58415, 62916 },
            { 58416, 62917 },
            { 58417, 62918 },
            { 58418, 62919 },
            { 58419, 62920 },
            { 58420, 62921 },
            { 58421, 62922 },
            { 58422, 62923 },
            { 58423, 62924 },
            { 58424, 62925 },
            { 58425, 62926 },
            { 58426, 62927 },
            { 58427, 62928 },
            { 58428, 62929 },
            { 58429, 62930 },
            { 58430, 62931 },
            { 58431, 62932 },
            { 58432, 62933 },
            { 58433, 62934 },
            { 58434, 62935 },
            { 58435, 62936 },
            { 58436, 62937 },
            { 58437, 62938 },
            { 58438, 62939 },
            { 58439, 62940 },
            { 58440, 62941 },
            { 58441, 62942 },
            { 58442, 62943 },
            { 58443, 62944 },
            { 58444, 62945 },
            { 58445, 62946 },
            { 58446, 62947 },
            { 58447, 62948 },
            { 58448, 62949 },
            { 58449, 62950 },
            { 58450, 62951 },
            { 58451, 62952 },
            { 58452, 62953 },
            { 58453, 62954 },
            { 58454, 62955 },
            { 58455, 62956 },
            { 58456, 62957 },
            { 58457, 62958 },
            { 58458, 62959 },
            { 58459, 62960 },
            { 58460, 62961 },
            { 58461, 62962 },
            { 58462, 62963 },
            { 58463, 62964 },
            { 58464, 62965 },
            { 58465, 62966 },
            { 58466, 62967 },
            { 58467, 62968 },
            { 58468, 62969 },
            { 58469, 62970 },
            { 58470, 62971 },
            { 58471, 62972 },
            { 58472, 63040 },
            { 58473, 63041 },
            { 58474, 63042 },
            { 58475, 63043 },
            { 58476, 63044 },
            { 58477, 63045 },
            { 58478, 63046 },
            { 58479, 63047 },
            { 58480, 63048 },
            { 58481, 63049 },
            { 58482, 63050 },
            { 58483, 63051 },
            { 58484, 63052 },
            { 58485, 63053 },
            { 58486, 63054 },
            { 58487, 63055 },
            { 58488, 63056 },
            { 58489, 63057 },
            { 58490, 63058 },
            { 58491, 63059 },
            { 58492, 63060 },
            { 58493, 63061 },
            { 58494, 63062 },
            { 58495, 63063 },
            { 58496, 63064 },
            { 58497, 63065 },
            { 58498, 63066 },
            { 58499, 63067 },
            { 58500, 63068 },
            { 58501, 63069 },
            { 58502, 63070 },
            { 58503, 63071 },
            { 58504, 63072 },
            { 58505, 63073 },
            { 58506, 63074 },
            { 58507, 63075 },
            { 58508, 63076 },
            { 58509, 63077 },
            { 58510, 63078 },
            { 58511, 63079 },
            { 58512, 63080 },
            { 58513, 63081 },
            { 58514, 63082 },
            { 58515, 63083 },
            { 58516, 63084 },
            { 58517, 63085 },
            { 58518, 63086 },
            { 58519, 63087 },
            { 58520, 63088 },
            { 58521, 63089 },
            { 58522, 63090 },
            { 58523, 63091 },
            { 58524, 63092 },
            { 58525, 63093 },
            { 58526, 63094 },
            { 58527, 63095 },
            { 58528, 63096 },
            { 58529, 63097 },
            { 58530, 63098 },
            { 58531, 63099 },
            { 58532, 63100 },
            { 58533, 63101 },
            { 58534, 63102 },
            { 58535, 63104 },
            { 58536, 63105 },
            { 58537, 63106 },
            { 58538, 63107 },
            { 58539, 63108 },
            { 58540, 63109 },
            { 58541, 63110 },
            { 58542, 63111 },
            { 58543, 63112 },
            { 58544, 63113 },
            { 58545, 63114 },
            { 58546, 63115 },
            { 58547, 63116 },
            { 58548, 63117 },
            { 58549, 63118 },
            { 58550, 63119 },
            { 58551, 63120 },
            { 58552, 63121 },
            { 58553, 63122 },
            { 58554, 63123 },
            { 58555, 63124 },
            { 58556, 63125 },
            { 58557, 63126 },
            { 58558, 63127 },
            { 58559, 63128 },
            { 58560, 63129 },
            { 58561, 63130 },
            { 58562, 63131 },
            { 58563, 63132 },
            { 58564, 63133 },
            { 58565, 63134 },
            { 58566, 63135 },
            { 58567, 63136 },
            { 58568, 63137 },
            { 58569, 63138 },
            { 58570, 63139 },
            { 58571, 63140 },
            { 58572, 63141 },
            { 58573, 63142 },
            { 58574, 63143 },
            { 58575, 63144 },
            { 58576, 63145 },
            { 58577, 63146 },
            { 58578, 63147 },
            { 58579, 63148 },
            { 58580, 63149 },
            { 58581, 63150 },
            { 58582, 63151 },
            { 58583, 63152 },
            { 58584, 63153 },
            { 58585, 63154 },
            { 58586, 63155 },
            { 58587, 63156 },
            { 58588, 63157 },
            { 58589, 63158 },
            { 58590, 63159 },
            { 58591, 63160 },
            { 58592, 63161 },
            { 58593, 63162 },
            { 58594, 63163 },
            { 58595, 63164 },
            { 58596, 63165 },
            { 58597, 63166 },
            { 58598, 63167 },
            { 58599, 63168 },
            { 58600, 63169 },
            { 58601, 63170 },
            { 58602, 63171 },
            { 58603, 63172 },
            { 58604, 63173 },
            { 58605, 63174 },
            { 58606, 63175 },
            { 58607, 63176 },
            { 58608, 63177 },
            { 58609, 63178 },
            { 58610, 63179 },
            { 58611, 63180 },
            { 58612, 63181 },
            { 58613, 63182 },
            { 58614, 63183 },
            { 58615, 63184 },
            { 58616, 63185 },
            { 58617, 63186 },
            { 58618, 63187 },
            { 58619, 63188 },
            { 58620, 63189 },
            { 58621, 63190 },
            { 58622, 63191 },
            { 58623, 63192 },
            { 58624, 63193 },
            { 58625, 63194 },
            { 58626, 63195 },
            { 58627, 63196 },
            { 58628, 63197 },
            { 58629, 63198 },
            { 58630, 63199 },
            { 58631, 63200 },
            { 58632, 63201 },
            { 58633, 63202 },
            { 58634, 63203 },
            { 58635, 63204 },
            { 58636, 63205 },
            { 58637, 63206 },
            { 58638, 63207 },
            { 58639, 63208 },
            { 58640, 63209 },
            { 58641, 63210 },
            { 58642, 63211 },
            { 58643, 63212 },
            { 58644, 63213 },
            { 58645, 63214 },
            { 58646, 63215 },
            { 58647, 63216 },
            { 58648, 63217 },
            { 58649, 63218 },
            { 58650, 63219 },
            { 58651, 63220 },
            { 58652, 63221 },
            { 58653, 63222 },
            { 58654, 63223 },
            { 58655, 63224 },
            { 58656, 63225 },
            { 58657, 63226 },
            { 58658, 63227 },
            { 58659, 63228 },
            { 58660, 63296 },
            { 58661, 63297 },
            { 58662, 63298 },
            { 58663, 63299 },
            { 58664, 63300 },
            { 58665, 63301 },
            { 58666, 63302 },
            { 58667, 63303 },
            { 58668, 63304 },
            { 58669, 63305 },
            { 58670, 63306 },
            { 58671, 63307 },
            { 58672, 63308 },
            { 58673, 63309 },
            { 58674, 63310 },
            { 58675, 63311 },
            { 58676, 63312 },
            { 58677, 63313 },
            { 58678, 63314 },
            { 58679, 63315 },
            { 58680, 63316 },
            { 58681, 63317 },
            { 58682, 63318 },
            { 58683, 63319 },
            { 58684, 63320 },
            { 58685, 63321 },
            { 58686, 63322 },
            { 58687, 63323 },
            { 58688, 63324 },
            { 58689, 63325 },
            { 58690, 63326 },
            { 58691, 63327 },
            { 58692, 63328 },
            { 58693, 63329 },
            { 58694, 63330 },
            { 58695, 63331 },
            { 58696, 63332 },
            { 58697, 63333 },
            { 58698, 63334 },
            { 58699, 63335 },
            { 58700, 63336 },
            { 58701, 63337 },
            { 58702, 63338 },
            { 58703, 63339 },
            { 58704, 63340 },
            { 58705, 63341 },
            { 58706, 63342 },
            { 58707, 63343 },
            { 58708, 63344 },
            { 58709, 63345 },
            { 58710, 63346 },
            { 58711, 63347 },
            { 58712, 63348 },
            { 58713, 63349 },
            { 58714, 63350 },
            { 58715, 63351 },
            { 58716, 63352 },
            { 58717, 63353 },
            { 58718, 63354 },
            { 58719, 63355 },
            { 58720, 63356 },
            { 58721, 63357 },
            { 58722, 63358 },
            { 58723, 63360 },
            { 58724, 63361 },
            { 58725, 63362 },
            { 58726, 63363 },
            { 58727, 63364 },
            { 58728, 63365 },
            { 58729, 63366 },
            { 58730, 63367 },
            { 58731, 63368 },
            { 58732, 63369 },
            { 58733, 63370 },
            { 58734, 63371 },
            { 58735, 63372 },
            { 58736, 63373 },
            { 58737, 63374 },
            { 58738, 63375 },
            { 58739, 63376 },
            { 58740, 63377 },
            { 58741, 63378 },
            { 58742, 63379 },
            { 58743, 63380 },
            { 58744, 63381 },
            { 58745, 63382 },
            { 58746, 63383 },
            { 58747, 63384 },
            { 58748, 63385 },
            { 58749, 63386 },
            { 58750, 63387 },
            { 58751, 63388 },
            { 58752, 63389 },
            { 58753, 63390 },
            { 58754, 63391 },
            { 58755, 63392 },
            { 58756, 63393 },
            { 58757, 63394 },
            { 58758, 63395 },
            { 58759, 63396 },
            { 58760, 63397 },
            { 58761, 63398 },
            { 58762, 63399 },
            { 58763, 63400 },
            { 58764, 63401 },
            { 58765, 63402 },
            { 58766, 63403 },
            { 58767, 63404 },
            { 58768, 63405 },
            { 58769, 63406 },
            { 58770, 63407 },
            { 58771, 63408 },
            { 58772, 63409 },
            { 58773, 63410 },
            { 58774, 63411 },
            { 58775, 63412 },
            { 58776, 63413 },
            { 58777, 63414 },
            { 58778, 63415 },
            { 58779, 63416 },
            { 58780, 63417 },
            { 58781, 63418 },
            { 58782, 63419 },
            { 58783, 63420 },
            { 58784, 63421 },
            { 58785, 63422 },
            { 58786, 63423 },
            { 58787, 63424 },
            { 58788, 63425 },
            { 58789, 63426 },
            { 58790, 63427 },
            { 58791, 63428 },
            { 58792, 63429 },
            { 58793, 63430 },
            { 58794, 63431 },
            { 58795, 63432 },
            { 58796, 63433 },
            { 58797, 63434 },
            { 58798, 63435 },
            { 58799, 63436 },
            { 58800, 63437 },
            { 58801, 63438 },
            { 58802, 63439 },
            { 58803, 63440 },
            { 58804, 63441 },
            { 58805, 63442 },
            { 58806, 63443 },
            { 58807, 63444 },
            { 58808, 63445 },
            { 58809, 63446 },
            { 58810, 63447 },
            { 58811, 63448 },
            { 58812, 63449 },
            { 58813, 63450 },
            { 58814, 63451 },
            { 58815, 63452 },
            { 58816, 63453 },
            { 58817, 63454 },
            { 58818, 63455 },
            { 58819, 63456 },
            { 58820, 63457 },
            { 58821, 63458 },
            { 58822, 63459 },
            { 58823, 63460 },
            { 58824, 63461 },
            { 58825, 63462 },
            { 58826, 63463 },
            { 58827, 63464 },
            { 58828, 63465 },
            { 58829, 63466 },
            { 58830, 63467 },
            { 58831, 63468 },
            { 58832, 63469 },
            { 58833, 63470 },
            { 58834, 63471 },
            { 58835, 63472 },
            { 58836, 63473 },
            { 58837, 63474 },
            { 58838, 63475 },
            { 58839, 63476 },
            { 58840, 63477 },
            { 58841, 63478 },
            { 58842, 63479 },
            { 58843, 63480 },
            { 58844, 63481 },
            { 58845, 63482 },
            { 58846, 63483 },
            { 58847, 63484 },
            { 58848, 63552 },
            { 58849, 63553 },
            { 58850, 63554 },
            { 58851, 63555 },
            { 58852, 63556 },
            { 58853, 63557 },
            { 58854, 63558 },
            { 58855, 63559 },
            { 58856, 63560 },
            { 58857, 63561 },
            { 58858, 63562 },
            { 58859, 63563 },
            { 58860, 63564 },
            { 58861, 63565 },
            { 58862, 63566 },
            { 58863, 63567 },
            { 58864, 63568 },
            { 58865, 63569 },
            { 58866, 63570 },
            { 58867, 63571 },
            { 58868, 63572 },
            { 58869, 63573 },
            { 58870, 63574 },
            { 58871, 63575 },
            { 58872, 63576 },
            { 58873, 63577 },
            { 58874, 63578 },
            { 58875, 63579 },
            { 58876, 63580 },
            { 58877, 63581 },
            { 58878, 63582 },
            { 58879, 63583 },
            { 58880, 63584 },
            { 58881, 63585 },
            { 58882, 63586 },
            { 58883, 63587 },
            { 58884, 63588 },
            { 58885, 63589 },
            { 58886, 63590 },
            { 58887, 63591 },
            { 58888, 63592 },
            { 58889, 63593 },
            { 58890, 63594 },
            { 58891, 63595 },
            { 58892, 63596 },
            { 58893, 63597 },
            { 58894, 63598 },
            { 58895, 63599 },
            { 58896, 63600 },
            { 58897, 63601 },
            { 58898, 63602 },
            { 58899, 63603 },
            { 58900, 63604 },
            { 58901, 63605 },
            { 58902, 63606 },
            { 58903, 63607 },
            { 58904, 63608 },
            { 58905, 63609 },
            { 58906, 63610 },
            { 58907, 63611 },
            { 58908, 63612 },
            { 58909, 63613 },
            { 58910, 63614 },
            { 58911, 63616 },
            { 58912, 63617 },
            { 58913, 63618 },
            { 58914, 63619 },
            { 58915, 63620 },
            { 58916, 63621 },
            { 58917, 63622 },
            { 58918, 63623 },
            { 58919, 63624 },
            { 58920, 63625 },
            { 58921, 63626 },
            { 58922, 63627 },
            { 58923, 63628 },
            { 58924, 63629 },
            { 58925, 63630 },
            { 58926, 63631 },
            { 58927, 63632 },
            { 58928, 63633 },
            { 58929, 63634 },
            { 58930, 63635 },
            { 58931, 63636 },
            { 58932, 63637 },
            { 58933, 63638 },
            { 58934, 63639 },
            { 58935, 63640 },
            { 58936, 63641 },
            { 58937, 63642 },
            { 58938, 63643 },
            { 58939, 63644 },
            { 58940, 63645 },
            { 58941, 63646 },
            { 58942, 63647 },
            { 58943, 63648 },
            { 58944, 63649 },
            { 58945, 63650 },
            { 58946, 63651 },
            { 58947, 63652 },
            { 58948, 63653 },
            { 58949, 63654 },
            { 58950, 63655 },
            { 58951, 63656 },
            { 58952, 63657 },
            { 58953, 63658 },
            { 58954, 63659 },
            { 58955, 63660 },
            { 58956, 63661 },
            { 58957, 63662 },
            { 58958, 63663 },
            { 58959, 63664 },
            { 58960, 63665 },
            { 58961, 63666 },
            { 58962, 63667 },
            { 58963, 63668 },
            { 58964, 63669 },
            { 58965, 63670 },
            { 58966, 63671 },
            { 58967, 63672 },
            { 58968, 63673 },
            { 58969, 63674 },
            { 58970, 63675 },
            { 58971, 63676 },
            { 58972, 63677 },
            { 58973, 63678 },
            { 58974, 63679 },
            { 58975, 63680 },
            { 58976, 63681 },
            { 58977, 63682 },
            { 58978, 63683 },
            { 58979, 63684 },
            { 58980, 63685 },
            { 58981, 63686 },
            { 58982, 63687 },
            { 58983, 63688 },
            { 58984, 63689 },
            { 58985, 63690 },
            { 58986, 63691 },
            { 58987, 63692 },
            { 58988, 63693 },
            { 58989, 63694 },
            { 58990, 63695 },
            { 58991, 63696 },
            { 58992, 63697 },
            { 58993, 63698 },
            { 58994, 63699 },
            { 58995, 63700 },
            { 58996, 63701 },
            { 58997, 63702 },
            { 58998, 63703 },
            { 58999, 63704 },
            { 59000, 63705 },
            { 59001, 63706 },
            { 59002, 63707 },
            { 59003, 63708 },
            { 59004, 63709 },
            { 59005, 63710 },
            { 59006, 63711 },
            { 59007, 63712 },
            { 59008, 63713 },
            { 59009, 63714 },
            { 59010, 63715 },
            { 59011, 63716 },
            { 59012, 63717 },
            { 59013, 63718 },
            { 59014, 63719 },
            { 59015, 63720 },
            { 59016, 63721 },
            { 59017, 63722 },
            { 59018, 63723 },
            { 59019, 63724 },
            { 59020, 63725 },
            { 59021, 63726 },
            { 59022, 63727 },
            { 59023, 63728 },
            { 59024, 63729 },
            { 59025, 63730 },
            { 59026, 63731 },
            { 59027, 63732 },
            { 59028, 63733 },
            { 59029, 63734 },
            { 59030, 63735 },
            { 59031, 63736 },
            { 59032, 63737 },
            { 59033, 63738 },
            { 59034, 63739 },
            { 59035, 63740 },
            { 59036, 63808 },
            { 59037, 63809 },
            { 59038, 63810 },
            { 59039, 63811 },
            { 59040, 63812 },
            { 59041, 63813 },
            { 59042, 63814 },
            { 59043, 63815 },
            { 59044, 63816 },
            { 59045, 63817 },
            { 59046, 63818 },
            { 59047, 63819 },
            { 59048, 63820 },
            { 59049, 63821 },
            { 59050, 63822 },
            { 59051, 63823 },
            { 59052, 63824 },
            { 59053, 63825 },
            { 59054, 63826 },
            { 59055, 63827 },
            { 59056, 63828 },
            { 59057, 63829 },
            { 59058, 63830 },
            { 59059, 63831 },
            { 59060, 63832 },
            { 59061, 63833 },
            { 59062, 63834 },
            { 59063, 63835 },
            { 59064, 63836 },
            { 59065, 63837 },
            { 59066, 63838 },
            { 59067, 63839 },
            { 59068, 63840 },
            { 59069, 63841 },
            { 59070, 63842 },
            { 59071, 63843 },
            { 59072, 63844 },
            { 59073, 63845 },
            { 59074, 63846 },
            { 59075, 63847 },
            { 59076, 63848 },
            { 59077, 63849 },
            { 59078, 63850 },
            { 59079, 63851 },
            { 59080, 63852 },
            { 59081, 63853 },
            { 59082, 63854 },
            { 59083, 63855 },
            { 59084, 63856 },
            { 59085, 63857 },
            { 59086, 63858 },
            { 59087, 63859 },
            { 59088, 63860 },
            { 59089, 63861 },
            { 59090, 63862 },
            { 59091, 63863 },
            { 59092, 63864 },
            { 59093, 63865 },
            { 59094, 63866 },
            { 59095, 63867 },
            { 59096, 63868 },
            { 59097, 63869 },
            { 59098, 63870 },
            { 59099, 63872 },
            { 59100, 63873 },
            { 59101, 63874 },
            { 59102, 63875 },
            { 59103, 63876 },
            { 59104, 63877 },
            { 59105, 63878 },
            { 59106, 63879 },
            { 59107, 63880 },
            { 59108, 63881 },
            { 59109, 63882 },
            { 59110, 63883 },
            { 59111, 63884 },
            { 59112, 63885 },
            { 59113, 63886 },
            { 59114, 63887 },
            { 59115, 63888 },
            { 59116, 63889 },
            { 59117, 63890 },
            { 59118, 63891 },
            { 59119, 63892 },
            { 59120, 63893 },
            { 59121, 63894 },
            { 59122, 63895 },
            { 59123, 63896 },
            { 59124, 63897 },
            { 59125, 63898 },
            { 59126, 63899 },
            { 59127, 63900 },
            { 59128, 63901 },
            { 59129, 63902 },
            { 59130, 63903 },
            { 59131, 63904 },
            { 59132, 63905 },
            { 59133, 63906 },
            { 59134, 63907 },
            { 59135, 63908 },
            { 59136, 63909 },
            { 59137, 63910 },
            { 59138, 63911 },
            { 59139, 63912 },
            { 59140, 63913 },
            { 59141, 63914 },
            { 59142, 63915 },
            { 59143, 63916 },
            { 59144, 63917 },
            { 59145, 63918 },
            { 59146, 63919 },
            { 59147, 63920 },
            { 59148, 63921 },
            { 59149, 63922 },
            { 59150, 63923 },
            { 59151, 63924 },
            { 59152, 63925 },
            { 59153, 63926 },
            { 59154, 63927 },
            { 59155, 63928 },
            { 59156, 63929 },
            { 59157, 63930 },
            { 59158, 63931 },
            { 59159, 63932 },
            { 59160, 63933 },
            { 59161, 63934 },
            { 59162, 63935 },
            { 59163, 63936 },
            { 59164, 63937 },
            { 59165, 63938 },
            { 59166, 63939 },
            { 59167, 63940 },
            { 59168, 63941 },
            { 59169, 63942 },
            { 59170, 63943 },
            { 59171, 63944 },
            { 59172, 63945 },
            { 59173, 63946 },
            { 59174, 63947 },
            { 59175, 63948 },
            { 59176, 63949 },
            { 59177, 63950 },
            { 59178, 63951 },
            { 59179, 63952 },
            { 59180, 63953 },
            { 59181, 63954 },
            { 59182, 63955 },
            { 59183, 63956 },
            { 59184, 63957 },
            { 59185, 63958 },
            { 59186, 63959 },
            { 59187, 63960 },
            { 59188, 63961 },
            { 59189, 63962 },
            { 59190, 63963 },
            { 59191, 63964 },
            { 59192, 63965 },
            { 59193, 63966 },
            { 59194, 63967 },
            { 59195, 63968 },
            { 59196, 63969 },
            { 59197, 63970 },
            { 59198, 63971 },
            { 59199, 63972 },
            { 59200, 63973 },
            { 59201, 63974 },
            { 59202, 63975 },
            { 59203, 63976 },
            { 59204, 63977 },
            { 59205, 63978 },
            { 59206, 63979 },
            { 59207, 63980 },
            { 59208, 63981 },
            { 59209, 63982 },
            { 59210, 63983 },
            { 59211, 63984 },
            { 59212, 63985 },
            { 59213, 63986 },
            { 59214, 63987 },
            { 59215, 63988 },
            { 59216, 63989 },
            { 59217, 63990 },
            { 59218, 63991 },
            { 59219, 63992 },
            { 59220, 63993 },
            { 59221, 63994 },
            { 59222, 63995 },
            { 59223, 63996 },
            { 63728, 160 },
            { 63729, 253 },
            { 63730, 254 },
            { 63731, 255 },
            { 63785, 64224 },
            { 63964, 64489 },
            { 64014, 64144 },
            { 64015, 64155 },
            { 64016, 64156 },
            { 64017, 64177 },
            { 64018, 64216 },
            { 64019, 64232 },
            { 64020, 64234 },
            { 64021, 64344 },
            { 64022, 64350 },
            { 64023, 64373 },
            { 64024, 64381 },
            { 64025, 64382 },
            { 64026, 64384 },
            { 64027, 64386 },
            { 64028, 64390 },
            { 64029, 64393 },
            { 64030, 64402 },
            { 64031, 64413 },
            { 64032, 64415 },
            { 64033, 64416 },
            { 64034, 64425 },
            { 64035, 64433 },
            { 64036, 64435 },
            { 64037, 64436 },
            { 64038, 64439 },
            { 64039, 64467 },
            { 64040, 64474 },
            { 64041, 64490 },
            { 64042, 64502 },
            { 64043, 64503 },
            { 64044, 64505 },
            { 64045, 64585 },
            { 65281, 33097 },
            { 65282, 64087 },
            { 65283, 33172 },
            { 65284, 33168 },
            { 65285, 33171 },
            { 65286, 33173 },
            { 65287, 64086 },
            { 65288, 33129 },
            { 65289, 33130 },
            { 65290, 33174 },
            { 65291, 33147 },
            { 65292, 33091 },
            { 65293, 33148 },
            { 65294, 33092 },
            { 65295, 33118 },
            { 65296, 33359 },
            { 65297, 33360 },
            { 65298, 33361 },
            { 65299, 33362 },
            { 65300, 33363 },
            { 65301, 33364 },
            { 65302, 33365 },
            { 65303, 33366 },
            { 65304, 33367 },
            { 65305, 33368 },
            { 65306, 33094 },
            { 65307, 33095 },
            { 65308, 33155 },
            { 65309, 33153 },
            { 65310, 33156 },
            { 65311, 33096 },
            { 65312, 33175 },
            { 65313, 33376 },
            { 65314, 33377 },
            { 65315, 33378 },
            { 65316, 33379 },
            { 65317, 33380 },
            { 65318, 33381 },
            { 65319, 33382 },
            { 65320, 33383 },
            { 65321, 33384 },
            { 65322, 33385 },
            { 65323, 33386 },
            { 65324, 33387 },
            { 65325, 33388 },
            { 65326, 33389 },
            { 65327, 33390 },
            { 65328, 33391 },
            { 65329, 33392 },
            { 65330, 33393 },
            { 65331, 33394 },
            { 65332, 33395 },
            { 65333, 33396 },
            { 65334, 33397 },
            { 65335, 33398 },
            { 65336, 33399 },
            { 65337, 33400 },
            { 65338, 33401 },
            { 65339, 33133 },
            { 65340, 33119 },
            { 65341, 33134 },
            { 65342, 33103 },
            { 65343, 33105 },
            { 65344, 33101 },
            { 65345, 33409 },
            { 65346, 33410 },
            { 65347, 33411 },
            { 65348, 33412 },
            { 65349, 33413 },
            { 65350, 33414 },
            { 65351, 33415 },
            { 65352, 33416 },
            { 65353, 33417 },
            { 65354, 33418 },
            { 65355, 33419 },
            { 65356, 33420 },
            { 65357, 33421 },
            { 65358, 33422 },
            { 65359, 33423 },
            { 65360, 33424 },
            { 65361, 33425 },
            { 65362, 33426 },
            { 65363, 33427 },
            { 65364, 33428 },
            { 65365, 33429 },
            { 65366, 33430 },
            { 65367, 33431 },
            { 65368, 33432 },
            { 65369, 33433 },
            { 65370, 33434 },
            { 65371, 33135 },
            { 65372, 33122 },
            { 65373, 33136 },
            { 65374, 33120 },
            { 65377, 161 },
            { 65378, 162 },
            { 65379, 163 },
            { 65380, 164 },
            { 65381, 165 },
            { 65382, 166 },
            { 65383, 167 },
            { 65384, 168 },
            { 65385, 169 },
            { 65386, 170 },
            { 65387, 171 },
            { 65388, 172 },
            { 65389, 173 },
            { 65390, 174 },
            { 65391, 175 },
            { 65392, 176 },
            { 65393, 177 },
            { 65394, 178 },
            { 65395, 179 },
            { 65396, 180 },
            { 65397, 181 },
            { 65398, 182 },
            { 65399, 183 },
            { 65400, 184 },
            { 65401, 185 },
            { 65402, 186 },
            { 65403, 187 },
            { 65404, 188 },
            { 65405, 189 },
            { 65406, 190 },
            { 65407, 191 },
            { 65408, 192 },
            { 65409, 193 },
            { 65410, 194 },
            { 65411, 195 },
            { 65412, 196 },
            { 65413, 197 },
            { 65414, 198 },
            { 65415, 199 },
            { 65416, 200 },
            { 65417, 201 },
            { 65418, 202 },
            { 65419, 203 },
            { 65420, 204 },
            { 65421, 205 },
            { 65422, 206 },
            { 65423, 207 },
            { 65424, 208 },
            { 65425, 209 },
            { 65426, 210 },
            { 65427, 211 },
            { 65428, 212 },
            { 65429, 213 },
            { 65430, 214 },
            { 65431, 215 },
            { 65432, 216 },
            { 65433, 217 },
            { 65434, 218 },
            { 65435, 219 },
            { 65436, 220 },
            { 65437, 221 },
            { 65438, 222 },
            { 65439, 223 },
            { 65504, 33169 },
            { 65505, 33170 },
            { 65506, 33226 },
            { 65507, 33104 },
            { 65508, 64085 },
            { 65509, 33167 }
        };
#if __cplusplus
            for( int i = 0; i < _LEN_DICT; i++ ){
                _DICT_SRC[i][0] = n[i][0];
                _DICT_SRC[i][1] = n[i][1];
            }
        }
#endif

#if __cplusplus
        static bool m_initialized;
        static std::map<int, int> _DICT;
#else
        static bool m_initialized = false;
        static bool m_cp932_available = false;
        static Encoding m_cp932 = null;
#endif

        static void init() {
#if __cplusplus
            init_cor();
            for( int i = 0; i < _LEN_DICT; i++ ){
                _DICT.insert( map<int, int>::value_type( _DICT_SRC[i][0], _DICT_SRC[i][1] ) );
            }
#else
            try {
                m_cp932 = Encoding.GetEncoding( 932 );
                m_cp932_available = true;
            } catch {
                m_cp932_available = false;
            }
#endif
#if DEBUG
            m_cp932_available = false;
#endif
            m_initialized = true;
        }

#if __cplusplus
    public:
        static vector<byte> convert( std::string str ){
#else
        public static byte[] convert( string str ) {
#endif
#if !__cplusplus
            if ( !m_initialized ) {
                init();
            }
            if ( m_cp932_available ) {
                return m_cp932.GetBytes( str );
            } else {
#endif
#if __cplusplus
                int len = str.size();
                wchar_t *arr = new wchar_t[len];
                vector<byte> list( 0 );
                for( int i = 0; i < len; i++ ){
#else
                char[] arr = str.ToCharArray();
                List<byte> list = new List<byte>();
                for ( int i = 0; i < arr.Length; i++ ) {
#endif
#if DEBUG
                    System.Diagnostics.Debug.WriteLine( "arr[i]=" + arr[i] );
#endif
#if __cplusplus
                    map<int, int>::const_iterator finder = _DICT.find( (int)arr[i] );
                    if( finder != _DICT.end() ){
#else
                    if ( _DICT.ContainsKey( (int)arr[i] ) ) {
#endif
                        int t = _DICT[(int)arr[i]];
                        if ( t > 0xff ) {
                            byte b1 = (byte)(t >> 8);
                            byte b2 = (byte)(t - (b1 << 8));
#if __cplusplus
                            list.push_back( b1 );
                            list.push_back( b2 );
#else
                            list.Add( b1 );
                            list.Add( b2 );
#endif
                        } else {
#if __cplusplus
                            list.push_back( (byte)t );
#else
                            list.Add( (byte)t );
#endif
                        }
                    } else {
#if __cplusplus
                        list.push_back( 0x63 );
#else
                        list.Add( 0x63 );
#endif
                    }
                }
#if __cplusplus
                return list;
#else
                return list.ToArray();
            }
#endif
        }


#if __cplusplus
    public:
        static string convert( vector<byte> dat ){
#else
        public static string convert( byte[] dat ) {
#endif
            if ( !m_initialized ) {
                init();
            }
#if !__cplusplus
            if ( m_cp932_available ) {
                char[] res = m_cp932.GetChars( dat );
                return new string( res );
            } else {
#endif
#if __cplusplus
                string sb( "" );
#else
                StringBuilder sb = new StringBuilder();
#endif
                int i = 0;
#if __cplusplus
                while( i < dat.size() ){
#else
                while ( i < dat.Length ) {
#endif
                    int b1 = dat[i];
                    bool found = false;
#if __cplusplus
                    for( int j = 0; j < _LEN_DICT; j++ ){
                        int key = _DICT_SRC[j][0];
                        int test = _DICT_SRC[j][1];
#else
                    foreach ( int key in _DICT.Keys ) {
                        int test = _DICT[key];
#endif
                        if ( b1 == test ) {
                            found = true;
#if __cplusplus
                            sb = sb + (char)key;
#else
                            sb.Append( ((char)key).ToString() );
#endif
                            break;
                        }
                    }
                    i++;
#if __cplusplus
                    if( !found && i < dat.size() ){
#else
                    if ( !found && i < dat.Length ) {
#endif
                        int b2 = (dat[i - 1] << 8) + dat[i];
#if __cplusplus
                        for( int j = 0; j < _LEN_DICT; j++ ){
                            int key = _DICT_SRC[j][0];
                            int test = _DICT_SRC[j][1];
#else
                        foreach ( int key in _DICT.Keys ) {
                            int test = _DICT[key];
#endif
                            if ( test == b2 ) {
#if __cplusplus
                                sb = sb + (char)key;
#else
                                sb.Append( ((char)key).ToString() );
#endif
                                break;
                            }
                        }
                        i++;
                    }
                }
#if __cplusplus
                return sb;
#else
                return sb.ToString();
            }
#endif
        }
    };

}
